Package: ambre
Title: Multi-Barrier Approach for Water Reuse Risk Assessment
Version: 2.1.2
Authors@R: c(
    person("Flora", "Girard", , "flora.girard@inrae.fr", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0537-4530")),
    person("Nicolas", "Forquet", , "nicolas.forquet@inrae.fr", role = "ctb",
           comment = c(ORCID = "0000-0003-1154-5498")),
    person("Arthur", "Bréant", , "arthur@thinkr.fr", role = "ctb"),
    person("Colin", "Fay", , "colin@thinkr.fr", role = "ctb",
           comment = c(ORCID = "0000-0001-7343-1846"))
  )
Description: Provides a Quantitative Microbial Risk Assessment (QMRA)
    framework for the reuse of treated wastewater in agricultural
    irrigation. Following a multi-barrier approach, the package simulates
    pathogen inflow and removal along a treatment train, estimates human
    exposure, and computes health risk indicators such as infection
    probability, illness probability, and Disability-Adjusted Life Years
    (DALYs). It also supports an economic analysis of the treatment
    scenarios considered. For more details see <doi:10.36904/20240595>.
License: MIT + file LICENSE
URL: https://forge.inrae.fr/reversaal/reut/ambre-package
BugReports: https://forge.inrae.fr/reversaal/reut/ambre-package/-/issues
Depends: R (>= 4.1.0)
Imports: colorspace, cowplot, dplyr, EnvStats, formattable, ggplot2,
        plyr, purrr, readxl, rlang, scales, sfsmisc, stats, stringr,
        tibble, tidyr, utils
Suggests: knitr, rmarkdown, spelling, testthat, writexl
VignetteBuilder: knitr
Config/fusen/version: 0.7.2
Config/roxygen2/version: 8.0.0
Encoding: UTF-8
Language: en-US
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-08-24 06:34:38 UTC; flgirard
Author: Flora Girard [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0537-4530>),
  Nicolas Forquet [ctb] (ORCID: <https://orcid.org/0000-0003-1154-5498>),
  Arthur Bréant [ctb],
  Colin Fay [ctb] (ORCID: <https://orcid.org/0000-0001-7343-1846>)
Maintainer: Flora Girard <flora.girard@inrae.fr>
Repository: CRAN
Date/Publication: 2026-09-04 19:50:02 UTC

# Generated by roxygen2: do not edit by hand

export(annual_cost)
export(collective_treatment_cost_calculation)
export(create_random_distribution)
export(create_scenario)
export(create_scenario_from_df)
export(dalys_calculation)
export(distribution_repeater)
export(dr.betapoisson)
export(dr.expo)
export(enumeration)
export(final_dose_calculation)
export(generate_random_values)
export(get_exposure_value_or_stop)
export(get_infection_prob)
export(get_risk_total)
export(get_scheme_events_wide)
export(get_tbl_risk)
export(get_treatment_data)
export(illness_probability_calculation)
export(infection_probability_calculation)
export(inflow_concentration)
export(inflow_concentration_custom)
export(initial_dose_calculation)
export(irrigation_need_calculation)
export(percentage_irrigation_need)
export(perimeter_calculation)
export(plot_comparison_qmra_initial_vs_supplementary_processes)
export(plot_dalys)
export(plot_infection_probability)
export(plot_inflow)
export(plot_logreduction)
export(plot_volumes)
export(process_cost_calculation)
export(query_barrier)
export(query_crop)
export(query_crop_height)
export(query_exp_path)
export(query_frequency)
export(query_irrigation_need)
export(query_matrix)
export(query_pathogen)
export(query_pop)
export(query_volume)
export(regulation_matrix_concentration)
export(regulation_matrix_log)
export(retrieve_specific_barrier)
export(run_economic_analysis)
export(run_qmra_custom)
export(run_qmra_initial_situation)
export(run_qmra_learning_case)
export(run_qmra_supplementary_process)
export(simulate_exposure)
export(simulate_inflow)
export(simulate_treatment)
export(total_area_calculation)
export(total_investment_cost)
export(total_irrigation_need)
export(update_concentration)
export(update_frequency)
export(update_frequency_with_desired_value)
export(update_logreduction_decay)
export(update_logreduction_path)
export(update_logreduction_specific_barrier)
export(update_monte_carlo)
export(update_pathogen)
export(update_treatment_scheme)
export(update_volume)
export(update_volume_with_desired_value)
export(water_price)
import(colorspace)
import(cowplot)
import(dplyr)
import(formattable)
import(ggplot2)
importFrom(EnvStats,rtri)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pick)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(formattable,formattable)
importFrom(plyr,rbind.fill)
importFrom(purrr,imap)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,pmap)
importFrom(readxl,read_excel)
importFrom(rlang,.data)
importFrom(sfsmisc,lseq)
importFrom(stats,qnorm)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stringr,str_wrap)
importFrom(tibble,tibble)
importFrom(tidyr,gather)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unnest)
importFrom(utils,globalVariables)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ambre <img src="man/figures/logo.svg" align="right" height="139" alt="ambre logo" />

<!-- badges: start -->

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![License:
MIT](https://img.shields.io/badge/license-MIT-green.svg)](https://opensource.org/licenses/MIT)
![Coverage
report](https://forge.inrae.fr/reversaal/reut/ambre-package/badges/master/coverage.svg?job=coverage)

<!-- badges: end -->

> Is it safe to reuse this water? **ambre** puts a number on it.

**ambre** (*Approche Multi-Barrières pour la Réutilisation de l’Eau* /
*Multi-Barrier Approach for Water Reuse*) is an open-source R package
for **Quantitative Microbial Risk Assessment (QMRA)** applied to the
reuse of treated wastewater for agricultural irrigation.

## Why ambre?

Reusing treated wastewater to irrigate crops helps face water scarcity,
but only if it is safe. Before reusing water, we need to perform risk
analysis to define what could be done and how. In real world, risk
analysis rely mainly on qualitative assessment, but in labs we use
quantitative ones (QMRA), much more precise and helpful for alternative
scenarios comparison. But most QMRA tools are complex, consider only one
target population at a time, and account for water treatment alone.
`ambre` was built to go further:

- 🔓 **Open source.** A transparent, reproducible QMRA workflow, from
  pathogen inflow all the way to DALYs (the healthy years of life lost).
- 🎲 **Monte-Carlo, multi-run.** Thousands of iterations yield realistic
  risk *ranges* instead of a single, fragile number.
- 🧱 **Barrier database.** Models the full chain of barriers: treatment
  plants **and** on-field agricultural practices (from Alice-Rose
  Thomas’s PhD), where earlier tools only carried treatment data. See
  [The barrier database](#the-barrier-database).
- 💶 👥 **Beyond health.** Weighs the **economic** cost and the **social
  organisation** to support the reuse scenario, not just the health
  risk.

<figure>
<img src="man/figures/ambre_schema_vulgarisation.png"
alt="How ambre works, from used water to a reuse decision." />
<figcaption aria-hidden="true">How ambre works, from used water to a
reuse decision.</figcaption>
</figure>

## Installation

`ambre` is not on CRAN yet. Install the development version from the
INRAE forge:

``` r
# install.packages("remotes")
remotes::install_git("https://forge.inrae.fr/reversaal/reut/ambre-package.git")
```

## A typical use case

A reuse scenario (crops, exposed populations, treatment scheme…) is
described in an Excel file. The package ships ready-to-run examples; use
them as templates for your own scenarios.

### 1. Health risk (QMRA) accounting for the treatment barriers

The input file should contain the following values:

| CropName | Area | PopulationName | nb_population | PathName | STEPtreatmentName | CollectiveTreatmentName | InitialProcessName | SupplementaryProcessName | nb_day_decay |
|:---|---:|:---|---:|:---|:---|:---|:---|:---|---:|
| Tomato | 10 | Maintenance staff | 1 | Ingestion following direct contact with a part of the plant during crop maintenance, followed by hand-to-mouth contact | Q.1 - Activated Sludge | Q.2 - Maturation Pond | Q.6 - Chlorination | E.1.1 - Micro-sprinkler | NA |
| Corn seed | 35 | Maintenance staff | 1 | Ingestion following direct contact with a part of the plant during crop maintenance, followed by hand-to-mouth contact | Q.1 - Activated Sludge | Q.2 - Maturation Pond | Q.6 - Chlorination | P.5 - Natural die-off | 3 |

``` r
library(ambre)


plots <- run_qmra_treatment(
  scenario_path = system.file("input_1culture_2pop.xlsx", package = "ambre"),
  pathogen      = c("Campylobacter jejuni", "Norovirus")
)
plots$dalys
```

#### health burden (DALYs)

<img src="man/figures/README-example3-1.png" width="100%" />

#### Probability of infection

``` r
plots$infection_probability
```

<img src="man/figures/README-example4-1.png" width="100%" />

### 2. Same scenario, now crediting the on-field barriers (practices)

``` r
scenario <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
regulation_concentration <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Country, RegulationID, Reduction))
run_qmra_supplementary_process(scenario = scenario, 
                               pathogen = c("Campylobacter jejuni"),
                               regulationConcentration = regulation_concentration )

# 3. Economic analysis of the scenario
run_economic_analysis(scenario = scenario,
                      price_per_m3 = 0.1, 
                      membership_fee = 200, 
                      grant = 0.5)
```

`run_qmra_initial_situation()` and `run_qmra_supplementary_process()`
each return a named list of `ggplot` objects (`concentration`, `volume`,
`logreduction`, `infection_probability`, `dalys`, `regulation matrix`)
so you can inspect every step of the assessment.

## The barrier database

The engine behind `ambre` is an original **multi-barrier database** that
puts a number on *every* measure a reuse scheme can place between the
wastewater and the people exposed to it — not just the treatment plant.
It operationalises the database built in **Alice-Rose Thomas’s PhD**
([INSA Lyon, 2024](https://theses.hal.science/tel-05009670); condensed
in the thesis’ *Tableau 13*) and used in her *Multiware* participatory
simulation tool ([Thomas *et al.*, 2024,
*TSM*](https://doi.org/10.36904/20240595)).

### Why it matters

International guidance no longer frames “safe reuse” as a treatment-only
problem:

- the **WHO** promotes a **multi-barrier approach** and a tolerable
  health target of **≤ 10⁻⁶ DALY per person per year** ([WHO,
  2006](https://www.who.int/publications/i/item/9241546832); [WHO QMRA,
  2016](https://www.who.int/publications/i/item/9789241565370));
- EU **Regulation (EU) 2020/741** (applicable since June 2023) sets four
  reclaimed-water classes (A–D, `E. coli` ≤ 10 / 100 / 1 000 / 10 000
  per 100 mL) and explicitly lets a scheme meet them through
  **additional barriers** rather than ever-heavier treatment
  ([EUR-Lex](https://eur-lex.europa.eu/eli/reg/2020/741/oj/eng));
- France transposed it with the **arrêté du 18 décembre 2023** (which
  repealed the 2010 arrêté and tightened the `E. coli` thresholds ~100×)
  ([Légifrance](https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000048679665));
- **ISO 16075-2** tabulates barrier log-reductions and “barrier
  equivalents” ([ISO](https://www.iso.org/standard/73483.html)).

`ambre`’s database turns that qualitative guidance into computable
numbers, adding two things the normative texts leave out:
**per-pathogen-type** reductions (rather than a single `E. coli`
indicator) and **economic cost**.

### What’s in it

**49 barriers**, grouped into the four families of the thesis’
classification:

| Family | What it is | Examples |
|----|----|----|
| 🚰 **Quality** (17) | treatment processes that lower pathogen *concentration* | activated sludge, membrane bioreactor, maturation lagoon, sand / micro- / ultra-filtration combined with UV, chlorination or electro-oxidation |
| 🔧 **Equipment** (14) | physical means deployed on the scheme | micro-sprinkling, surface & subsurface drip, storage, partition cover, signage, access control, automatic irrigation, mechanised maintenance & harvest, PPE |
| 🌱 **Cultivation & irrigation practices** (12) | how the crop is grown, watered and prepared | 70 m buffer distance, natural die-off before harvest, night-time irrigation, stop-in-strong-wind, non-food crop choice, post-harvest cooking / drying / peeling / washing / disinfection |
| 🛠️ **System operation** (6) | keeps the chain working over time | training, awareness-raising, health monitoring, microbiological monitoring, equipment maintenance |

Each barrier is described by **how** it cuts risk and **for whom**:

- the **concentration** it removes (log₁₀, per pathogen *type* —
  bacteria, viruses, protozoa);
- the **exposure** it prevents or the **volume** it reduces, resolved by
  **exposure route** (water, air, plant, soil, product) and by **exposed
  population** (irrigation, maintenance and harvesting staff; local
  residents; passers-by; consumers);
- credits that depend on **where the edible part sits** relative to the
  emitter (in-soil / at surface / ≥ 25 cm / ≥ 50 cm above);
- and an **economic cost** (capex/opex, per m³, per ha or per person).

Crucially, every entry is stored as a **probability distribution**
(uniform, normal, …) rather than a point estimate — this is what powers
`ambre`’s Monte-Carlo, multi-run engine and lets it report a risk
*range*. A few illustrations drawn from the data:

- *Activated sludge* → 1–2 log (bacteria/viruses), 0.5–1 log (protozoa);
  *ultrafiltration + UV + chlorination* → up to ~16 / 12 / 9.5 log.
- *Subsurface drip* → ~0 log when the edible part is in the soil, but ~6
  log once it sits ≥ 25 cm above the buried emitter.
- *Waiting for natural decay* → up to ~1–2 log credited to field die-off
  between the last irrigation and harvest.
- Costs, e.g. maturation lagoon ≈ 3.5 €/m³ (capex) + 0.07 €/m³/yr
  (opex); subsurface drip ≈ 7 933 €/ha + 68 €/ha/yr; perimeter sign ≈ 90
  €/ha; gloves ≈ 70 €/person.

### Where it lives

The database ships as five CSVs in `data-raw/`, assembled into the
bundled `config_ambre` dataset:

| File (`data-raw/`) | `config_ambre` slot | Content |
|----|----|----|
| `ambre_barriere_general.csv` | `treatment$processes` | log₁₀ concentration reduction, per barrier × pathogen type |
| `ambre_barriere_voie.csv` | `treatment$barriere_path` | exposure / volume reduction, per route × population |
| `ambre_barriere_specifique.csv` | `treatment$barriere_specific` | crop-height-dependent credits (irrigation systems) |
| `ambre_barriere_deperissement.csv` | `treatment$barriere_decay` | natural die-off, per crop height × population × pathogen |
| `ambre_barriere_cout.csv` | `economic$cost` | capex / opex, with units |

`run_qmra_initial_situation()` credits only the *curent situation*,
while `run_qmra_supplementary_process()` evaluates the additional
measures relating to *Quality*, *Equipment*, *Practices* and *System
operation*. That difference is the whole point of the multi-barrier
approach: reaching the WHO health target by **combining** measures
across the chain instead of over-treating the water.

## Documentation

- **User documentation.** The [{ambre}
  website](https://ambre-package-8e1a37.pages-forge.inrae.fr) gathers
  the *Get started* guide and articles built from the package vignettes.
- **Technical documentation.** The [function
  reference](https://ambre-package-8e1a37.pages-forge.inrae.fr/reference/)
  and the source code on the [INRAE
  forge](https://forge.inrae.fr/reversaal/reut/ambre-package). `ambre`
  is developed with [{fusen}](https://thinkr-open.github.io/fusen/): the
  source of truth lives in the `dev/flat_*.Rmd` files.

## Acknowledgements

`ambre` is developed at INRAE
([REVERSAAL](https://annuaire.inrae.fr/structure/1468)). Its barrier
database builds on the PhD work of **Alice-Rose Thomas** (*Application
de l’approche Multi-barrières pour la réutilisation des eaux usées
traitées*, [INSA Lyon, 2024](https://theses.hal.science/tel-05009670))
and the associated [*Multiware*](https://doi.org/10.36904/20240595)
tool. Released under the MIT license.

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Get table risk
#' 
#' Get table risk to calculate infection probability
#' @param scenario data.frame with columns inflow concentration, final dose and volume
#' @return tbl_risk
#' 
#' @export
#' @examples
#'
#' library(dplyr)
#' library(purrr)
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#'
#' scenario_conc <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni", "Escherichia coli"))
#'
#' scenario_volume <-
#'   scenario_conc |>                     
#'   mutate(
#'     volume = map(
#'       .x = config,                                         
#'       .f = ~ simulate_exposure(config = .x)              
#'     )
#'   )
#'
#' scenario_dose_ini <- initial_dose_calculation(scenario_volume)
#'
#' scenario_scheme <- update_treatment_scheme(scenario_dose_ini)
#'
#' scenario_with_logreduc_and_co <- scenario_scheme |> 
#'   mutate( log_reduction = map(config, simulate_treatment))
#'
#' scenario_final_dose_and_co_test <- final_dose_calculation(scenario_with_logreduc_and_co)
#' tbl_risk_test <- get_tbl_risk(scenario_final_dose_and_co_test)
get_tbl_risk <- function(scenario){
  
  if (!"inflow_concentration" %in% names(scenario)) {
  stop("The inflow_concentration column is missing from the scenario object; run the inflow_concentration function to create it")
  }
  
  if (!"volume" %in% names(scenario)) {
  stop("The volume column is missing from the scenario object; run the simulate_exposure function to create it")
}
  
  if (!"final_dose" %in% names(scenario)) {
  stop("The final_dose column is missing from the scenario object; run the final_dose_calculation function to create it")
  }
 
  tbl_risk <- map(scenario$final_dose, ~.x)
  tbl_risk <- map(tbl_risk, ~rename(.x, dose_perEvent = final_dose))
  tbl_risk <- map(tbl_risk, ~rename(.x, logreduction = total_logreduction))
  tbl_risk <- map(tbl_risk, ~{
                      .x$effluent <- 10^(log10(.x$inflow) - .x$logreduction)
                      .x
                      })
  tbl_risk <- map(tbl_risk, ~{
                      .x$exposure_perEvent = .x$effluent * .x$volume_perEvent
                      .x
                      })
  
  return(tbl_risk)
}

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("get_tbl_risk works", {
  scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
  
  expect_error(get_tbl_risk(scenario_test))

  scenario_conc <- inflow_concentration(scenario = scenario_test,
   pathogenName = c("Campylobacter jejuni", "Escherichia coli"))
  
  expect_error(get_tbl_risk(scenario_conc))
  
  scenario_volume <-
    scenario_conc |>                     
    mutate(
      volume = map(
        .x = config,                                         
        .f = ~ simulate_exposure(config = .x)              
      )
    )
  
  expect_error(get_tbl_risk(scenario_volume))
  
  scenario_dose_ini <- initial_dose_calculation(scenario_volume)
  
  scenario_scheme <- update_treatment_scheme(scenario_dose_ini)
  
  expect_error(get_tbl_risk(scenario_scheme))
  
  scenario_with_logreduc_and_co <- scenario_scheme |> 
    mutate( log_reduction = map(config, simulate_treatment))
  
  expect_error(get_tbl_risk(scenario_with_logreduc_and_co))
  
  scenario_final_dose_and_co_test <- final_dose_calculation(scenario_with_logreduc_and_co)
  
  result <- get_tbl_risk(scenario_final_dose_and_co_test)
  
  expect_equal(object = names(result[[1]]),
                expected = c("repeatID", "eventID", "inflow", "PathogenID", "PathogenName", 
"PathogenGroup", "volume_perEvent", "logreduction", "dose_perEvent", 
"effluent", "exposure_perEvent"))
  
  expect_equal(object = length(result),
                expected = 2)
  
  expect_equal(object = length(result[[1]]),
                expected = 11)
  expect_equal(object = length(result[[1]]$logreduction),
                expected = 120000)
})

# WARNING - Generated by {fusen} from dev/flat_kwb_infection.Rmd: do not edit by hand # nolint: line_length_linter.

#' Dose-response model: beta-poisson
#'
#' @param dose vector of dose data (default: 
#'   \code{sfsmisc::lseq(from = 0.1, to = 10^10, length = 1000)})
#' @param alpha  alpha (default: 3.28E-01)
#' @param N50  N50 (default: 5.43E+03)
#' @return tibble
#' @export
#' @importFrom sfsmisc lseq
#' @references Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' 
dr.betapoisson <- function(
  dose = sfsmisc::lseq(from = 1, to = 10^10, length = 1000), 
  alpha = 3.28E-01,
  N50 = 5.43E+03
)
{
  dplyr::as_tibble(data.frame(
    model = "betapoisson",
    dose = dose, 
    infectionProbability = 1 - (1 + dose * (2 ^ (1/alpha) - 1)/N50) ^ -alpha, 
    alpha = alpha, 
    N50 = N50
  ))
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Create random distribution based on configuration file
#' 
#' This function does the same thing as kwb.utils::generate_random_values(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' @param config as retrieved by config_read()
#' @param number_of_repeatings how often should the random distribution with the
#' same parameters be generated (default: 1)
#' @param number_of_events number of events
#' @param debug print debug information (default: TRUE)
#' @return list random distributions based on configuration file
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @export
#' @examples
#' # Example usage of generate_random_values
#' # Create a sample config object
#' config <- list(
#'   type = "norm",
#'   value = 10,
#'   min = 5,
#'   max = 15,
#'   mean = 10,
#'   sd = 2,
#'   meanlog = log(10),
#'   sdlog = 0.5,
#'   mode = 10
#' )
#'
#' # Call generate_random_values with the sample config
#' result <- generate_random_values(
#'   config = config,
#'   number_of_repeatings = 2,
#'   number_of_events = 10,
#'   debug = FALSE
#' )
#'
#' # Print the results
#' cat("Simulated events:")
#' print(result$events)
#'
#' cat("Parameters used for simulation:")
#' print(result$paras)
generate_random_values <- function(
  config, number_of_repeatings = 1, number_of_events, debug = TRUE
) 
{
  default <- function(x, default) {
    if (is.na(x)) default else x
  }
  
  # Set defaults based on min/max for function create_random_distribution() if
  # values are missing
  if (config$type == "norm") {
    config$mean <- default(config$mean, (config$max + config$min) / 2)
    config$sd <- default(config$sd, sd(c(config$mean, config$max)) / 2)
  }

  if (config$type == "lognorm") {
    config$meanlog <- default(config$meanlog, log((config$min + config$max) / 2))
    config$sdlog <- default(config$sdlog, sd(log((c(config$min, config$max)))))
  }

  if (config$type == "triangle") {
    if (is.na(config$mode)) {
      if (config$min == config$max) {
        catIf(debug,
          "Distribution set from 'triangle' to 'uniform' because 'min'", 
          "equals 'max'\n"
        )
        config$type <- "uniform"
      } else {
        config$mode <- (config$min + config$max) / 2
      }
    }
  }
  
  create_random_distribution(
    type = config$type,
    number_of_repeatings = number_of_repeatings,
    number_of_events = number_of_events,
    value = config$value,
    min = config$min,
    max = config$max,
    mean = config$mean,
    sdev = config$sd,
    meanlog = config$meanlog,
    sdlog = config$sdlog,
    mode = config$mode,
    debug = debug
  )
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("generate_random_values works correctly", {
  # Deterministic RNG so the range checks below are reproducible. Bounds are set
  # to +/- 5 sd: wide enough that valid draws never fall outside them, so the
  # test asserts the distribution's location/scale family, not an exact draw.
  set.seed(42)

  # Create a sample config object
  config <- list(
    type = "norm",
    value = 10,
    min = 5,
    max = 15,
    mean = 10,
    sd = 2,
    meanlog = log(10),
    sdlog = 0.5,
    mode = 10
  )

  # Test with default parameters
  result <- generate_random_values(
    config = config,
    number_of_repeatings = 1,
    number_of_events = 365,
    debug = FALSE
  )
  expect_equal(nrow(result$events), 365)
  expect_equal(ncol(result$events), 3)
  expect_equal(ncol(result$paras), 5)
  expect_true(all(result$events$values >= config$mean - 5 * config$sd & result$events$values <= config$mean + 5 * config$sd, na.rm = TRUE))

  # Test with different distribution type (lognorm)
  config$type <- "lognorm"
  result <- generate_random_values(
    config = config,
    number_of_repeatings = 1,
    number_of_events = 365,
    debug = FALSE
  )
  expect_equal(nrow(result$events), 365)
  expect_true(all(result$events$values >= exp(config$meanlog - 5*config$sdlog) &
                 result$events$values <= exp(config$meanlog + 5*config$sdlog), na.rm = TRUE))

  # Test with different distribution type (triangle)
  config$type <- "triangle"
  result <- generate_random_values(
    config = config,
    number_of_repeatings = 1,
    number_of_events = 365,
    debug = FALSE
  )
  expect_equal(nrow(result$events), 365)
  expect_true(all(result$events$values >= config$min & result$events$values <= config$max, na.rm = TRUE))

  # Test with different distribution type (value)
  config$type <- "value"
  result <- generate_random_values(
    config = config,
    number_of_repeatings = 1,
    number_of_events = 365,
    debug = FALSE
  )
  expect_equal(nrow(result$events), 365)
  expect_true(all(result$events$values == config$value, na.rm = TRUE))

  # Test with custom number_of_repeatings and number_of_events
  result <- generate_random_values(
    config = config,
    number_of_repeatings = 2,
    number_of_events = 10,
    debug = FALSE
  )
  expect_equal(nrow(result$events), 20)
  expect_equal(length(unique(result$events$repeatID)), 2)
  expect_equal(length(unique(result$events$eventID)), 10)

  # Test with debug = TRUE (should not cause errors)
  result <- generate_random_values(
    config = config,
    number_of_repeatings = 1,
    number_of_events = 365,
    debug = TRUE
  )
  expect_equal(nrow(result$events), 365)

  # Test with missing mean and sd for norm distribution
  config$type <- "norm"
  config$mean <- NA
  config$sd <- NA
  result <- generate_random_values(
    config = config,
    number_of_repeatings = 1,
    number_of_events = 365,
    debug = FALSE
  )
  expect_equal(nrow(result$events), 365)
  # mean defaults to (min + max)/2 = 10; generous fixed band, deterministic via seed
  expect_true(all(result$events$values >= 0 & result$events$values <= 20, na.rm = TRUE))

  # Test with missing meanlog and sdlog for lognorm distribution
  config$type <- "lognorm"
  config$meanlog <- NA
  config$sdlog <- NA
  result <- generate_random_values(
    config = config,
    number_of_repeatings = 1,
    number_of_events = 365,
    debug = FALSE
  )
  expect_equal(nrow(result$events), 365)

  # Test with missing mode for triangle distribution
  config$type <- "triangle"
  config$mode <- NA
  result <- generate_random_values(
    config = config,
    number_of_repeatings = 1,
    number_of_events = 365,
    debug = FALSE
  )
  expect_equal(nrow(result$events), 365)
  expect_true(all(result$events$values >= config$min & result$events$values <= config$max, na.rm = TRUE))

  # Test with min == max for triangle distribution (should switch to uniform)
  config$type <- "triangle"
  config$min <- 10
  config$max <- 10
  result <- generate_random_values(
    config = config,
    number_of_repeatings = 1,
    number_of_events = 365,
    debug = FALSE
  )
  expect_equal(nrow(result$events), 365)
  expect_true(all(result$events$values == 10, na.rm = TRUE))
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Remove Columns from a Data Frame
#' 
#' This function does the same thing as kwb.utils::catIf(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' @param dframe data frame,
#' @param columns vector of column names giving the columns to remove
#' @param columnsToRemove deprecated. Use argument \code{columns} instead.
#' @param pattern regular expression matching the names of the columns to be
#'   removed. Will only be evaluated if no explicit column names are given in 
#'   \code{columns}.
#' @param drop if FALSE, a data frame is returned in any case, otherwise the
#'   result may be a vector if only one column remains
#' @param dbg if \code{TRUE} (the default is \code{FALSE}), the deletion of
#'   columns is reported on the screen
#' @return \emph{dframe} with columns given in \emph{columns} being removed.
#'   User attributes of \emph{dframe} are restored.
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' 
#' @examples
#' # Example usage of removeColumns
#' df <- data.frame(
#'   a = 1:3,
#'   b = letters[1:3],
#'   c = c(TRUE, FALSE, TRUE)
#' )
#'
#' # Remove column 'b' using explicit column name
#' df_no_b <- removeColumns(df, columns = "b")
#'
#' # Remove columns matching pattern 'a' or 'c'
#' df_no_a_c <- removeColumns(df, pattern = "^[ac]$")
#'
#' # Print results
#' print("Original data frame:")
#' print(df)
#'
#' print("Data frame with column 'b' removed:")
#' print(df_no_b)
#'
#' print("Data frame with columns 'a' and 'c' removed:")
#' print(df_no_a_c)
removeColumns <- function(
  dframe, columns = NULL, columnsToRemove = NULL, pattern = NULL, drop = FALSE,
  dbg = FALSE
)
{ 
  if (is.null(columns) && ! is.null(columnsToRemove)) {
    
    warning(
      "The argument 'columnsToRemove' is deprecated. Please use the new ", 
      "argument 'columns' instead.", call. = FALSE
    )
    
    columns <- columnsToRemove
  }
  
  all_columns <- names(dframe)
  
  if (is.null(columns)) {
    
    if (is.null(pattern)) {
      
      stop(
        "Either 'columns' or 'pattern' must be given to removeColumns()",
        call. = FALSE
      )
      
    } else {
      
      columns <- grep(pattern, all_columns, value = TRUE)
    }
  }
  
  columns_keep <- setdiff(all_columns, columns)
  
  if (dbg) {
    
    columns_remove <- intersect(all_columns, columns)
    
    cat(sprintf(
      "Removing column%s %s from '%s'.\n", 
      ifelse(length(columns_remove) > 1, "s", ""), stringList(columns_remove), 
      deparse(substitute(dframe))
    ))
  }
  
  hsRestoreAttributes(dframe[, columns_keep, drop = drop], attributes(dframe))
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("removeColumns works correctly with explicit columns", {
  df <- data.frame(a = 1:3, b = letters[1:3], c = c(TRUE, FALSE, TRUE))

  # Remove single column
  result <- removeColumns(df, columns = "b")
  expect_equal(names(result), c("a", "c"))
  expect_equal(result$a, df$a)
  expect_equal(result$c, df$c)

  # Remove multiple columns
  result <- removeColumns(df, columns = c("a", "c"))
  expect_equal(names(result), "b")
  expect_equal(result$b, df$b)
})

test_that("removeColumns works correctly with pattern", {
  df <- data.frame(a = 1:3, b = letters[1:3], c = c(TRUE, FALSE, TRUE))

  # Remove columns matching pattern
  result <- removeColumns(df, pattern = "^[ac]$")
  expect_equal(names(result), "b")
  expect_equal(result$b, df$b)
})

test_that("removeColumns handles non-existent columns", {
  df <- data.frame(a = 1:3, b = letters[1:3])

  # Should return original data frame unchanged
  result <- removeColumns(df, columns = "x")
  expect_equal(result, df)

  # Should return original data frame unchanged
  result <- removeColumns(df, pattern = "x")
  expect_equal(result, df)
})

test_that("removeColumns maintains attributes", {
  df <- data.frame(a = 1:3, b = letters[1:3])
  attr(df, "my_attr") <- "test"

  result <- removeColumns(df, columns = "b")
  expect_equal(attr(result, "my_attr"), "test")
})

test_that("removeColumns handles deprecated columnsToRemove argument", {
  df <- data.frame(a = 1:3, b = letters[1:3])

  # Test warning is issued
  expect_warning(removeColumns(df, columnsToRemove = "b"))

  # Test functionality remains the same (deprecation warning already asserted above)
  result <- suppressWarnings(removeColumns(df, columnsToRemove = "b"))
  expect_equal(names(result), "a")
})

test_that("removeColumns requires either columns or pattern", {
  df <- data.frame(a = 1:3, b = letters[1:3])

  # Should throw error
  expect_error(removeColumns(df))
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Print New Line Character to the Console if Condition is Met
#'
#' This function does the same thing as kwb.utils::catIf(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' @param condition if \code{TRUE} the new line is printed else not
#' @return Returns the condition, invisibly so that it can be reused
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' 
#' @examples
#' catNewLineIf(TRUE)
#' catNewLineIf(FALSE)
catNewLineIf <- function(condition)
{
  catIf(condition, "\n")
  
  invisible(condition)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("catNewLineIf works correctly", {
  # Test that catNewLineIf prints a new line when condition is TRUE
  output <- capture.output(catNewLineIf(TRUE))
  expect_equal(output, "")

  # Test that catNewLineIf doesn't print a new line when condition is FALSE
  output <- capture.output(catNewLineIf(FALSE))
  expect_equal(output, character(0))
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Irrigation need calculation
#' 
#' Calculate irrigation need according to irrigation need of the culture simulated and its surface
#' @param scenario data.frame scenario obtained with create_scenario function
#' @importFrom dplyr mutate
#' @return scenario_updated
#' 
#' @export
#' @examples
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
#' print(irrigation_need$irrigation_need_m3an)
irrigation_need_calculation <- function(scenario){
  scenario <- query_irrigation_need(scenario)
  
  scenario_updated <- scenario |> mutate("irrigation_need_m3an" = IrrigationNeed * Area)
  return(scenario_updated)
    
}

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("treatment_cost_calculation works", {
  scenario_apprentissage <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
  result <- collective_treatment_cost_calculation(irrigation_need, grant = 0.5)
   
  expect_true(all(c("scenario", "allocation_key") %in% names(result))) 
  
  scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
  irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
  crop <- scenario_apprentissage$CropName |> unique()
  allocation_custom <- data.frame(CropName = crop,
                                allocation = c(0.5, 0.5))
  result <- collective_treatment_cost_calculation(scenario = irrigation_need, 
                                      grant = 0,
                                      allocation_key = allocation_custom)
  
  
  expect_equal(object = result$allocation_key,
               expected = structure(list(grant = c(0, 0), CropName = c("Tomato", "Corn seed"
), allocation = c(0.5, 0.5)), row.names = 1:2, class = "data.frame"))
  
  allocation_custom <- data.frame(allocation = c(0.5, 05))
  expect_error(object = collective_treatment_cost_calculation(scenario = irrigation_need, 
                                      grant = 0,
                                      allocation_key = allocation_custom), 
               regexp = "'allocation_key' must contain the following columns: CropName, allocation. Missing: CropName" )
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("total_irrigation_need works", {
  scenario_apprentissage <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
  result <- total_irrigation_need(irrigation_need)
  expect_equal(object = result,
               expected = 222500 ) 
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("barrier_cost_calculation works", {
  scenario_apprentissage <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
  result <- process_cost_calculation(scenario = irrigation_need,
                                     membership_fee = 200,
                                    charge = 0.1,
                                    initialSituation = TRUE)
  expect_equal(object = names(result),
               expected = c("CropName", "Area", "PopulationName", "nb_population", "PathName", 
"STEPtreatmentName", "CollectiveTreatmentName", "InitialProcessName", 
"SupplementaryProcessName", "nb_day_decay", "config", "CropID", 
"CropHeight", "PopulationID", "PathID", "MatrixID", "STEPtreatmentID", 
"CollectiveTreatmentID", "InitialProcessID", "SupplementaryProcessID", 
"InitialTrainName", "InitialTrainID", "SupplementaryTrainName", 
"SupplementaryTrainID", "IrrigationNeed", "irrigation_need_m3an", 
"perimeter", "capex_supplementaryProcess", "capex_supplementaryProcess_20y", 
"opex_supplementaryProcess")) 
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("water_price works", {
  scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
  irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
  result <- water_price(scenario = irrigation_need,
            price_per_m3 = 0.01,
            membership_fee = 200)
  expect_true(all(c("water_charge", "annual_membership_fee") %in% names(result))) 
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Perimeter calculation
#' 
#' Calculate perimeter (linear meter) according to surface, and add the value to the scenario dataframe
#' @param scenario data.frame scenario obtained with create_scenario function
#' @importFrom dplyr mutate
#' @return scenario_updated
#' 
#' @export
#' @examples
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#'
#' scenario_apprentissage_updated <- perimeter_calculation(scenario_apprentissage)
#' print(scenario_apprentissage_updated$perimeter)
perimeter_calculation <- function(scenario){
    
    scenario_updated <- scenario |> mutate("perimeter" = sqrt(Area)*8)
    return(scenario_updated)
}

# WARNING - Generated by {fusen} from dev/flat_simulate_treatment.Rmd: do not edit by hand # nolint: line_length_linter.

#' Simulate treatment data for pathogen processes
#'
#' This function simulates treatment data for a set of pathogen processes, including
#' random values for each process and optional parameters. It can be used to generate
#' synthetic data for analysis or testing.
#' 
#' This function does the same thing as kwb.qmra::get_treatment_data(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#'
#' @param processes A data frame containing information about pathogen processes,
#'   including treatment names and pathogen groups.
#' @param n_repeatings The number of times each process should be repeated.
#' @param n_events The number of events to simulate for each process.
#' @param debug Logical indicating whether to print debug information.
#' @param include_paras Logical indicating whether to include parameter data.
#' 
#' @return If `include_paras` is FALSE, returns a data frame of simulated events.
#'   If `include_paras` is TRUE, returns a list containing both the events data frame
#'   and a data frame of parameters.
#'
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' 
#' @export
#' @examples
#' # Example usage (assuming 'processes' is defined)
#' scenario_test <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' scenario_updated <- update_pathogen(scenario = scenario_test,
#'  pathoName = c("Campylobacter jejuni", "Rotavirus"))
#' config = scenario_updated$config[[1]]
#' processes_simulated <- config$treatment$processes
#'
#' result <- get_treatment_data(processes_simulated, n_repeatings = 10, n_events = 5)
#' print(result)
get_treatment_data <- function(
  processes, n_repeatings, n_events, debug = TRUE, include_paras = TRUE
)
{
  # Helper function to select a treatment from "processes" and report about it
  get_treatment <- function(i) {
    treatment <- processes[i, ]
    if (debug) cat(sprintf(
      "Simulated treatment: %s for %s\n", 
      treatment$TreatmentName, 
      treatment$PathogenGroup
    ))
    treatment[, c("TreatmentID", "PathogenGroup")]
  }
  
  # Provide row indices
  indices <- seq_len(nrow(processes))
  
  # Create random values for each process
  random_list <- lapply(indices, function(i) {
    generate_random_values(
      config = processes[i, ], 
      number_of_repeatings = n_repeatings,
      number_of_events = n_events,
      debug = debug
    )
  })

  # Function to cbind random events and treatment
  cbind_events_treatment <- function(i) {
    cbind(random_list[[i]]$events, get_treatment(i), row.names = NULL)
  }
  
  # Build data frame "events"
  events <- lapply(indices, cbind_events_treatment) |>
    do.call(what = rbind) |>
    renameColumns(list(values = "logreduction"))
  
  # Return the events if parameters are not requested
  if (! include_paras) {
    return(events)
  }
  
  # Function to cbind random events and treatment
  cbind_paras_treatment <- function(i) {
    cbind(random_list[[i]]$paras, get_treatment(i), row.names = NULL)
  }
  
  # Return a list with events and parameters
  list(
    events = events,
    paras = lapply(indices, cbind_paras_treatment) |>
      do.call(what = plyr::rbind.fill)
  )
}

# WARNING - Generated by {fusen} from dev/flat_simulate_treatment.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("get_treatment_data returns correct structure", {
  scenario_test <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  scenario_updated <- update_pathogen(scenario = scenario_test,
   pathoName = c("Campylobacter jejuni", "Rotavirus"))
  config = scenario_updated$config[[1]]
  test_processes <- config$treatment$processes

  # Test with include_paras = FALSE
  result_events <- get_treatment_data(test_processes, n_repeatings = 2, n_events = 3,
   debug = FALSE, include_paras = FALSE)
  expect_equal(nrow(result_events), 504) # 84 processes * 2 repeatings * 3 events
  expect_true(all(c("logreduction", "TreatmentID", "PathogenGroup") %in% colnames(result_events)))
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Total area
#' 
#' Calculate the total area of each culture of the simulated scenario
#' @param scenario data.frame scenario created with create_scenario function
#' @importFrom dplyr select distinct
#' @return surface_total data.frame 
#' 
#' @export
#' @examples
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' total_area_calculation(scenario = scenario_apprentissage)
total_area_calculation <- function(scenario){
  surface_totale_df <- scenario |>  select (CropID, Area) |>  distinct()
  surface_total <- sum(surface_totale_df$Area)
  
  return(surface_total)
    
}

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("total_area_calculation works", {
  scenario_apprentissage <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  result <- total_area_calculation(scenario = scenario_apprentissage)
  expect_equal(object = result, 
               expected = 45) 
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Barrier Query
#' 
#' retrieve Barrier ID according to its name in the barrier database
#' @param config list configuration file default value is config_ambre dataset
#' @param barrierName character Name of the barrier you want to find the ID
#' @param na.rm boolean FALSE by default
#' @importFrom dplyr filter pull
#' @importFrom purrr map
#' @return barrierID
#' 
#' @export
#' @examples
#' query_barrier(barrierName = "Q.1 - Activated Sludge")
#' query_barrier(barrierName = c("Q.1 - Activated Sludge", "Q.2 - Maturation Pond"))
#' query_barrier(barrierName = "P.9 - Peeling, P.7 - Cooking")
query_barrier <- function(config = config_ambre, barrierName, na.rm = FALSE){
  
  if (length(barrierName) == 1) {
    if (grepl(",", barrierName)) {
      barrierName <- trimws(strsplit(barrierName, ",")[[1]])
    } else {
      barrierName <- trimws(barrierName)
    }
  }
  
  # Gestion des NA
  if (any(is.na(barrierName))) {
    if (na.rm) {
      barrierName <- barrierName[!is.na(barrierName)]
      if (length(barrierName) == 0) {
        return(NA)
      }
    } else {
      return(NA)
    }
  }
  
  allowed_barrier <- c("Q.1 - Activated Sludge", 
                        "Q.2 - Maturation Pond", 
                        "Q.3 - UV Reactor",  
                        "Q.4 - Sand Filter and UV", 
                        "Q.5 - Microfiltration and UV",  
                        "Q.6 - Chlorination",   
                        "Q.7 - Microfiltration and UV and Chlorination", 
                        "P.1 - Non-edilble crop",
                        "P.2 - Distance of 70m", 
                        "P.3 - Night-time irrigation",  
                        "P.4 - Shutdown irrigation in strong winds", 
                        "P.5 - Natural die-off",  
                        "P.6 - Agri-food processing", 
                        "P.7 - Cooking",
                        "P.8 - Drying", 
                       "P.9 - Peeling", 
                        "P.10 - Rinsing with drinking water", 
                        "P.11 - Washing with disinfectant", 
                        "E.1 - Automatic irrigation",  
                        "E.2 - Mechanised crop maintenance", 
                        "E.3 - Mechanised harvesting", 
                        "E.4 - Signage",
                        "E.5 - Fences", 
                        "E.7 - Sheet mulching", 
                        "E.8 - Personnal Protective Equipement", 
                        "E.1.1 - Micro-sprinkler",
                        "E.1.2 - Surface drip irrigation",  
                        "E.1.3 - Subsurface drip irrigation"
                        )
  if (any(!barrierName %in% allowed_barrier)) {
    stop("Barrier name doesn't exist")}
  
  barrierID <- config_ambre$treatment$processes |> 
                filter(TreatmentName %in% barrierName) |> 
                pull(TreatmentID) |> 
                unique()
  
  return(barrierID)
    
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("query_barrier works", {
  expect_equal(object = query_barrier(barrierName = "Q.1 - Activated Sludge"),
               expected = 1)
  expect_equal(object = query_barrier(barrierName = "P.9 - Peeling, P.7 - Cooking"),
               expected = c(14, 16))
  expect_equal(object = query_barrier(barrierName = NA),
               expected = NA)
  expect_error(object = query_barrier(barrierName = "Activated Sludge"),
               regexp = "Barrier name doesn't exist")
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Barrier Cost Calculation
#' 
#' Calculate cost of supplementary barriers
#' @param scenario data.frame scenario with irrigation_need_m3an column obtained from irrigation_need_calculation function
#' @param membership_fee numeric annual membership fee
#' @param charge numeric water charge
#' @param initialSituation boolean TRUE to simulate initial simulation cost, FALSE to simulate supplementary process cost
#' @importFrom dplyr rename left_join filter mutate inner_join case_when
#' @importFrom tidyr unnest
#' @return scenario_updated
#' 
#' @export
#' @examples
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre"))
#' irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
#' process_cost_calculation(scenario = irrigation_need,
#'                          membership_fee = 200,
#'                          charge = 0.1,
#'                          initialSituation = TRUE)
#'
#'
#' process_cost_calculation(scenario = irrigation_need,
#'                          membership_fee = 200,
#'                          charge = 0.1,
#'                          initialSituation = FALSE)
process_cost_calculation <- function(scenario, membership_fee, charge, initialSituation){
 
  scenario_updated <- scenario
  perimeter_calc <- perimeter_calculation(scenario)
  scenario_updated$perimeter <- perimeter_calc$perimeter
  
  if(initialSituation == TRUE){
    supplementary_cost_capex <- scenario_updated |>  rename("TreatmentID" = "InitialProcessID") |> 
                                      unnest(TreatmentID) |>
                                      left_join(config_ambre$economic$cost, by = "TreatmentID", 
                                                relationship = "many-to-many") |> 
                                      filter(CostType == "capex" | is.na(CostType))|> 
                                      mutate("capex" = case_when(
                                                  unit == "\u20ac/ha" ~ value * Area,
                                                  unit == "\u20ac/ml" ~ value * perimeter,
                                                  unit == "\u20ac/p" ~ value * nb_population,
                                                  unit == "\u20ac"   ~ value,
                                                  unit == "\u20ac/m3"   ~ value * irrigation_need_m3an,
                                                  TRUE ~ NA_real_)) |>
                                      group_by(CropName, PopulationName, PathName) |>
                                        summarise(
                                          capex = sum(capex, na.rm = TRUE),
                                          .groups = "drop"
                                        )
    scenario_updated$capex_supplementaryProcess <- supplementary_cost_capex$capex
    scenario_updated$capex_supplementaryProcess_20y <- scenario_updated$capex_supplementaryProcess/20
    
    supplementary_cost_opex <- scenario_updated |>  rename("TreatmentID" = "InitialProcessID") |> 
                                      unnest(TreatmentID) |>
                                      left_join(config_ambre$economic$cost, by = "TreatmentID",
                                                relationship = "many-to-many") |> 
                                      filter(CostType == "opex" | is.na(CostType)) |> 
                                      mutate("opex" = case_when(
                                                  unit == "\u20ac/ha" ~ value * Area,
                                                  unit == "\u20ac/ml" ~ value * perimeter,
                                                  unit == "\u20ac/p" ~ value * nb_population,
                                                  unit == "\u20ac"   ~ value,
                                                  unit == "\u20ac/m3/y"   ~ value * irrigation_need_m3an,
                                                  TRUE ~ NA_real_)) |>
                                      group_by(CropName, PopulationName, PathName) |>
                                        summarise(
                                          opex = sum(opex, na.rm = TRUE),
                                          .groups = "drop"
                                        )
    
    scenario_updated$opex_supplementaryProcess <- supplementary_cost_opex$opex
  }
  
  else if(initialSituation == FALSE){
    supplementary_cost_capex <- scenario_updated |>  rename("TreatmentID" = "SupplementaryProcessID") |> 
                                      unnest(TreatmentID) |>
                                      left_join(config_ambre$economic$cost, by = "TreatmentID", 
                                                relationship = "many-to-many") |> 
                                      filter(CostType == "capex" | is.na(CostType))|> 
                                      mutate("capex" = case_when(
                                                  unit == "\u20ac/ha" ~ value * Area,
                                                  unit == "\u20ac/ml" ~ value * perimeter,
                                                  unit == "\u20ac/p" ~ value * nb_population,
                                                  unit == "\u20ac"   ~ value,
                                                  unit == "\u20ac/m3"   ~ value * irrigation_need_m3an,
                                                  TRUE ~ NA_real_))|>
                                      group_by(CropName, PopulationName, PathName) |>
                                        summarise(
                                          capex = sum(capex, na.rm = TRUE),
                                          .groups = "drop"
                                        )
    scenario_updated$capex_supplementaryProcess <- supplementary_cost_capex$capex
    scenario_updated$capex_supplementaryProcess_20y <- scenario_updated$capex_supplementaryProcess/20
    
    supplementary_cost_opex <- scenario_updated |>  rename("TreatmentID" = "SupplementaryProcessID") |> 
                                      unnest(TreatmentID) |>
                                      left_join(config_ambre$economic$cost, by = "TreatmentID",
                                                relationship = "many-to-many") |> 
                                      filter(CostType == "opex" | is.na(CostType)) |> 
                                      mutate("opex" = case_when(
                                                  unit == "\u20ac/ha" ~ value * Area,
                                                  unit == "\u20ac/ml" ~ value * perimeter,
                                                  unit == "\u20ac/p" ~ value * nb_population,
                                                  unit == "\u20ac"   ~ value,
                                                  unit == "\u20ac/m3/y"   ~ value * irrigation_need_m3an,
                                                  TRUE ~ NA_real_))|>
                                      group_by(CropName, PopulationName, PathName) |>
                                        summarise(
                                          opex = sum(opex, na.rm = TRUE),
                                          .groups = "drop"
                                        )
    
    scenario_updated$opex_supplementaryProcess <- supplementary_cost_opex$opex
  }

  return(scenario_updated)
    
}

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' QMRA on learning case
#' 
#' Run a learning case to evaluate risk on scenario with 6 crops (Corn Seed, Tomato, Potato, Salad, Onion) and 6 populations (maintainer, harverster, irrigation, consumer, local resident, passerby). Run QMRA calculation with barrier measures
#' @importFrom purrr map pmap
#' @importFrom dplyr mutate filter select
#' @return all_plots list
#' 
#' @export
#' @examples
#' \donttest{
#' run_qmra_learning_case()
#' }
run_qmra_learning_case <- function(){
  scenario_example <- create_scenario(system.file("input_cas_apprentissage_complet.xlsx", package = "ambre"))


  frequency_up <- c(3L, 12L, 20L, 20L, 3L, 10L, 20L, 20L, 48L,
                    3L, 20L, 20L, 96L, 3L, 48L, 12L, 20L, 96L,
                    3L, 48L, 48L, 20L, 48L, 3L, 48L, 48L, 48L)
  scenario_frequency <- update_frequency_with_desired_value(scenario = scenario_example,                                                          frequency = frequency_up)
  
  volumes_up <- data.frame("min" = c(0.05,0.001,0.00174,0.001,0.03,0.0001,0.0008,0.001,0.009,0.03,0.0008,0.001,0.006,0.01,0.001,0.001,0.0008,0.00375,0.01,0.001,0.001,0.0008,0.015,0.005,0.001,0.001,0.0025), 
                           "max" = c(0.05,0.001,0.00174,0.001,0.03,0.0001,0.0008,0.001,0.009,0.03,0.0008,0.001,0.006,0.01,0.001,0.001,0.0008,0.00375,0.01,0.001,0.001,0.0008,0.015,0.005,0.001,0.001,0.0025),
                           "type" = rep("triangle", nrow(scenario_example)))
  scenario_volume <- update_volume_with_desired_value(scenario = scenario_frequency,                                                    volume = volumes_up)
  scenario_exposure <- scenario_volume |> mutate(
       volume = map(
           .x = config,                                         
           .f = ~ simulate_exposure(config = .x)))
  plot_volume <- plot_volumes(scenario_exposure)
  
  scenario_patho <- update_pathogen(scenario = scenario_exposure, pathoName = c("Rotavirus"))
  concentration = map(.x = scenario_patho$config,
                     .f= ~ simulate_inflow(.x))
  scenario_concentration <- scenario_patho |> mutate(inflow_concentration = concentration)
  plot_inflow <- plot_inflow(scenario_concentration)
  
  scenario_ini_dose <- initial_dose_calculation(scenario = scenario_concentration)
  
  scenario_barrier <- update_treatment_scheme(scenario = scenario_ini_dose, initial_situation = TRUE)
  scenario_barrier$config <- pmap(
    list(
      config = scenario_barrier$config,
      barrierID = scenario_barrier$InitialProcessID,
      cropHeight = scenario_barrier$CropHeight,
      matrixID = scenario_barrier$MatrixID,
      popID = scenario_barrier$PopulationID,
      decay = scenario_barrier$nb_day_decay
    ),
    retrieve_specific_barrier
  )
  
  scenario_logreduction <- scenario_barrier |> 
    mutate(log_reduction = map(config, simulate_treatment))
  plot_log_reduc <- plot_logreduction(scenario=scenario_logreduction)
  
  regulation_concentration <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                        select(-c(Country, RegulationID, Reduction))
  regulation_reduction <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                        select(-c(Concentration, Country, RegulationID))
  matrix_log <- regulation_matrix_log(scenario = scenario_logreduction,
                                      regulation = regulation_reduction)
  matrix_concentration <- regulation_matrix_concentration(scenario = scenario_logreduction,
                                                          regulation = regulation_concentration)
    
  scenario_final_dose <- final_dose_calculation(scenario_logreduction)
  scenario_inf_proba <- infection_probability_calculation(scenario_final_dose)
  scenario_illness_proba <- illness_probability_calculation(scenario_inf_proba)
  scenario_dalys <- dalys_calculation(scenario_illness_proba)
  scenario_risk_total <- get_risk_total(scenario_dalys)
  plot_dalys <- plot_dalys(scenario_risk_total)
  
  economic_result <- run_economic_analysis(scenario = scenario_risk_total,
                      price_per_m3 = 0.1, 
                      membership_fee = 200, 
                      grant = 0.5,
                      initialSituation = TRUE)
  
  all_plots <- list(
    concentration = plot_inflow,
    volume = plot_volume,
    logreduction = plot_log_reduc,
    dalys = plot_dalys,
    regul_matrix_log = matrix_log,
    regul_matrix_concentration = matrix_concentration,
    economic = economic_result
  )
  
  return(all_plots)
    
}

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("run_qmra_learning_case works", {
  result <- run_qmra_learning_case()
  expect_equal(object = names(result),
               expected = c("concentration", "volume", "logreduction", 
                            "dalys", "regul_matrix_log", "regul_matrix_concentration", "economic")) 
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Query frequency
#' 
#' Retrieve min and max frequency values according to path defined in the scenario (database ambre_frequence)
#' @param pathID integer ID of the path for which we want to retrieve the frequency values
#' @importFrom dplyr filter select
#' @return frequency
#' 
#' @export
#' @examples
#' scenario_example <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#'
#' query_frequency(pathID = scenario_example$PathID)
query_frequency <- function(pathID){
    frequency <- config_ambre$path$frequency |> dplyr::filter(PathID %in% pathID) |>
        dplyr::select(min, max)
    return(frequency)
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("query_frequency works", {
  scenario_example <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  expected_result <- c("min", "max")
  expect_equal(object = colnames(query_frequency(scenario_example$PathID)),
               expected = expected_result)
})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Initial dose concentration
#'
#' This function calculates the initial dose that is a product of inflow concentration and volume for each exposure events.
#' @param scenario dataFrame scenario with volume and inflow_concentration columns
#' 
#' @importFrom purrr map2
#' @importFrom dplyr mutate
#' @return scenario_with_initial_dose
#' @export
#' @examples
#' library(dplyr)
#' library(purrr)
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#'
#' scenario_conc <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni"))
#'
#' scenario_volume <-
#'   scenario_conc |>                     
#'   mutate(
#'     volume = map(
#'       .x = config,                                         
#'       .f = ~ simulate_exposure(config = .x)              
#'     )
#'   )
#'
#' initial_dose_calculation(scenario_volume)
initial_dose_calculation <- function(scenario){

  if (!"inflow_concentration" %in% names(scenario)) {
  stop("The inflow_concentration column is missing from the scenario object; run the inflow_concentration function to create it")
}
  
  if (!"volume" %in% names(scenario)) {
  stop("The volume column is missing from the scenario object; run the simulate_exposure function to create it")
}
  scenario_with_initial_dose<- scenario |>      
  mutate(
    initial_dose = map2(
      .x = inflow_concentration,   
      .y = volume,
      .f = ~ .x$events$inflow * .y$volumes$events$volume_perEvent              
    )
  )
  return(scenario_with_initial_dose)
}

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("retrieve_specific_barrier applies a vector of barriers cumulatively", {
  scenario <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
  scenario_conc <- inflow_concentration(scenario = scenario, pathogenName = "Campylobacter jejuni")
  scenario_vol <- scenario_conc |>
    dplyr::mutate(volume = purrr::map(config, ~ simulate_exposure(config = .x)))
  scenario_ttt <- update_treatment_scheme(
    scenario = initial_dose_calculation(scenario_vol),
    initial_situation = TRUE
  )

  cfg <- scenario_ttt$config[[1]]
  ch <- scenario_ttt$CropHeight[[1]]
  matrix_id <- scenario_ttt$MatrixID[[1]]
  pop_id <- scenario_ttt$PopulationID[[1]]
  decay <- scenario_ttt$nb_day_decay[[1]]

  # A vector of barrier ids == applying each one after the other (cumulative).
  expect_identical(
    retrieve_specific_barrier(cfg, c(9, 13), ch, matrix_id, pop_id, decay),
    retrieve_specific_barrier(
      retrieve_specific_barrier(cfg, 9, ch, matrix_id, pop_id, decay),
      13, ch, matrix_id, pop_id, decay
    )
  )
  # An empty role (NA) matches no branch and leaves the config unchanged.
  expect_identical(retrieve_specific_barrier(cfg, NA, ch, matrix_id, pop_id, decay), cfg)
})

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("regulation_matrix works", {
  scenario_example <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

  scenario_exposure <- scenario_example |> mutate(
     volume = map(
         .x = config,                                         
         .f = ~ simulate_exposure(config = .x)))
  
  scenario_patho <- update_pathogen(scenario = scenario_exposure, pathoName = c("Rotavirus", "Campylobacter jejuni"))
  concentration = map(.x = scenario_patho$config,
                     .f= ~ simulate_inflow(.x))
  scenario_concentration <- scenario_patho |> mutate(inflow_concentration = concentration)
  scenario_ini_dose <- initial_dose_calculation(scenario = scenario_concentration)
  
  scenario_barrier <- update_treatment_scheme(scenario = scenario_ini_dose, initial_situation = TRUE)
  
  scenario_barrier$config <- purrr::pmap(
    list(
      config = scenario_barrier$config,
      barrierID = scenario_barrier$SupplementaryProcessID,
      cropHeight = scenario_barrier$CropHeight,
      matrixID = scenario_barrier$MatrixID,
      popID = scenario_barrier$PopulationID,
      decay = scenario_barrier$nb_day_decay
    ),
    function(config, barrierID, cropHeight, matrixID, popID, decay) {
  
      if (barrierID %in% c(24, 26, 27, 28) &&
          matrixID %in% c(3, 5)) {
  
        update_logreduction_specific_barrier(
          config = config,
          cropHeight = cropHeight,
          barrierID = barrierID
        )
  
      } else if (barrierID %in% c(12)) {
  
        update_logreduction_decay(
          config = config,
          cropHeight = cropHeight,
          barrierID = barrierID,
          popID = popID,
          nb_day_decay = decay
        )
  
      } else if (barrierID %in% c(9,13,14,15,16,17,18,24,26,27,28)) {
  
        update_logreduction_path(
          config = config,
          matrixID = matrixID,
          barrierID = barrierID,
          popID = popID
        )
  
      } else {
        config
      }
    }
  )
    
  scenario_logreduction <- scenario_barrier |> 
      mutate(log_reduction = map(config, simulate_treatment))
  
  regulation_reduction <- config_ambre$regulation$regulation_value |> dplyr::filter(Country == "France") |>
                          dplyr::select(-c(Concentration, Country, RegulationID))
  result <- regulation_matrix_log(scenario = scenario_logreduction,
                                  regulation = regulation_reduction)
  expect_equal(object = names(result),
               expected = c("Crop name", "Target water class", "Bacteria", "Viruses"))
  
  expect_error(object = regulation_matrix_log(scenario = scenario_barrier,
                                              regulation = "France"),
               regexp = "The log_reduction column is missing from the scenario object; run simulate_treatment function to create it")
})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

library(dplyr)
library(purrr)
test_that("final_dose_calculation works", {
  scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

scenario_conc <- inflow_concentration(scenario = scenario_test,
 pathogenName = c("Campylobacter jejuni", "Escherichia coli"))

scenario_volume <-
  scenario_conc |>                     
  mutate(
    volume = map(
      .x = config,                                         
      .f = ~ simulate_exposure(config = .x)              
    )
  )

scenario_dose_ini <- initial_dose_calculation(scenario_volume)

scenario_scheme <- update_treatment_scheme(scenario_dose_ini)

scenario_with_logreduc_and_co <- scenario_scheme |> 
  mutate( log_reduction = map(config, simulate_treatment))

  result <- final_dose_calculation(scenario_with_logreduc_and_co)
  
  expect_true("final_dose" %in% names(result)
  ) 
})

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("plot_logreduction works", {
  scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

  scenario_conc <- inflow_concentration(scenario = scenario_test,
   pathogenName = c("Campylobacter jejuni"))
  
  scenario_volume <-
    scenario_conc |>                     
    mutate(
      volume = map(
        .x = config,                                         
        .f = ~ simulate_exposure(config = .x)              
      )
    )
  
  scenario_dose_ini <- initial_dose_calculation(scenario_volume)
  
  scenario_scheme <- update_treatment_scheme(scenario_dose_ini)
  
  scenario_with_logreduc_and_co <- scenario_scheme |> 
    mutate( log_reduction = map(config, simulate_treatment))
  
  result <- plot_logreduction(scenario_with_logreduc_and_co)
  expect_type(result, "list")
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)

## ----setup--------------------------------------------------------------------
library(ambre)
set.seed(2024)

## ----families-----------------------------------------------------------------
tn <- sort(unique(config_ambre$treatment$processes$TreatmentName))
head(tn, 15)

## ----input--------------------------------------------------------------------
readxl::read_excel(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)[, c("STEPtreatmentName", "CollectiveTreatmentName",
      "InitialProcessName", "SupplementaryProcessName")]

## ----compare, results = "hide"------------------------------------------------
library(dplyr)
scenario_example <- create_scenario(filepath = system.file("input_1culture_2pop.xlsx", package = "ambre"))
regulation_reduction <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                          select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                        select(-c(Country, RegulationID, Reduction))

comparison <- plot_comparison_qmra_initial_vs_supplementary_processes(
  scenario = scenario_example,
  pathogen = c("Campylobacter jejuni"),
  regulationLog = regulation_reduction,
  regulationConcentration = regulation_concentration
)

## ----compare-names------------------------------------------------------------
names(comparison)

## ----compare-logreduction, fig.height = 4-------------------------------------
comparison$log_reduction

## ----compare-dalys, fig.height = 4--------------------------------------------
comparison$dalys

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

#' Concentration reduction matrix calculation
#' 
#' Calculate the difference of achieve concentration in effluent according to regulation 
#' @param scenario data.frame with log_reduction column obtained with simulate_treatment function
#' @param regulation data.frame regulation table to compare to simulation results
#' @importFrom formattable formattable
#' @importFrom purrr map
#' @import dplyr
#' @importFrom tidyr unnest pivot_wider
#' @return formattable 
#' 
#' @export
#' @examples
#' \donttest{
#' library(dplyr)
#' library(purrr)
#' scenario_example <- create_scenario(system.file("input_1culture_2pop.xlsx", 
#'                                                 package = "ambre"))
#'
#' scenario_exposure <- scenario_example |> mutate(
#'      volume = map(
#'          .x = config,                                         
#'          .f = ~ simulate_exposure(config = .x)))
#'
#' scenario_patho <- update_pathogen(scenario = scenario_exposure, 
#'                                   pathoName = c("Rotavirus", "Campylobacter jejuni", "Norovirus"))
#' concentration = map(.x = scenario_patho$config,
#'                    .f= ~ simulate_inflow(.x))
#' scenario_concentration <- scenario_patho |> mutate(inflow_concentration = concentration)
#' scenario_ini_dose <- initial_dose_calculation(scenario = scenario_concentration)
#'
#' scenario_barrier <- update_treatment_scheme(scenario = scenario_ini_dose, initial_situation = TRUE)
#'
#' scenario_barrier$config <- purrr::pmap(
#'   list(
#'     config = scenario_barrier$config,
#'     barrierID = scenario_barrier$SupplementaryProcessID,
#'     cropHeight = scenario_barrier$CropHeight,
#'     matrixID = scenario_barrier$MatrixID,
#'     popID = scenario_barrier$PopulationID,
#'     decay = scenario_barrier$nb_day_decay
#'   ),
#'   function(config, barrierID, cropHeight, matrixID, popID, decay) {
#'
#'     if (barrierID %in% c(24, 26, 27, 28) &&
#'         matrixID %in% c(3, 5)) {
#'
#'       update_logreduction_specific_barrier(
#'         config = config,
#'         cropHeight = cropHeight,
#'         barrierID = barrierID
#'       )
#'
#'     } else if (barrierID %in% c(12)) {
#'
#'       update_logreduction_decay(
#'         config = config,
#'         cropHeight = cropHeight,
#'         barrierID = barrierID,
#'         popID = popID,
#'         nb_day_decay = decay
#'       )
#'
#'     } else if (barrierID %in% c(9,13,14,15,16,17,18,24,26,27,28)) {
#'
#'       update_logreduction_path(
#'         config = config,
#'         matrixID = matrixID,
#'         barrierID = barrierID,
#'         popID = popID
#'       )
#'
#'     } else {
#'       config
#'     }
#'   }
#' )
#'   
#' scenario_logreduction <- scenario_barrier |> 
#'     mutate(log_reduction = map(config, simulate_treatment))
#'
#' regulation_concentration <- config_ambre$regulation$regulation_value |> 
#'   dplyr::filter(Country == "France") |>
#'   dplyr::select(-c(Country, RegulationID, Reduction))
#'
#' regulation_matrix_concentration(scenario = scenario_logreduction,
#'                                 regulation = regulation_concentration)
#' }
regulation_matrix_concentration <- function(scenario, regulation){
  
  if (!"log_reduction" %in% names(scenario)) {
  stop("The log_reduction column is missing from the scenario object; run simulate_treatment function to create it")
}
 
  total_logreduction_regulation <- map(
    scenario$log_reduction,
    ~ .x$events_long |>
        filter(TreatmentID %in% 1:7) |>
        group_by(repeatID, eventID, PathogenGroup) |>
        summarise(total_logreduction = round(sum(logreduction),0), .groups = "drop")
  )
    
  scenario$log_reduction_regulation <- total_logreduction_regulation
  
  
  regulation_evaluation <- map2(.x= scenario$inflow_concentration, 
                                .y = scenario$log_reduction_regulation,
                                .f= ~left_join(.x$events, .y, by = c("repeatID", "eventID", "PathogenGroup")) |>
                          mutate(effluent = 10^(log10(inflow) - total_logreduction))
                          )
    
  
  scenario$regulation <- regulation_evaluation  
  
  abattement_concentration <- scenario |> select(CropName, regulation)
  reductionC_achieve <- map(
    abattement_concentration$regulation,
    ~ .x %>%
      group_by(PathogenGroup, PathogenName) %>%
      summarise(
        max_effluent = round(max(effluent, na.rm = TRUE),0),
        .groups = "drop"
      )
  )
  pathogen_group_C <- map(.x = reductionC_achieve, .f = ~.x$PathogenGroup)
  pathogen_name_C <- map(.x = reductionC_achieve, .f = ~.x$PathogenName)
  concentration_effluent <- map(.x = reductionC_achieve, .f = ~.x$max_effluent)
  
  abattement_concentration$PathogenGroup <- pathogen_group_C
  abattement_concentration$PathogenName <- pathogen_name_C
  abattement_concentration$reductionC_achieve <- concentration_effluent
  
  abattementC <- abattement_concentration |> select(-regulation) |> 
                                              unnest(c(PathogenGroup, PathogenName, reductionC_achieve)) |> 
                                              group_by(PathogenGroup, PathogenName, CropName) %>%
                                                summarise(
                                                  max_reductionC_achieve = round(max(reductionC_achieve, na.rm = TRUE),0),
                                                  .groups = "drop"
                                                )
  abattementC_class <- abattementC |> 
    left_join(config_ambre$regulation$regulation_crop, by = "CropName")
  
  
  abattementC_class_regulation <- abattementC_class |> 
    left_join(regulation, by= c("Class","PathogenGroup")) |>
    unique() |>
    rename(concentration_regulation = Concentration)|>
    rename(target_class = Class) |>
                          mutate(color = case_when(
                                           max_reductionC_achieve > concentration_regulation ~ "#e74c3c",
                                            max_reductionC_achieve <= concentration_regulation ~ "#2ecc71",   # vert clair
                                             TRUE         ~ "#bdbdbd"
                                          ))
  tableau_valeurC <- abattementC_class_regulation |>
    select(CropName, PathogenName, max_reductionC_achieve, target_class) |>
    rename("Crop name" = CropName) |> 
    rename("Target water class" = target_class) |>
    pivot_wider(names_from=PathogenName, values_from = max_reductionC_achieve)
  
  tableau_couleurC <- abattementC_class_regulation |>
    select(CropName, PathogenName, color, target_class) |>
    rename("Crop name" = CropName) |> 
    rename("Target water class" = target_class) |>
    pivot_wider(names_from=PathogenName, values_from = color)
  
  
  tableau_nameC <- tableau_valeurC |> select(-c("Crop name", "Target water class"))
  colonnes_pathogenesC <- names(tableau_nameC)
   
  liste_formatsC <- setNames(
    lapply(colonnes_pathogenesC, function(col) {
      format_color(tableau_couleurC[[col]])
    }),
    colonnes_pathogenesC
  )
  
  table <- formattable(tableau_valeurC, liste_formatsC)
  return(table)
    
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Updated frequency with desired value
#' 
#' Update the value of the frequency in the simulated configuration according to a value defined by the user
#' @param scenario data.frame
#' @param frequency numeric, list of max frequency to simulate
#' @importFrom purrr map2
#' @importFrom dplyr mutate
#' @return scenario_updated
#' 
#' @export
#' @examples
#' scenario_example <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' scenario_example_updated <- update_frequency_with_desired_value(scenario = scenario_example, 
#'                                                                 frequency = c(2L,3L))
#' print(scenario_example_updated$config[[1]]$exposure)
#' print(scenario_example_updated$config[[2]]$exposure)
update_frequency_with_desired_value <- function(scenario, frequency){
  if (length(frequency) != length(scenario$config)){
    stop("A frequency value is required for all lines in the scenario")
  }
  
  if (!is.integer(frequency)) {
  stop("'frequency' must be an integer vector (e.g. c(1L, 2L, 3L)).")
}

  scenario_updated <- scenario |>
    mutate(
      config = map2(
        .x = config,
        .y = frequency,
        .f = ~{
          .x$exposure$value[[2]] <- .y  
          .x                          
        }
      )
    )
  return(scenario_updated)
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("update_frequency_with_desired_value works", {
  scenario_example <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  result <- update_frequency_with_desired_value(scenario = scenario_example, 
                                                                frequency = c(2L,3L))
  expect_equal(object = result$config[[1]]$exposure$value[[2]], 
               expected = 2) 
  expect_equal(object = result$config[[2]]$exposure$value[[2]], 
               expected = 3)
})

test_that("update_frequency_with_desired_value error", {
  scenario_example <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  
  expect_error(object = update_frequency_with_desired_value(scenario = scenario_example, 
                                                                frequency = 2), 
               regexp = "A frequency value is required for all lines in the scenario") 
  expect_error(object = update_frequency_with_desired_value(scenario = scenario_example, 
                                                                frequency = c(2.2,3)), 
               regexp = "'frequency' must be an integer vector (e.g. c(1L, 2L, 3L)).",
               fixed = TRUE)
})

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

#' Format color
#' 
#' Format color for matrix regulation table
#' @param couleurs A character vector of CSS colours (one per cell) used as the cell background.
#' @import formattable
#' @return A formattable `formatter` (span) that renders a coloured, centered cell.
#'
#' @noRd
format_color <- function(couleurs){
  formatter(
      "span",
      style = x ~ style(
        display            = "block",
        "border-radius"    = "4px",
        "padding"          = "4px",
        "background-color" = couleurs,
        color              = "white",
        "text-align"       = "center",
        "font-weight"      = "bold"
      )
  )
}

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

#' Plot log reduction
#' 
#' Boxplot of log reduction for each barrier according to population exposed and pathogen Name
#' @param scenario with log reduction column obtained with simulate_treatment function
#' @importFrom tibble tibble
#' @importFrom purrr map
#' @importFrom tidyr unnest
#' @import ggplot2
#' @import colorspace
#' @return plots
#' 
#' @export
#' @examples
#' library(purrr)
#' library(tidyr)
#' library(dplyr)
#'
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#'
#' scenario_conc <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni", "Norovirus"))
#'
#' scenario_volume <-
#'   scenario_conc |>                     
#'   mutate(
#'     volume = map(
#'       .x = config,                                         
#'       .f = ~ simulate_exposure(config = .x)              
#'     )
#'   )
#'
#' scenario_dose_ini <- initial_dose_calculation(scenario_volume)
#'
#' scenario_scheme <- update_treatment_scheme(scenario_dose_ini)
#'
#' scenario_with_logreduc_and_co <- scenario_scheme |> 
#'   mutate( log_reduction = map(config, simulate_treatment))
#'
#' plot_logreduction(scenario_with_logreduc_and_co)
plot_logreduction <- function(scenario){
  plot_log <- tibble(
        logReduction = map(
              scenario$log_reduction,
              ~ .x$events_long$logreduction
              ),
        PopulationName = scenario$PopulationName,
        PathogenGroup = map(
              .x = scenario$log_reduction,
              .f = ~ .x$events_long$PathogenGroup
              ),
        CropName = scenario$CropName,
        TreatmentName = map(scenario$log_reduction, 
                            ~.x$events_long$TreatmentName
                            )
        )
  
  plot_log_long <- plot_log |> unnest(c(logReduction, TreatmentName, PathogenGroup))
  
  n_treat <- length(unique(plot_log_long$TreatmentName))
  
  plots <- plot_log_long |>
    split(~ PathogenGroup) |>
    imap(function(df, pathogen){

      ggplot(df, aes(x = PopulationName, y = logReduction, fill = TreatmentName)) +
    geom_boxplot()+
    facet_grid(~CropName)+
    scale_fill_manual(
      values = rainbow_hcl(n_treat)
    ) +
    theme_minimal() +
    labs(title = pathogen, 
         x = "Population exposed",
        y = "log reduction",
        fill = "Barrier name")+
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))
    })

  
  return(plots)
    
}

# WARNING - Generated by {fusen} from dev/flat_simulate_treatment.Rmd: do not edit by hand # nolint: line_length_linter.

#' Update treatment scheme
#' 
#' Update treatment scheme list in config data frame according to scenario define
#' @param scenario created with function create_scenario
#' @param initial_situation boolean to retrieve simulate treatment train if TRUE or from Barrier train if FALSE
#' @importFrom purrr pmap
#' @return updated_scenario
#' @export
#' @examples
#' scenario_test <- create_scenario(
#'   system.file("input_cas_apprentissage_complet.xlsx", package = "ambre")
#' )
#' scenario_ini <- update_treatment_scheme(scenario_test, initial_situation = TRUE)
#' scenario_supp <- update_treatment_scheme(scenario_test, initial_situation = FALSE)
update_treatment_scheme <- function(scenario, initial_situation = TRUE){
 
  updated_scenario <- scenario
  
  if (initial_situation == TRUE){
    
    updated_scenario$config <- purrr::pmap(
        updated_scenario[c("config", "InitialTrainID", "InitialTrainName")],
        \(config, InitialTrainID, InitialTrainName) {
      
          config$treatment$schemes <- tibble(
            TreatmentSchemeID   = seq_len(length(InitialTrainID)),
            TreatmentSchemeName = InitialTrainName,
            TreatmentName       = InitialTrainName,
            TreatmentID         = InitialTrainID
          )
      
          config
        }
      )
  }
    
  else {
    updated_scenario$config <- purrr::pmap(
        updated_scenario[c("config", "SupplementaryTrainID", "SupplementaryTrainName")],
        \(config, SupplementaryTrainID, SupplementaryTrainName) {
      
          config$treatment$schemes <- tibble(
            TreatmentSchemeID   = seq_len(length(SupplementaryTrainID)),
            TreatmentSchemeName = SupplementaryTrainName,
            TreatmentName       = SupplementaryTrainName,
            TreatmentID         = SupplementaryTrainID
          )
      
          config
        }
      )
    
    
  }
  
  return(updated_scenario)
}

# WARNING - Generated by {fusen} from dev/flat_simulate_treatment.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("update_treatment_scheme works", {
  scenario_test <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  result <- update_treatment_scheme(scenario_test, initial_situation = TRUE)
  
  expect_true("InitialTrainName" %in% colnames(scenario_test))
  
  expect_equal(object = colnames(result$config[[1]][["treatment"]][["schemes"]]),
               expected = c("TreatmentSchemeID", 
                            "TreatmentSchemeName", 
                            "TreatmentName",
                            "TreatmentID"))
  
  result <- update_treatment_scheme(scenario_test, initial_situation = FALSE)
  
  expect_true("SupplementaryTrainName" %in% colnames(scenario_test))
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Simulate: inflow
#'
#' This function does the same thing as kwb.utils::simulate_inflow(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' Simulate pathogen concentrations
#' @param config list contained in the scenario object
#' @param debug print debug information (default: TRUE)
#' @param lean if \code{TRUE}, only the "events" are returned (in a reduced
#'   version, i.e. without column \code{PathogenName} and with all ID columns
#'   being of class \code{integer}), otherwise a list with the events in element
#'   \code{events}" and the corresponding parameters in element \code{paras}.
#'   The default is \code{FALSE}, i.e. events and parameters are returned in a
#'   list.
#' @importFrom plyr rbind.fill
#' @return list with parameters of user defined random distribution and
#'   corresponding values
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @export
#' @examples
#' # Example usage of simulate_inflow
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#' scenario_updated <- update_pathogen(scenario = scenario_test,
#'  pathoName = c("Campylobacter jejuni", "Rotavirus"))
#'
#' # Call simulate_inflow with the sample config
#' result_1 <- simulate_inflow(config = scenario_updated$config[[1]], debug = FALSE)
#'
#' # Print the results
#' cat("Parameters used for simulation:")
#' print(result_1$paras)
#'
#' # Example with lean = TRUE
#' lean_result_1 <- simulate_inflow(config = scenario_updated$config[[1]], debug = FALSE, lean = TRUE)
#' cat("Lean result (only events):")
#' print(lean_result_1)
simulate_inflow <- function(config, debug = TRUE, lean = FALSE){
  # Only pathogens to be simulated
  patho_ids <- config$inflow$PathogenID[config$inflow$simulate == 1]

  # Helper function to select a row from config$inflow
  get_inflow_config_row <- function(patho_id) {
    inflow_config <- config$inflow[config$inflow$PathogenID == patho_id, ] 
    catIf(debug, sprintf(
      "Simulated pathogen: %s\n", inflow_config$PathogenName
    ))
    inflow_config
  }

  random_objects <- lapply(patho_ids, function(patho_id) {

    inflow_config_row <- get_inflow_config_row(patho_id)

    lapply(
      generate_random_values(
        config = inflow_config_row,
        number_of_repeatings = number_of_repeatings(config),
        number_of_events = number_of_exposures(config),
        debug = debug
      ),
      cbind,
      inflow_config_row[, c("PathogenID", "PathogenName", "PathogenGroup")]
    )
  })

  events <- catAndRun(
    "Providing inflow events",
    do.call(rbind, lapply(random_objects, "[[", "events")) |>
      renameColumns(list(values = "inflow"))
  )

  if (lean) return(
    events |>
      removeColumns("PathogenName") |>
      id_columns_to_integer()
  )

  list(
    events = events,
    paras = catAndRun(
      "Providing inflow paras",
      do.call(plyr::rbind.fill, lapply(random_objects, "[[", "paras"))
    )
  )
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("simulate_inflow works", {
  scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
  scenario_updated <- update_pathogen(scenario = scenario_test, pathoName = c("Campylobacter jejuni"))

  result_1 <- simulate_inflow(config = scenario_updated$config[[1]], debug = FALSE)

  expect_equal(object = result_1$paras,
               expected = structure(list(type = "uniform", repeatings = 1000, events = 365,
    min = 100, max = 5000, PathogenID = 1, PathogenName = "Campylobacter jejuni",
    PathogenGroup = "Bacteria"), class = "data.frame", row.names = c(NA,
-1L)))
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Path Query
#' 
#' Query to return path ID from path Name using the database
#' @param config list configuration file default value is config_ambre dataset
#' @param pathName character Name of the path for which you want to find the ID
#' @importFrom dplyr filter pull
#' @return pathID integer ID of the population define
#' 
#' @export
#' @examples
#' query_exp_path(pathName = "Consumption of the final product potatoes")
query_exp_path <- function(config = config_ambre, pathName){
  allowed_path <- c("Ingestion of water droplets during maintenance of the irrigation system",
                      "Ingestion of water droplets during a power outage",
                      "Inhalation of aerosols while moving through fields during irrigation",
                      "Ingestion following direct contact with a part of the plant during crop maintenance, followed by hand-to-mouth contact",
                      "Ingestion following direct contact with the soil during crop maintenance, followed by hand-to-mouth contact",
                      "Ingestion following direct contact with a part of the plant during harvesting, followed by hand-to-mouth contact",
                      "Ingestion following direct contact with the ground during harvesting, followed by hand-to-mouth contact",
                      "Ingestion of water droplets during irrigation near residential areas",
                      "Inhalation of aerosols during irrigation near residential areas",
                      "Ingestion of water droplets while moving near irrigated areas",
                      "Inhalation of aerosols while driving near irrigated areas",
                      "Consumption of the final product potatoes",
                      "Consumption of the final product onion",
                      "Consumption of the final product apple",
                      "Consumption of the final product salad",
                      "Consumption of the final product tomato",
                      "Consumption of the final product surface and leafy (e.g., salad)",
                      "Consumption of the final product surface, not leaf (e.g., tomato)",
                      "Consumption of the final product through the soil roots")
  if (any(!pathName %in% allowed_path)) {
    stop("Path name doesn't exist")}
  
  pathID <- config_ambre$path$description |> filter(PathDescription %in% pathName) |> 
                                          pull(PathID)
  matrixID <- config_ambre$path$description |> filter(PathDescription %in% pathName) |> 
                                          pull(MatrixID)
  result <- list(PathID = pathID, MatrixID = matrixID)
  return(pathID)

}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("query_exp_path works", {
  expect_equal(object = query_exp_path(pathName = "Consumption of the final product potatoes"),
               expected = 12)
})

test_that("query_path errors", {
  expect_error(object = query_exp_path(pathName = "Consumption of the final product"),
               regexp = "Path name doesn't exist")
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Default Max
#' 
#' This function does the same thing as kwb.utils::default_max(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' @param org_max original maximum value 
#' @param new_max new maximum value for replacement
#' @param f function apply on "org_max" to transform to the correct 
#' dimension (e.g. "log" in case of "lognorm" or "log10" in case of log10_norm) 
#' (default: c())
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' @examples
#' default_max(org_max = 2, new_max = 0.01, f = log10)
#' default_max(org_max = 0, new_max = 0.01, f = log10)
#' default_max(org_max = 2, new_max = 0.01, f = log)
#' default_max(org_max = 0, new_max = 0.01, f = log)
default_max <- function(org_max, new_max, f = c)
{
  ifelse(org_max > 0, f(org_max), f(new_max))
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("default_max works correctly", {
  # Test with org_max > 0
  expect_equal(default_max(2, 0.01, f = log10), log10(2))

  # Test with org_max == 0
  expect_equal(default_max(0, 0.01, f = log10), log10(0.01))

  # Test with no transformation function
  expect_equal(default_max(2, 0.01), 2)

  # Test with log transformation
  expect_equal(default_max(2, 0.01, f = log), log(2))

  # Test with org_max == 0 and no transformation function
  expect_equal(default_max(0, 0.01), 0.01)
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Percentage irrigation need
#' 
#' Calculate percentage irrigation need for each culture according to total irrigation need
#' @param scenario data.frame with irrigation_need_m3an column obtained with irrigation_need_calculation function
#' @importFrom dplyr mutate
#' @return (scenario_updated)
#' 
#' @export
#' @examples
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
#'
#' irrigation_pct <- percentage_irrigation_need(irrigation_need)
#' print(irrigation_pct$irrigation_need_percentage)
percentage_irrigation_need <- function(scenario){
  total_irrigation_need <- total_irrigation_need(scenario)
  scenario_updated <- scenario |> mutate("irrigation_need_percentage" = round(irrigation_need_m3an / total_irrigation_need * 100))
  
  return(scenario_updated)
    
}

#' config_ambre
#'
#' A structured database containing the parameters and values needed to perform Quantitative Microbial Risk Assessment (QMRA) and cost calculations.
#'
#' @format A list of 3 lists and 5 tibbles :
#' \describe{
#'   \item{ exposure }{  Exposure condition : number of repeatings for Monte Carlo simulation, number of exposures per year, volume (L) per exposure event }
#'   \item{ inflow }{  Pathogen concentration (number of pathogen per L) }
#'   \item{ treatment }{  Database of log reductions for risk management measures }
#'   \item{ processes }{  General values for the log reductions }
#'   \item{ schemes }{  Risk management process (sequence of measures) assessed }
#'   \item{ barrier_path }{  Specific values for log reductions based on the assessed route of exposure }
#'   \item{ barrier_specific }{  Specific values for log reductions based on the assessed measure }
#'   \item{ barrier_decay }{  Specific values for log reductions for the natural die off measure }
#'   \item{ doseresponse }{  Parameters for dose response model (https://qmrawiki.org/framework/dose-response/highest-quality-models-and-parameters) }
#'   \item{ path }{  Details on pathways of exposure }
#'   \item{ description }{  Description of pathways of exposure }
#'   \item{ frequency }{  Frequency values of pathways of exposure }
#'   \item{ volume }{  Volume values of pathways of exposure }
#'   \item{ crop }{  Description of crop }
#'   \item{ health }{ Parameters for risk characterization  }
#'   \item{ economic }{  Parameters for economic analysis }
#'   \item{ water_need }{  Value of irrigation need according to crop type }
#'   \item{ population }{  Description of population exposed }
#'   \item{ cost }{  Database of opex and capex cost of risk management measures (https://theses.hal.science/tel-05009670) }
#'   \item{ regulation_value }{  Database of required microorganism removal performance (in terms of concentration or removal rate) by regulatory class and pathogen group }
#'   \item{ regulation_crop }{  Database of required water class acording to crop }
#' }
#' @examples
#' names(config_ambre)
"config_ambre"

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.


#' Rename Columns in a Data Frame
#' 
#' This function does the same thing as kwb.utils::catIf(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' Rename columns in a data frame giving tupels of original name and substitute
#' name as named elements in list "renames"
#' 
#' @param x data.frame
#' @param renamings list with named elements each of which defines a column
#'   rename in the form <old-name> = <new-name>
#' @return \emph{dframe} with columns renamed as specified in \emph{renames}
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' 
#' @examples
#' # Example usage of renameColumns
#' df <- data.frame(
#'   old_name1 = 1:3,
#'   old_name2 = letters[1:3],
#'   old_name3 = c(TRUE, FALSE, TRUE)
#' )
#'
#' # Define renaming rules
#' renamings <- list(
#'   old_name1 = "new_name1",
#'   old_name2 = "new_name2"
#' )
#'
#' # Rename columns
#' renamed_df <- renameColumns(df, renamings)
#'
#' # Print results
#' print("Original data frame:")
#' print(df)
#'
#' print("Renamed data frame:")
#' print(renamed_df)
renameColumns <- function(x, renamings = NULL)
{
  if (is.null(renamings)) {
    
    return(x)
  }
  
  columns <- names(x)
  
  for (column in names(renamings)) {
    
    columns[columns == column] <- renamings[[column]]
  }
  
  structure(x, names = columns)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("renameColumns works correctly", {
  # Create a test data frame
  df <- data.frame(
    old_name1 = 1:3,
    old_name2 = letters[1:3],
    old_name3 = c(TRUE, FALSE, TRUE)
  )

  # Define renaming rules
  renamings <- list(
    old_name1 = "new_name1",
    old_name2 = "new_name2"
  )

  # Rename columns
  renamed_df <- renameColumns(df, renamings)

  # Test that columns were renamed correctly
  expect_equal(names(renamed_df), c("new_name1", "new_name2", "old_name3"))

  # Test that column values remain unchanged
  expect_equal(renamed_df$new_name1, df$old_name1)
  expect_equal(renamed_df$new_name2, df$old_name2)
  expect_equal(renamed_df$old_name3, df$old_name3)

  # Test with NULL renamings
  expect_equal(renameColumns(df, NULL), df)
})

test_that("renameColumns handles non-existent columns", {
  df <- data.frame(a = 1:3, b = letters[1:3])
  renamings <- list(c = "d")

  # Should return original data frame unchanged
  expect_equal(renameColumns(df, renamings), df)
})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Final dose calculation
#' 
#' Calculation of the final dose according to initial dose and log reduction of the treatments simulated
#' @param scenario dataFrame scenario with initial_dose and log_reduction columns
#' @importFrom purrr map2 map
#' @importFrom dplyr mutate group_by summarise
#' @return scenario_with_final_dose
#' 
#' @export
#' @examples
#' library(dplyr)
#' library(purrr)
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#'
#' scenario_conc <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni", "Escherichia coli"))
#'
#' scenario_volume <-
#'   scenario_conc |>                     
#'   mutate(
#'     volume = map(
#'       .x = config,                                         
#'       .f = ~ simulate_exposure(config = .x)              
#'     )
#'   )
#'
#' scenario_dose_ini <- initial_dose_calculation(scenario_volume)
#'
#' scenario_scheme <- update_treatment_scheme(scenario_dose_ini)
#'
#' scenario_with_logreduc_and_co <- scenario_scheme |> 
#'   mutate( log_reduction = map(config, simulate_treatment))
#'
#' final_dose_calculation(scenario_with_logreduc_and_co)
final_dose_calculation <- function(scenario){
  if (!"initial_dose" %in% names(scenario)) {
  stop("The initial_dose column is missing from the scenario object; run the initial_dose_calculation function to create it")
}
  
  if (!"log_reduction" %in% names(scenario)) {
  stop("The log_reduction column is missing from the scenario object; run the simulate_treatment function to create it")
  }
  
  total_logreduction <- map(
  scenario$log_reduction,
  ~ .x$events_long |>
      group_by(repeatID, eventID, PathogenGroup) |>
      summarise(total_logreduction = sum(logreduction), .groups = "drop")
)
  
  scenario_total_logreduction <- scenario |> mutate(total_logreduction = total_logreduction)

  
  ppp <- map2(.x = scenario$inflow_concentration, 
              .y = scenario$volume,
              .f = ~dplyr::left_join(.x$events, .y$volumes$events, by = c("repeatID", "eventID")))
  
  
  ccc <- map2(.x= ppp, 
              .y = total_logreduction, 
              .f= ~left_join(.x, .y, by = c("repeatID", "eventID", "PathogenGroup")))
  
  
  final_dose <- map(.x = ccc , ~{
    .x$final_dose <- (.x$inflow * .x$volume_perEvent)*10^(-.x$total_logreduction)
    .x
  })
  
  scenario_with_final_dose <- scenario_total_logreduction |> mutate(
    final_dose = final_dose)
  
  return(scenario_with_final_dose)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Check for Column Existence
#' 
#' Stops if data frame \emph{frm} does not contain all columns of which the 
#' names are given in \emph{reqCols}.
#' 
#' This function does the same thing as kwb.utils::checkForMissingColumns(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' @param frm data frame
#' @param reqCols vector of names of which existence in \emph{frm} shall be
#'   checked
#' @param do.stop if TRUE, stop() is called else warning() if a column is
#'   missing
#' @param dataFrameName the name of the data frame to be shown in the error
#'   message if a column was missing
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' @return TRUE if all required columns are available, else FALSE
#'   
#' @examples
#'
#' # Example usage of checkForMissingColumns
#' df <- data.frame(a = 1:3, b = letters[1:3], c = c(TRUE, FALSE, TRUE))
#'
#' # Check for existing columns
#' checkForMissingColumns(df, c("a", "b"))  # Should return TRUE
#'
#' # Check for non-existing columns (with warning)
#' checkForMissingColumns(df, c("a", "x"), do.stop = FALSE)
#'
#' # Check for non-existing columns (with error)
#' tryCatch({
#'   checkForMissingColumns(df, c("a", "x"), do.stop = TRUE)
#' }, error = function(e) {
#'   print("Caught expected error:")
#'   print(e$message)
#' })
checkForMissingColumns <- function(
  frm, reqCols, do.stop = TRUE, dataFrameName = deparse(substitute(frm))
)
{
  columnNames <- names(frm)
  
  missing <- reqCols[!(reqCols %in% columnNames)]
  
  if (! isNullOrEmpty(missing)) {
    
    msg <- noSuchElements(
      x = missing, 
      available = columnNames, 
      type = "column",
      sorted = TRUE,
      suffix = sprintf(" in data frame '%s'", dataFrameName)
    )

    do.call(ifelse(do.stop, "stop", "warning"), list(msg, call. = FALSE))
  }
  
  isNullOrEmpty(missing)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)

## ----setup--------------------------------------------------------------------
library(ambre)
set.seed(2024)

## ----signature, eval = FALSE--------------------------------------------------
#  run_economic_analysis(scenario, membership_fee,
#                        price_per_m3, grant, initialSituation = FALSE)

## ----cost-table---------------------------------------------------------------
config_ambre$economic$cost[, c("TreatmentName", "CostType", "value", "unit")] |>
  head(8)

## ----scenario-----------------------------------------------------------------
scenario <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)

## ----run, results = "hide"----------------------------------------------------
plots <- run_economic_analysis(
  scenario,
  membership_fee = 200,   # €/ha/year
  price_per_m3   = 0.1,   # €/m3
  grant          = 0.5,    # half the capital cost is subsidised,
  initialSituation = FALSE # calcul the cost of Supplementary process
)

## ----annual-------------------------------------------------------------------
plots$annual

## ----capex--------------------------------------------------------------------
plots$total_investement

## ----allocation-key-----------------------------------------------------------
plots$allocation_key

## ----run-custom---------------------------------------------------------------

crop <- scenario$CropName
allocation_custom <- data.frame(CropName = crop,
                                allocation = c(0.5, 0.5))
run_economic_analysis(
  scenario,
  membership_fee = 200,   
  price_per_m3   = 0.1,   
  grant          = 0,   
  initialSituation = FALSE, 
  allocation_key = allocation_custom # No subvention, collective treatment price 50% for each crop
)

## ----numbers, results = "hide"------------------------------------------------
scenario_need <- irrigation_need_calculation(scenario)

water_price <- water_price(scenario = scenario_need,
                          price_per_m3 = 0.01,
                          membership_fee = 200)
supplementary_cost <- process_cost_calculation(scenario = scenario_need,
                                              membership_fee = 200,
                                              charge = 0.1,
                                              initialSituation = FALSE)


## ----numbers-cols-------------------------------------------------------------
setdiff(names(water_price), names(supplementary_cost))

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Helper function: gets number of exposures from config
#' 
#' This function does the same thing as kwb.utils::number_of_exposures(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' @param config list config
#' @return number_of_exposure
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd 
#' @examples
#' number_of_exposures(config = config_ambre)
number_of_exposures <- function(config)
{
  # HARD CODED: number_of_exposures MUST be of type value!      
  get_exposure_value_or_stop(config, "number_of_exposures")
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("number_of_exposures works", {
  expect_equal(object = number_of_exposures(config = config_ambre), 
               expected = 365) 
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Message on Available Elements
#'
#' This function does the same thing as kwb.utils::hintAvailable(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#'
#' @param x names of elements that are available
#' @param type type of element to appear in the message. Default: "element"
#' @param sorted logical. Whether or not to print the available elements in
#'   lexical order. Default: \code{TRUE}
#' @param suffix suffix to be appended to \code{type}. Can be used to
#'   distinguish between singular and plural form. Default: \code{""}
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' @examples
#' # Example usage of hintAvailable
#' cat(hintAvailable(c("a", "b", "c"), type = "element"))
#' cat(hintAvailable(c("x", "y"), type = "element", suffix = "s"))
hintAvailable <- function(x, type = "element", sorted = TRUE, suffix = "")
{
  paste0("Available ", enumeration(x, type, sorted, suffix))
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("hintAvailable works correctly", {
  # Test with single element
  expect_equal(
    hintAvailable("x", type = "element"),
    "Available element:\n- 'x'\n"
  )

  # Test with multiple elements
  expect_equal(
    hintAvailable(c("x", "y"), type = "element", suffix = "s"),
    "Available elements (sorted)s:\n- 'x'\n- 'y'\n"
  )

  # Test with sorted = FALSE
  expect_equal(
    hintAvailable(c("b", "a"), type = "item", sorted = FALSE),
    "Available items:\n- 'b'\n- 'a'\n"
  )

  # Test with numeric vector
  expect_equal(
    hintAvailable(1:3),
    "Available elements (sorted):\n- 1\n- 2\n- 3\n"
  )

  # Test with logical vector
  expect_equal(
    hintAvailable(c(TRUE, FALSE)),
    "Available elements (sorted):\n- FALSE\n- TRUE\n"
  )
})

# WARNING - Generated by {fusen} from dev/flat_kwb_infection.Rmd: do not edit by hand # nolint: line_length_linter.

#' Dose-response model: exponential
#' 
#' @param dose vector of dose data (default: 
#'   \code{sfsmisc::lseq(from = 0.1, to = 10^10, length = 1000)})
#' @param k k-value (default: 5.72E-01)
#' @return tibble
#' @export
#' @importFrom sfsmisc lseq
#' @references Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' 
dr.expo <- function(
  dose = sfsmisc::lseq(from = 1, to = 10^10, length = 1000), 
  k = 5.72E-01
)
{
  dplyr::as_tibble(data.frame(
    model = "exponential", 
    dose = dose, 
    infectionProbability = 1 - exp(-k * dose),
    k = k
  ))
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Pathogen update
#' 
#' Update the configuration with list of pathogen to be simulated according to pathogen ID
#' @param scenario list obtain from create_scenario()
#' @param pathoName character name of the pathogens simulated
#' @importFrom dplyr mutate if_else
#' @importFrom purrr map
#' @return scenario_updated list scenario updated with the pathogens to be simulated
#' 
#' @export
#' @examples
#' scenario_test <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#'
#' scenario_updated <- update_pathogen(scenario = scenario_test,
#'  pathoName = c("Campylobacter jejuni", "Rotavirus"))
update_pathogen <- function(scenario, pathoName){
  pathogen_id <- query_pathogen(pathogenName = pathoName)
  
  scenario_updated <- scenario |> mutate(
    config = map(config, ~ {
        .x$inflow$simulate <- if_else(.x$inflow$PathogenID %in% pathogen_id, 1, .x$inflow$simulate)
        .x 
      }
    )
  )
   
  return(scenario_updated)
}

# WARNING - Generated by {fusen} from dev/flat_simulate_exposure.Rmd: do not edit by hand # nolint: line_length_linter.


test_that("simulate_exposure returns expected structure", {
  scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
  scenario_updated <- update_pathogen(scenario = scenario_test,
   pathoName = c("Campylobacter jejuni", "Rotavirus"))
  result <- simulate_exposure(config = scenario_updated$config[[1]])

  expect_is(result, "list")
  expect_type(result$volumes, "list")
  expect_s3_class(result$volumes$events, "data.frame")
  expect_true(all(c("repeatID", "eventID", "volume_perEvent") %in% colnames(result$volumes$events)))
  expect_true(nrow(result$volumes$events) > 0)
})

test_that("simulate_exposure handles debug parameter", {
  scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
  scenario_updated <- update_pathogen(scenario = scenario_test,
   pathoName = c("Campylobacter jejuni", "Rotavirus"))

  # Test with debug=TRUE (catIf uses cat(), so output goes to stdout)
  expect_output(simulate_exposure(config = scenario_updated$config[[1]], debug = TRUE),
                "Simulated exposure: volume per event")

  # Test with debug=FALSE
  expect_silent(simulate_exposure(config = scenario_updated$config[[1]], debug = FALSE))
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("update_concentration works", {
  scenario_test <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)

  pathogenName <- c("Norovirus", "Rotavirus")
  concentration_custom <- data.frame(PathogenName = pathogenName,
                                     min = c(1, 10),
                                     max = c(2,20),
                                     type = c("uniform", "uniform"))
  
  scenario_updated_patho <- update_pathogen(scenario = scenario_test, pathoName = pathogenName)
  result <- update_concentration(scenario = scenario_updated_patho ,
                       concentration = concentration_custom)
  
  result_noro <- dplyr::filter(result$config[[1]]$inflow, PathogenName == "Norovirus")
  
  expect_equal(object = result_noro$min, expected = 1)
  expect_equal(object = result_noro$max, expected = 2)
  
  result_rota <- dplyr::filter(result$config[[1]]$inflow, PathogenName == "Rotavirus")
  expect_equal(object = result_rota$min, expected = 10)
  expect_equal(object = result_rota$max, expected = 20)
  
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("update_pathogen works with", {
  scenario_test <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  scenario_updated <- update_pathogen(scenario = scenario_test,
   pathoName = c("Campylobacter jejuni", "Rotavirus"))
  
  expect_equal(object = scenario_updated$config[[1]]$inflow$simulate[[1]],
               expected = 1)
  expect_equal(object = scenario_updated$config[[1]]$inflow$simulate[[26]],
               expected = 1)
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Pathogen Query
#' 
#' Query to return pathogen ID from pathogen Name using the database
#' @param config list configuration file default value is config_ambre dataset
#' @param pathogenName character Name of the pathogen for which you want to find the ID
#' @importFrom dplyr filter pull
#' @return pathogenID integer ID of the pathogen define
#' 
#' @export
#' @examples
#' query_pathogen(pathogenName = "Campylobacter jejuni")
#'
#' query_pathogen(pathogenName = c("Campylobacter jejuni", "Rotavirus"))
#'
query_pathogen <- function(config = config_ambre, pathogenName) {
  allowed_pathogen <- c(
      "Campylobacter jejuni",
      "Escherichia coli enterohemorrhagic (EHEC)",
      "Escherichia coli",
      "Legionella pneumophila",
      "Listeria monocytogenes (Death as response)",
      "Listeria monocytogenes (Infection)",
      "Listeria monocytogenes (Stillbirths)",
      "Pseudomonas aeruginosa (Contact lens)",
      "Pseudomonas aeruginosa (bacterimia)",
      "Salmonella Typhi",
      "Salmonella anatum",
      "Salmonella meleagridis",
      "Salmonella nontyphoid",
      "Salmonella serotype newport",
      "Shigella",
      "Staphylococcus aureus",
      "Vibrio cholerae",
      "Yersinia pestis",
      "Bacillus anthracis",
      "Burkholderia pseudomallei",
      "Coxiella burnetii",
      "Francisella tularensis",
      "Mycobacterium avium",
      "Rickettsia rickettsi",
      "Norovirus",
      "Rotavirus",
      "Adenovirus",
      "Echovirus",
      "Enteroviruses",
      "SARS",
      "Influenza",
      "Lassa virus",
      "Poliovirus",
      "Rhinovirus",
      "Cryptosporidium parvum",
      "Giardia duodenalis",
      "Endamoeba coli",
      "Naegleria fowleri" )
  
  if (any(!pathogenName %in% allowed_pathogen)) {
    stop("Pathogen name doesn't exist")
  }
  
  pathogenID <- config_ambre$inflow |> filter(PathogenName %in% pathogenName) |> 
                                          pull(PathogenID)
  return(pathogenID)
    
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("query_pathogen works", {
  expect_equal(object = query_pathogen(pathogenName = "Campylobacter jejuni"),
               expected = 1)
})

test_that("query_pathogen error", {
  expect_error(object = query_pathogen(pathogenName = "Wrong Pathogen"),
               regexp = "Pathogen name doesn't exist")
})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Run QMRA on initial situation
#' 
#' Run the full QMRA workflow
#' 1) simulate inflow concentration, volume of exposure 
#' 2) calculate initial dose
#' 3) simulate log reduction for treatment processes
#' 4) calculate final dose
#' 5) calculate infection probability, illness probability and dalys reduction
#' 6) calculate annual total risk
#' 
#' @param scenario data.frame scenario obtained with create_scenario function path of the scenario file
#' @param pathogen character Name of the pathogen to simulate
#' @param regulationLog data.frame regulation value of the log reduction to compare with simulation result
#' @param regulationConcentration data.frame regulation value of the log reduction to compare with simulation result
#' @importFrom purrr map
#' 
#' @return all_plots list of ggplot
#' 
#' @export
#' @examples
#' \donttest{
#' library(dplyr)
#' scenario <- create_scenario(system.file("input_cas_apprentissage_complet.xlsx", package = "ambre"))
#' regulation_reduction <- config_ambre$regulation$regulation_value |> 
#'   dplyr::filter(Country == "France") |>
#'   dplyr::select(-c(Concentration, Country, RegulationID))
#' regulation_concentration <- config_ambre$regulation$regulation_value |> 
#'   dplyr::filter(Country == "France") |>
#'   dplyr::select(-c(Country, RegulationID, Reduction))
#' run_qmra_initial_situation(scenario = scenario, 
#'                    pathogen = c("Campylobacter jejuni", "Escherichia coli"),
#'                    regulationLog = regulation_reduction,
#'                    regulationConcentration = regulation_concentration)
#' }
run_qmra_initial_situation <- function(scenario, pathogen, regulationLog, regulationConcentration){
  
  scenario_concentration <- inflow_concentration(scenario = scenario,
   pathogenName = pathogen)

  scenario_volume <-
    scenario_concentration |>                     
    mutate(
      volume = map(
        .x = config,                                         
        .f = ~ simulate_exposure(config = .x)              
      )
    )

  scenario_dose_ini <- initial_dose_calculation(scenario_volume)
  
  scenario_update_ttt_scheme <- update_treatment_scheme(scenario = scenario_dose_ini, initial_situation = TRUE)
  
  scenario_update_ttt_scheme$config <- purrr::pmap(
  list(
    config = scenario_update_ttt_scheme$config,
    barrierID = scenario_update_ttt_scheme$InitialProcessID,
    cropHeight = scenario_update_ttt_scheme$CropHeight,
    matrixID = scenario_update_ttt_scheme$MatrixID,
    popID = scenario_update_ttt_scheme$PopulationID,
    decay = scenario_update_ttt_scheme$nb_day_decay
  ),
  retrieve_specific_barrier
)
  
  scenario_logreduction <- scenario_update_ttt_scheme |>
    mutate(log_reduction = map(.x = config, .f = ~ simulate_treatment(.x)))
  
  log_reduction_plot <- plot_logreduction(scenario_logreduction)
  
  matrix_log <- regulation_matrix_log(scenario = scenario_logreduction,
                                      regulation = regulationLog)
  
  matrix_concentration <- regulation_matrix_concentration(scenario = scenario_logreduction,
                                                          regulation = regulationConcentration)
  
  scenario_final_dose <- final_dose_calculation(scenario_logreduction)
  
  scenario_inf_proba <- infection_probability_calculation(scenario_final_dose)
  
  scenario_illness_proba <- illness_probability_calculation(scenario_inf_proba)
  
  scenario_dalys <- dalys_calculation(scenario_illness_proba)
  
  scenario_risk_total <-get_risk_total(scenario_dalys)
  
  dalys_plot <- plot_dalys(scenario_risk_total)

  
  all_plots <- list(
    logreduction = log_reduction_plot,
    dalys = dalys_plot,
    regul_matrix_log = matrix_log,
    regul_matrix_concentration = matrix_concentration
  )
  
  return(all_plots)
    
}

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("run_qmra_initial_situation works", {
  scenario <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
  regulation_reduction <- config_ambre$regulation$regulation_value |> dplyr::filter(Country == "France") |>
                          dplyr::select(-c(Concentration, Country, RegulationID))
  regulation_concentration <- config_ambre$regulation$regulation_value |> dplyr::filter(Country == "France") |>
                          dplyr::select(-c(Country, RegulationID, Reduction))
  result <- run_qmra_initial_situation(scenario = scenario, 
                     pathogen = c("Campylobacter jejuni", "Escherichia coli"),
                     regulationLog = regulation_reduction,
                     regulationConcentration = regulation_concentration)
  expect_equal(object = names(result),
               expected = c("logreduction", "dalys", "regul_matrix_log", "regul_matrix_concentration"))
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Collective treatment cost calculation
#' 
#' Calculate cost of the simulated collective treatment train
#' @param scenario data.frame scenario with irrigation_need_m3an column obtained with irrigation_need_calculation
#' @param grant numeric amount of the grant in percentage 
#' @param allocation_key data.frame allocation key, NULL by default
#' @importFrom dplyr mutate filter
#' @return cost
#' 
#' @export
#' @examples
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
#' collective_treatment_cost_calculation(scenario = irrigation_need, 
#'                                       grant = 0.5)
#'
#' crop <- scenario_apprentissage$CropName |> unique()
#' allocation_custom <- data.frame(CropName = crop,
#'                                 allocation = c(0.5, 0.1, 0.1, 0.1, 0.1, 0.1))
#' collective_treatment_cost_calculation(scenario = irrigation_need, 
#'                                       grant = 0,
#'                                       allocation_key = allocation_custom)
collective_treatment_cost_calculation <- function(
  scenario,
  grant,
  allocation_key = NULL
){
  collective_ids <- unique(
    stats::na.omit(
      unlist(
        scenario$CollectiveTreatmentID,
        use.names = FALSE
      )
    )
  )
  if (length(collective_ids) == 0) {
    stop("No collective treatement defined in the scenario simulated")
  }

  if (!is.null(allocation_key)) {
    if (!is.data.frame(allocation_key)) {
      stop("'allocation_key' must be a data.frame.")
    }

    required_cols <- c("CropName", "allocation")
    missing_cols <- setdiff(required_cols, names(allocation_key))

    if (length(missing_cols) > 0) {
      stop(
        "'allocation_key' must contain the following columns: ",
        paste(required_cols, collapse = ", "),
        ". Missing: ",
        paste(missing_cols, collapse = ", ")
      )
    }
  }

  capex_cost <- filter(
    config_ambre$economic$cost,
    TreatmentID %in% collective_ids
  ) |>
    filter(CostType == "capex")

  opex_cost <- filter(
    config_ambre$economic$cost,
    TreatmentID %in% collective_ids
  ) |>
    filter(CostType == "opex")


  cost <- data.frame(
      volume_to_treat = total_irrigation_need(scenario),
      capex = sum(
        capex_cost$value,
        na.rm = TRUE
      ),
      opex = sum(
        opex_cost$value,
        na.rm = TRUE
      )
    ) |>
      mutate(
        investissement = capex * volume_to_treat
      ) |>
      mutate(
        operation_cost = opex * volume_to_treat
  )

  scenario_updated <- scenario
  scenario_updated$collective_opex <- cost$opex * scenario_updated$irrigation_need_m3an
  
# CAPEX
  total_area <- total_area_calculation(scenario = scenario_updated)
  capex_ttt_collectif <- cost$investissement
  
  if (is.null(allocation_key)){
    scenario_updated <- scenario_updated |>
                      mutate("allocation" = round((1 - grant) * Area / total_area, 4))
  } else {
    scenario_updated <- scenario_updated |>
      left_join(allocation_key, by = "CropName")
  }
  
  
  scenario_updated <- scenario_updated |>
                      mutate("capex_collective_treatment" = round(allocation * capex_ttt_collectif, 0)) |>
                      mutate("loan_asa" = capex_collective_treatment/(20*Area)) |>
  mutate(loan_asa = round(loan_asa*Area, 0))
  
  allocation_key <- data.frame(grant = grant,
                               CropName = scenario_updated$CropName,
                               allocation = scenario_updated$allocation) |>
    unique()
  
  result <- list(scenario = scenario_updated,
                 allocation_key = allocation_key)
  
  return(result)
    
}

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Total irrigation need
#' 
#' Calculate total irrigation need according to irrigation need of each culture of the simulated scenario
#' @param scenario data.frame scenario with irrigation_need_m3an column obtained with irrigation_need_calculation function
#' @importFrom dplyr distinct select
#' @return total_irrigation_need
#' 
#' @export
#' @examples
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
#' total_irrigation_need(irrigation_need)
total_irrigation_need <- function(scenario){
  
  total_irrigation_need_df <- scenario |> select(CropID, irrigation_need_m3an) |>  distinct()
  total_irrigation_need <- sum(total_irrigation_need_df$irrigation_need_m3an)
  return(total_irrigation_need)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Shortcut to paste(x, collapse = collapse)
#' 
#' This is just a shortcut to paste(x, collapse = collapse)
#' 
#' This function does the same thing as kwb.utils::collapsed(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#'
#' @param x vector of character
#' @param collapse character string to separate the elements in x (passed to 
#'   \code{\link[base]{paste}})
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' 
#' @examples
#'
#' cat(collapsed(c("apple", "banana", "cherry"), collapse = ", "), "\n")
#' cat(collapsed(c("red", "green", "blue"), collapse = "; "), "\n")
collapsed <- function(x, collapse = " ")
{
  paste(as.character(x), collapse = collapse)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("collapsed works correctly", {
  # Test with default collapse
  expect_equal(collapsed(c("a", "b", "c")), "a b c")

  # Test with custom collapse
  expect_equal(collapsed(c("a", "b", "c"), collapse = ", "), "a, b, c")

  # Test with single element
  expect_equal(collapsed(c("a")), "a")

  # Test with empty vector
  expect_equal(collapsed(character(0)), "")

  # Test with numeric vector
  expect_equal(collapsed(1:3), "1 2 3")

  # Test with logical vector
  expect_equal(collapsed(c(TRUE, FALSE)), "TRUE FALSE")
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Total investment cost
#' 
#' Calculates the total investment required to implement risk management processes relating to the reuse of wastewater for each simulated configuration 
#' @param scenario data.frame scenario obtained with create_scenario function
#' @param price_per_m3 numeric, water price in euro per cubic meter
#' @param membership_fee numeric annual membership fee in euro per hectare
#' @param grant numeric amount of the grant in percentage
#' @param allocation_key numeric allocation_key, NULL by default
#' @return result list
#' 
#' @export
#' @examples
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#'
#' total_investment_cost(scenario = scenario_apprentissage,
#'                       price_per_m3 = 0.1, 
#'                       membership_fee = 200, 
#'                       grant = 0.5)
#'
#'
#' crop <- scenario_apprentissage$CropName
#' allocation_custom <- data.frame(CropName = crop,
#'                                 allocation = c(0.5, 0.5))
#' total_investment_cost(scenario = scenario_apprentissage,
#'                       price_per_m3 = 0.1, 
#'                       membership_fee = 200, 
#'                       grant = 0,
#'                       allocation_key = allocation_custom)
total_investment_cost <- function(scenario, price_per_m3, membership_fee, grant, allocation_key = NULL){
  irrigation_need <- irrigation_need_calculation(scenario)

# 1. Water price

irrigation_need_with_waterprice <- water_price(scenario = irrigation_need,
            price_per_m3 = price_per_m3,
            membership_fee = membership_fee)


# 2. Mesure collective
if (all(!is.na(irrigation_need_with_waterprice$CollectiveTreatmentID))) {
    if (is.null(allocation_key)){
      irrigation_need_with_collectivecost <- collective_treatment_cost_calculation(
                        scenario = irrigation_need_with_waterprice,
                        grant = grant
                        )
    } else {
      irrigation_need_with_collectivecost <- collective_treatment_cost_calculation(
                        scenario = irrigation_need_with_waterprice,
                        grant = grant,
                        allocation_key = allocation_key)
    }
  
} else {
  irrigation_need_with_collectivecost <- list(scenario = irrigation_need_with_waterprice,
                                                    allocation_key = NA)
  irrigation_need_with_collectivecost$scenario <-
  irrigation_need_with_collectivecost$scenario %>%
  mutate(
    capex_collective_treatment = NA,
    collective_opex = NA,
    loan_asa = NA
  )
}


# 3. Mesure individuelle : opex et 
irrigation_need_with_mesurecost <- process_cost_calculation(scenario = irrigation_need_with_collectivecost$scenario, 
                                                            membership_fee = membership_fee, 
                                                            charge = price_per_m3,
                                                            initialSituation = FALSE)
total_investment <- irrigation_need_with_mesurecost |>
  select(c(CropName, capex_supplementaryProcess, capex_collective_treatment)) |>
  rename("Investment collective process" = capex_collective_treatment,
         "Investment individual process" = capex_supplementaryProcess) |>
  pivot_longer(cols = !CropName, names_to = "cost_type", values_to = "value") |>
  unique()

  plot <- ggplot(total_investment, aes(x = CropName, 
                                 y = value, 
                                 fill = cost_type, 
                                 label = value))+
  geom_bar(stat = "identity") +
  geom_text(size = 3, position = position_stack(vjust = 0.5)) +
  scale_fill_manual(
    values = c(
      "Investment collective process" = "#d28433",
      "Investment individual process" = "#167779"
    )
  )+ 
  labs(fill = "Cost Type", y = "Total cost (EURO)") + 
  theme_minimal() + 
  theme(legend.position = "bottom")
  
  result <- list(plot_total_cost = plot,
                 allocation_key = irrigation_need_with_collectivecost$allocation_key)
  return(result)
    
}

# WARNING - Generated by {fusen} from dev/flat_kwb_infection.Rmd: do not edit by hand # nolint: line_length_linter.

#' Get dose response values
#' 
#' Result from beta Poisson or exponential models
#' 
#' @param dose numeric dose_perEvent values calculate with final_dose_calculation function
#' @param k numeric survival probability
#' @param alpha numeric derived from survival probability
#' @param n50 numeric dose at which 50% of the population is expected to be affected.
#' @param pathogen_name name of the pathogen simulated
#' @references Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
get_dose_response_values <- function(dose, k, alpha, n50, pathogen_name)
{
  stopifnot(length(k) == 1, length(alpha) == 1, length(n50) == 1)
  
  if (is.na(k) && ! is.na(alpha) && ! is.na(n50)) {
    
    dr.betapoisson(dose = dose, alpha = alpha, N50 = n50)
    
  } else if (! is.na(k) & is.na(alpha) & is.na(n50)) {
    
    dr.expo(dose = dose, k = k)
    
  } else {
    
    stop(sprintf(
      get_stop_text("doseresponse_config_incomplete"), pathogen_name
    ))
  }
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Helper function: get percentile 
#' 
#' This function does the same thing as kwb.utils::get_percentile(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' @param percent_within_minmax percent of data point within min/max (default: 
#' 0.9 i.e. 90 percent)
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' @examples
#' get_percentile(0.9)
#' get_percentile(0.95)
get_percentile <- function(percent_within_minmax = 0.9)
{
  qnorm(percent_within_minmax + (1 - percent_within_minmax) / 2)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.


test_that("get_percentile works correctly", {
  # Test with default parameter
  expect_equal(get_percentile(), qnorm(0.9 + (1 - 0.9) / 2))

  # Test with custom parameter
  expect_equal(get_percentile(0.95), qnorm(0.95 + (1 - 0.95) / 2))

  # Test with edge case (0)
  expect_equal(get_percentile(0), qnorm(0 + (1 - 0) / 2))

  # Test with edge case (1)
  expect_equal(get_percentile(1), qnorm(1 + (1 - 1) / 2))
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Query crop height
#' 
#' Retrieve height of the crop simulated
#' @param cropName integer, identifier of the crop simulated
#' @return height
#' 
#' @export
#' @examples
#' query_crop_height("Tomato")
query_crop_height <- function(cropName){
  allowed_crop <- c("Corn seed", "Potato", "Tomato", "Onion", "Apple", "Salad")
  if (any(!cropName %in% allowed_crop)){
    stop("Crop name doesn't exist")}

  height <- config_ambre$crop |>
    filter(CropName %in% cropName) |>
    pull(CropHeight) 
    return(height)
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("query_crop_height works", {
  expect_equal(object = query_crop_height("Tomato"),
               expected = 25)
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Volume update
#' 
#' Update the configuration with min and max volumes to be simulated according to path ID
#' @param scenario dataframe scenario obtained with function create_scenario
#' @importFrom dplyr mutate
#' @importFrom purrr map2 map
#' @return scenario_updated list configuration updated with the min and max volumes to be simulated
#' 
#' @export
#' @examples
#' scenario_example <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' scenario_updated <- update_volume(scenario_example)
#' scenario_updated$config[[1]]$exposure
update_volume <- function(scenario) {
  
  new_volume <- map(.x = scenario$PathID, .f = ~ query_volume(.x))   # $min et $max

  scenario_updated <- scenario |>
    mutate(
      config = map2(
        .x = config,
        .y = new_volume,
        .f = ~{
          .x$exposure$min[[3]] <- .y$min
          .x                          
        }
      )
    ) |>
    mutate(
      config = map2(
        .x = config,
        .y = new_volume,
        .f = ~{
          .x$exposure$max[[3]] <- .y$max
          .x
        }
      )
    )
  return(scenario_updated)
}

# WARNING - Generated by {fusen} from dev/flat_simulate_treatment.Rmd: do not edit by hand # nolint: line_length_linter.

#' Calculate Wide-Format Treatment Scheme Events
#' 
#' This function and returns the results in a wide-format data frame.
#' 
#' This function does the same thing as kwb.qmra::get_scheme_events_wide(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' @param config A configuration object containing treatment scheme information.
#' @param treatment_events_wide A data frame in wide format containing treatment events
#'   with columns for each treatment ID and log-reduction values.
#' 
#' @importFrom stats setNames
#' 
#' @return A data frame with columns for event ID, pathogen group, and one column
#'   for each treatment scheme containing the sum of log-reduction values for that scheme.
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @export
#' @examples
#' # Example usage of get_scheme_events_wide
#' scenario_test <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' scenario_updated <- update_pathogen(scenario = scenario_test,
#'  pathoName = c("Campylobacter jejuni", "Rotavirus"))
#' config <- scenario_updated$config[[1]]
#'
#' # Simulate treatment data
#' treatment_data <- get_treatment_data(config$treatment$processes, n_repeatings = 5, n_events = 3)
#'
#' treatment_events_wide <- treatment_data$events |> tidyr::spread(
#'       key = TreatmentID,
#'       value = logreduction
#'     )
#'
#' # Apply get_scheme_events_wide
#' scheme_events_wide <- get_scheme_events_wide(config, treatment_events_wide)
#'
#' # Print results
#' print(scheme_events_wide)
get_scheme_events_wide <- function(config, treatment_events_wide){
  schemes <- config$treatment$schemes
  
  schemeIDs <- unique(schemes$TreatmentSchemeID)
  
  schemes_events_wide <- data.frame()

  for (schemeID in schemeIDs) {
    
    is_scheme <- schemes$TreatmentSchemeID == schemeID
    
    scheme_treatmentIDs <- unique(schemes$TreatmentID[is_scheme])
    
    event_row_sums <- rowSums(
      treatment_events_wide[, as.character(scheme_treatmentIDs), drop = FALSE]
    )

    if (schemeID == schemeIDs[1]) {
      
      schemes_events_wide <- cbind(
        treatment_events_wide[, c("eventID", "PathogenGroup")],
        event_row_sums,
        row.names = NULL
      )
      
    } else {
      
      schemes_events_wide <- cbind(
        schemes_events_wide,
        event_row_sums,
        row.names = NULL
      )
    }
  }
  
  stats::setNames(schemes_events_wide, c(
    "eventID", "PathogenGroup", paste0("scheme_", schemeIDs)
  ))
}

# WARNING - Generated by {fusen} from dev/flat_simulate_treatment.Rmd: do not edit by hand # nolint: line_length_linter.

#' Simulate Treatment
#'
#' This function simulates treatment data for pathogen processes based on the provided configuration.
#' It generates treatment events and parameters, and can return results in either long or wide format.
#' 
#' This function does the same thing as kwb.qmra::simulate_treatment(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#'
#' @param config A configuration object containing treatment scheme and process information.
#' @param wide Logical indicating whether to return results in wide format.
#' @param debug Logical indicating whether to print debug information.
#' @param lean Logical indicating whether to return only essential results for web applications.
#'
#' @return If `lean` is TRUE, returns a list containing only the long-format treatment events.
#'   If `wide` is TRUE, returns a list containing long-format events, wide-format events,
#'   wide-format scheme events, treatment schemes, and treatment parameters.
#'   If `wide` is FALSE, returns a list containing long-format events, treatment schemes,
#'   and treatment parameters.
#' 
#' @importFrom dplyr filter left_join select group_by slice
#'
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @export
#' @examples
#' # Example usage of simulate_treatment
#' \donttest{
#' scenario_test <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' scenario_updated <- update_pathogen(scenario = scenario_test,
#'  pathoName = c("Campylobacter jejuni","Rotavirus"))
#' config <- scenario_updated$config[[1]]
#'
#' # Simulate treatment data
#' result <- simulate_treatment(config, wide = TRUE, debug = FALSE, lean = FALSE)
#'
#' }
simulate_treatment <- function(
  config, wide = FALSE, debug = TRUE, lean = FALSE
){
  inflow_config_simulated <- config$inflow |>
    dplyr::filter(.data$simulate == 1)

  schemes_config <- config$treatment$schemes

  processes_simulated <- config$treatment$processes |>
    dplyr::filter(
      .data$PathogenGroup %in% unique(inflow_config_simulated$PathogenGroup),
      .data$TreatmentID %in% unique(schemes_config$TreatmentID)
    )
  
  treatment_data <- get_treatment_data(
    processes = processes_simulated, 
    n_repeatings = number_of_repeatings(config), 
    n_events = number_of_exposures(config), 
    debug = debug,
    include_paras = ! lean
  )

  treatment_events <- treatment_data$events |>
    dplyr::left_join(schemes_config)
  
  # Return only the result that is required by the web app if "lean" is TRUE
  if (lean) {
    
    return(list(events_long = treatment_events))
  } 
  
  if (wide) {
    
    treatment_events_wide <- tidyr::spread(
      data = treatment_events,
      key = .data$TreatmentID,
      value = .data$logreduction
    )
    
    schemes_events_wide <- get_scheme_events_wide(config, treatment_events_wide)
  }
  
  # Create and return further results only if "lean" is FALSE
  treatment_id_to_name <- config$treatment$processes |>
    dplyr::select(.data$TreatmentID, .data$TreatmentName) |>
    dplyr::group_by(.data$TreatmentID) |> 
    dplyr::slice(1)
  
  treatment_paras <- treatment_data$paras |>
    dplyr::left_join(treatment_id_to_name)
  
  if (wide) list(
    
    events_long = treatment_events,
    events_wide = treatment_events_wide,
    schemes_events_wide = schemes_events_wide,
    schemes = schemes_config,
    paras = treatment_paras
    
  ) else list(
    
    events_long = treatment_events,
    schemes = schemes_config,
    paras = treatment_paras
  )
}

# WARNING - Generated by {fusen} from dev/flat_simulate_treatment.Rmd: do not edit by hand # nolint: line_length_linter.

#' Update log reduction with specific value
#' 
#' Update log reduction value according to treatment, crop, and population simulated
#' @param config list configuration simulated
#' @param cropHeight numeric, height of th crop simulated
#' @param barrierID numeric, ID of the barrier simulated
#' @importFrom dplyr filter mutate
#' @return config_updated
#' 
#' @export
#' @examples
#' scenario_example <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#'
#' scenario_example$config <- purrr::pmap(
#'   list(
#'     config = scenario_example$config,
#'     barrierID = scenario_example$SupplementaryProcessID,
#'     matrixID = scenario_example$MatrixID,
#'     cropHeight = scenario_example$CropHeight
#'   ),
#'   function(config, barrierID, matrixID, cropHeight) {
#'
#'     if (barrierID %in% c(24, 26, 27, 28) &&
#'         matrixID %in% c(3, 5)) {
#'
#'       update_logreduction_specific_barrier(
#'         config = config,
#'         cropHeight = cropHeight,
#'         barrierID = barrierID
#'       )
#'
#'     } else {
#'       config
#'     }
#'   }
#' )
update_logreduction_specific_barrier <- function(config, cropHeight, barrierID){
  reduc_log_spe <- filter(config_ambre$treatment$barrier_specific, 
                                   TreatmentID %in% barrierID,
                                    CropHeight %in% cropHeight)
  
  barrier_modif <- filter(
    config$treatment$processes,
    TreatmentID %in% reduc_log_spe$TreatmentID
  )
  
  
  barrier_modif_updated <- mutate(
    barrier_modif,
    min = reduc_log_spe$min,
    max = reduc_log_spe$max
  )
  
  config_updated <- config
  processes <- config_updated$treatment$processes
  
  idx <- match(
        paste(processes$TreatmentID, processes$PathogenGroup),
        paste(barrier_modif_updated$TreatmentID, barrier_modif_updated$PathogenGroup)
      )
  
  sel <- !is.na(idx)
  
  processes$min[sel] <- barrier_modif_updated$min[idx[sel]]
  processes$max[sel] <- barrier_modif_updated$max[idx[sel]]
  
  config_updated$treatment$processes <- processes
  return(config_updated)
}

# WARNING - Generated by {fusen} from dev/flat_simulate_treatment.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("update_logreduction_specific_value works", {
  scenario_example <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  result <- purrr::pmap(
                list(
                  config = scenario_example$config,
                  barrierID = scenario_example$SupplementaryProcessID,
                  matrixID = scenario_example$MatrixID,
                  cropHeight = scenario_example$CropHeight
                ),
                function(config, barrierID, matrixID, cropHeight) {
              
                  if (barrierID %in% c(24, 26, 27, 28) &&
                      matrixID %in% c(3, 5)) {
              
                    update_logreduction_specific_barrier(
                      config = config,
                      cropHeight = cropHeight,
                      barrierID = barrierID
                    )
              
                  } else {
                    config
                  }
                }
              )
  
  difference_min_value <- scenario_example$config[[1]]$treatment$processes[76, ]$min - result[[1]]$treatment$processes[76, ]$min
  difference_max_value <- scenario_example$config[[1]]$treatment$processes[76, ]$max - result[[1]]$treatment$processes[76, ]$max
  
  expect_equal(object = difference_min_value,
               expected = 0) 
  
  expect_equal(object = difference_max_value,
               expected = 0)
  
  difference_min_value <- scenario_example$config[[1]]$treatment$processes[76, ]$min - result[[2]]$treatment$processes[76, ]$min
  difference_max_value <- scenario_example$config[[1]]$treatment$processes[76, ]$max - result[[2]]$treatment$processes[76, ]$max
  
  expect_equal(object = difference_min_value,
               expected = 0)
  
  expect_equal(object = difference_max_value,
               expected = 0)
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)

## ----setup--------------------------------------------------------------------
library(ambre)
set.seed(2024)

## ----exposure-control---------------------------------------------------------
config_ambre$exposure[, c("name", "type", "value", "min", "max")]

## ----crd----------------------------------------------------------------------
draw <- create_random_distribution(
  type = "uniform",
  number_of_repeatings = 3,
  number_of_events = 5,
  min = 1, max = 10,
  debug = FALSE
)
head(draw$events)
draw$paras

## ----grv----------------------------------------------------------------------
vol_row <- config_ambre$exposure[config_ambre$exposure$name == "volume_perEvent", ]
gv <- generate_random_values(
  vol_row,
  number_of_repeatings = 2,
  number_of_events = 4,
  debug = FALSE
)
summary(gv$events$volume_perEvent)

## ----percentile---------------------------------------------------------------
ambre:::get_percentile(0.9)

## ----triangle-collapse--------------------------------------------------------
degenerate <- data.frame(
  type = "triangle", value = NA,
  min = 2, max = 2, mode = NA,
  mean = NA, sd = NA, meanlog = NA, sdlog = NA
)
gv2 <- generate_random_values(degenerate, number_of_repeatings = 1,
                              number_of_events = 3, debug = FALSE)
unique(gv2$events$values)

## ----seed---------------------------------------------------------------------
set.seed(1)
a <- create_random_distribution(type = "uniform", number_of_repeatings = 1,
                                number_of_events = 4, min = 0, max = 1, debug = FALSE)
set.seed(1)
b <- create_random_distribution(type = "uniform", number_of_repeatings = 1,
                                number_of_events = 4, min = 0, max = 1, debug = FALSE)
identical(a$events$values, b$events$values)

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Update concentration with desired value
#' 
#' Update the min, max and distribution law of the concentration in the simulated configuration according to a value defined by the user
#' 
#' @param scenario data.frame
#' @param concentration data.frame of pathogenName, min, max and distribution law to simulate
#' @importFrom purrr map
#' @importFrom dplyr mutate select
#' @return scenario_updated
#' 
#' @export
#' @examples
#' scenario_test <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#'
#' pathogenName <- c("Norovirus", "Rotavirus")
#' concentration_custom <- data.frame(PathogenName = pathogenName,
#'                                    min = c(1, 1),
#'                                    max = c(2,2),
#'                                    type = c("uniform", "uniform"))
#'
#' scenario_updated_patho <- update_pathogen(scenario = scenario_test, pathoName = pathogenName)
#' result <- update_concentration(scenario = scenario_updated_patho ,
#'                      concentration = concentration_custom)
#'
#' dplyr::filter(result$config[[1]]$inflow, PathogenName == "Norovirus")
#'
update_concentration <- function(scenario, concentration){
  
  if (isFALSE("min" %in% names(concentration))){
    stop("List of concentration must contain min values")
  }
  
  if (isFALSE("max" %in% names(concentration))){
    stop("List of concentration must contain max values")
  }
  
  if (isFALSE("type" %in% names(concentration))){
    stop("List of concentration must contain type distribution")
  }
  
  if (isFALSE("PathogenName" %in% names(concentration))){
    stop("List of concentration must contain PathogenName")
  }
  
  ### Jointure avec nom pathggen 
  scenario_updated <- scenario 
  
  
  scenario_updated$config <- map(
  scenario_updated$config,
  \(config) {
    config$inflow <- config$inflow |>
      left_join(
        concentration,
        by = "PathogenName"
      ) |>
      mutate(
        min = coalesce(min.y, min.x),
        max = coalesce(max.y, max.x),
        type = coalesce(type.y, type.x)
      ) |>
      select(-min.x, -min.y, -max.x, -max.y, -type.x, -type.y)

    config
  }
)
  return(scenario_updated)
}

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Water charge
#' 
#' Calculate  cost of water charge
#' @param scenario data.frame, with irrigation_need_m3an obtained with irrigation_need_calculation function
#' @param price_per_m3 numeric, water price in euro per cubic meter
#' @param membership_fee numeric annual membership fee in euro per hectare
#' @return updated_scenario
#' 
#' @export
#' @examples
#'
#'
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
#' water_price(scenario = irrigation_need,
#'             price_per_m3 = 0.01,
#'             membership_fee = 200)
water_price <- function(scenario, price_per_m3, membership_fee){
  scenario_updated <- scenario
  scenario_updated$water_charge <- price_per_m3 * scenario_updated$irrigation_need_m3an
  scenario_updated$annual_membership_fee <- membership_fee * scenario_updated$Area
  return(scenario_updated)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Select Columns from a Data Frame
#' 
#' Select columns from a data frame. Stop with message if columns do not exist
#' 
#' This function does the same thing as kwb.utils::selectColumns(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' @param x data frame
#' @param columns vector of column names. If \code{columns} is of length 0 or
#'   \code{NULL} (default) or \code{NA} \code{x} is returned unchanged.
#' @param pattern regular expression matching the names of the columns to be
#'   selected. Will only be evaluated if no explicit column names are given in 
#'   \code{columns}.
#' @param drop if \code{TRUE} and if only one column is to be selected the
#'   result is a vector (one dimensional) containing the values of the selected
#'   column and not a data frame. One dimension has been \emph{dropped} then.
#'   See the \code{help("[.data.frame")}. The default is \code{TRUE} if 
#'   \code{length(columns) == 1}, else \code{FALSE}.
#' @param do.stop this flag controls whether the function stops (\code{do.stop =
#'   TRUE}) or not (\code{do.stop = FALSE}) if there are non-existing columns to
#'   be selected. If \code{do.stop = FALSE} only those columns are selected that
#'   actually exist
#' @return data frame containing the columns of \code{x} that are specified in 
#'   \code{columns} or \code{x} itself if \code{columns} is \code{NULL} or a
#'   vector containing the values of column \code{value} if \code{columns} is of
#'   length 1 and \code{drop = TRUE} (which is the default in this case).
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' 
#' @examples
#'
#' # Example usage of selectColumns
#' df <- data.frame(a = 1:3, b = letters[1:3], c = c(TRUE, FALSE, TRUE))
#'
#' # Select specific columns
#' selected <- selectColumns(df, columns = c("a", "b"))
#' print(selected)
#'
#' # Select columns using pattern
#' pattern_selected <- selectColumns(df, pattern = "a|b")
#' print(pattern_selected)
#'
#' # Select single column with drop = TRUE (returns vector)
#' single_column <- selectColumns(df, columns = "a", drop = TRUE)
#' print(single_column)
#'
#' # Try to select non-existing column (with warning)
#' tryCatch({
#'   selectColumns(df, columns = c("a", "x"), do.stop = FALSE)
#' }, warning = function(w) {
#'   print("Caught expected warning:")
#'   print(w$message)
#' })
#'
#' # Try to select non-existing column (with error)
#' tryCatch({
#'   selectColumns(df, columns = c("a", "x"), do.stop = TRUE)
#' }, error = function(e) {
#'   print("Caught expected error:")
#'   print(e$message)
#' })
#'
selectColumns <- function(
  x, columns = NULL, pattern = NULL, drop = (length(columns) == 1), 
  do.stop = TRUE
)
{
  if (! is.data.frame(x)) {
    stop(call. = FALSE, sprintf(
      "%s given to selectColumns() must be a data frame but is of class: %s", 
      deparse(substitute(x)),
      stringList(class(x))
    ))
  }
  
  if (is.null(columns) || length(columns) == 0 || all(is.na(columns))) {
    
    if (is.null(pattern)) {
      
      return(x)
      
    } else {
      
      columns <- grep(pattern, names(x), value = TRUE)
    }
  }
  
  ok <- checkForMissingColumns(
    x, columns, dataFrameName = deparse(substitute(x)), do.stop = do.stop
  )
  
  if (! ok) {
    warning("Only the existing columns are selected.")
    columns <- intersect(columns, names(x))
  }
  
  x[, columns, drop = drop]
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.


test_that("selectColumns works correctly", {
  df <- data.frame(a = 1:3, b = letters[1:3], c = c(TRUE, FALSE, TRUE))

  # Test with explicit columns
  expect_equal(selectColumns(df, columns = c("a", "b")), df[, c("a", "b")])

  # Test with pattern
  expect_equal(selectColumns(df, pattern = "a|b"), df[, c("a", "b")])

  # Test with drop = TRUE
  expect_equal(selectColumns(df, columns = "a", drop = TRUE), df$a)

  # Test with non-existing columns (do.stop = FALSE warns instead of stopping)
  expect_warning(res <- selectColumns(df, columns = c("a", "x"), do.stop = FALSE), "No such column")
  expect_equal(res, df[, "a"])

  # Test with NULL columns
  expect_equal(selectColumns(df, columns = NULL), df)

  # Test with NA columns
  expect_equal(selectColumns(df, columns = NA), df)

  # Test with empty columns
  expect_equal(selectColumns(df, columns = character(0)), df)

  # Test with non-data frame input
  expect_error(selectColumns(1:3, columns = "a"), ".*must be a data frame.*")

})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Inflow concentration
#'
#' To calculate inflow concentration for specified pathogen
#' @param scenario list scenario created with create_scenario
#' @param pathogenName character names of the pathogens to be simulated
#' @param monteCarlo integer number of simulation iterations for Monte Carlo
#' @importFrom purrr map
#' @importFrom dplyr mutate
#' @return scenario_with_concentration
#'
#' @export
#' @examples
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#' result <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni", "Norovirus"),
#'  monteCarlo = 2000)
#'
inflow_concentration <- function(scenario, pathogenName, monteCarlo = 1000){

  scenario_updated_patho <- update_pathogen(scenario = scenario, pathoName = pathogenName)
  scenario_updated_freq <- update_frequency(scenario_updated_patho)
  scenario_updated_vol <- update_volume(scenario_updated_freq)
  scenario_updated_monte_carlo <- update_monte_carlo(scenario = scenario_updated_vol,
                                                     monte_carlo = monteCarlo)

  concentration = map(.x = scenario_updated_monte_carlo$config,
                      .f= ~ simulate_inflow(.x))

  scenario_with_concentration <- scenario_updated_monte_carlo |>
     mutate(inflow_concentration = concentration)
  
  return(scenario_with_concentration)

}

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("plot_inflow works", {
  scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
  scenario_with_inflow <- inflow_concentration(scenario = scenario_test,
   pathogenName = c("Campylobacter jejuni", "Norovirus"))
  result <- plot_inflow(scenario_with_inflow)
  expect_s3_class(result, "ggplot")
})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("inflow_concentration works", {
  scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
  result <- inflow_concentration(scenario = scenario_test,
   pathogenName = c("Campylobacter jejuni", "Norovirus"))
  expect_true("inflow_concentration" %in% colnames(result))
})

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

library(dplyr)
library(purrr)
test_that("plot_volumes works", {
  scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

  scenario_conc <- inflow_concentration(scenario = scenario_test,
   pathogenName = c("Campylobacter jejuni"))
  
  scenario_volume <-
    scenario_conc |>                     
    mutate(
      volume = map(
        .x = config,                                         
        .f = ~ simulate_exposure(config = .x)              
      )
    )
  
  result <- plot_volumes(scenario_volume)
  expect_s3_class(result, "ggplot")
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)

## ----setup--------------------------------------------------------------------
library(ambre)
set.seed(2024)

## ----libs---------------------------------------------------------------------
library(dplyr)
library(purrr)

## ----default-exposure, results = "hide"---------------------------------------
scenario <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario_conc <- inflow_concentration(
  scenario = scenario,
  pathogenName = "Campylobacter jejuni"
)

## ----default-exposure-show----------------------------------------------------
# Row 1 of this two-row scenario: irrigation-staff droplet ingestion
scenario_conc$config[[1]]$exposure

## ----default-concentration-show-----------------------------------------------
# Row 1 of this two-row scenario: Campylobacter jejuni concentration
dplyr::filter(scenario_conc$config[[1]]$inflow, PathogenName == "Campylobacter jejuni")

## ----inspect-pathid-----------------------------------------------------------
pid <- query_exp_path(
  pathName = "Ingestion of water droplets during maintenance of the irrigation system"
)
pid

## ----inspect-values-----------------------------------------------------------
scenario$PathID          # the two paths in this input file

query_volume(scenario$PathID)     # litres per event: min / max
query_frequency(scenario$PathID)  # events per year: min / max

## ----override-volume----------------------------------------------------------
scenario_ppe <- update_volume_with_desired_value(
  scenario = scenario,
  volume = data.frame(
    min = c(0.0005, 0.0005),   # 0.5 mL per event, one entry per row
    max = c(0.0005, 0.0005),
    type = c("triangle", "triangle")
  )
)

scenario_ppe$config[[1]]$exposure

## ----override-frequency-------------------------------------------------------
scenario_night <- update_frequency_with_desired_value(
  scenario = scenario,
  frequency = c(30L, 30L)   # cap both paths at 30 events / year
)

scenario_night$config[[1]]$exposure

## ----override-concentration---------------------------------------------------
concentration_custom <- data.frame(PathogenName = c("Campylobacter jejuni"),
                                   min = c(1),
                                   max = c(2),
                                   type = c("uniform"))

scenario_pathogen <- update_pathogen(scenario = scenario, pathoName = concentration_custom$PathogenName)
scenario_low_pathogen <- update_concentration(scenario = scenario_pathogen ,
                     concentration = concentration_custom)

dplyr::filter(scenario_low_pathogen$config[[1]]$inflow, PathogenName == "Campylobacter jejuni")


## ----qmra-custom, results = "hide"--------------------------------------------
sc <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

regulation_reduction <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Country, RegulationID, Reduction))

concentration_custom <- data.frame(PathogenName = c("Rotavirus"),
                                   min = c(1000),
                                   max = c(2000),
                                   type = c("uniform"))

set.seed(2024)
bare <- run_qmra_custom(scenario = sc,
                  concentration = concentration_custom,
                  volume = data.frame(min=c(0.005,0.005), max = c(0.01, 0.01), type = c("triangle", "triangle")),
                  frequency = c(48L, 60L),
                  regulationLog = regulation_reduction,
                  regulationConcentration = regulation_concentration,
                  initialSituation = TRUE) # min 5 mL, max 10 mL

set.seed(2024)
ppe  <- run_qmra_custom(scenario = sc,
                  concentratio = concentration_custom,
                  volume = data.frame(min=c(0.0005,0.0005), max = c(0.001, 0.001), type = c("triangle", "triangle")),
                  frequency = c(48L, 60L),
                  regulationLog = regulation_reduction,
                  regulationConcentration = regulation_concentration,
                  initialSituation = TRUE) # min 0.5 mL min 0.1 mL


## ----qmra-custom-comparison, results = "hide"---------------------------------
library(ggplot2)
cowplot::plot_grid(
      bare$dalys$Rotavirus +
        labs(subtitle = "min 5 mL, max 10 mL") +
        theme(plot.subtitle = element_text(hjust = 0.5)),
      ppe$dalys$Rotavirus +
        labs(subtitle = "min 0.5 mL min 0.1 mL") +
        theme(plot.subtitle = element_text(hjust = 0.5)),
      ncol = 2,
      align = "h"
    )

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Get exposure value or stop
#' 
#' This function does the same thing as kwb.utils::exposure_value_or_stop(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' Description
#' @param config list config
#' @param name character name of the column
#' @return value
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @export
#' @examples
#' get_exposure_value_or_stop(config_ambre, "number_of_exposures")
get_exposure_value_or_stop <- function(config, name)
{
  # HARD CODED: "name" MUST be of type value!
  value <- config$exposure$value[config$exposure$name == name]

  if (is.na(value)) {
    stop(sprintf(get_stop_text("must_be_value"), name), call. = FALSE)
  }
  
  value
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("get_exposure_value_or_stop works", {
  expect_equal(object = get_exposure_value_or_stop(config_ambre, "number_of_exposures"),
               expected = 365) 
})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Get risk total
#' 
#' Calcul the total illness probability and total dalys that are the sum of every exposure event
#' @param scenario data.frame with risk column
#' @importFrom dplyr group_by summarise n
#' @return scenario_updated
#' 
#' @export
#' @examples
#' library(dplyr)
#' library(purrr)
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#'
#' scenario_conc <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni"))
#'
#' scenario_volume <-
#'   scenario_conc |>                     
#'   mutate(
#'     volume = map(
#'       .x = config,                                         
#'       .f = ~ simulate_exposure(config = .x)              
#'     )
#'   )
#'
#' scenario_dose_ini <- initial_dose_calculation(scenario_volume)
#'
#' scenario_scheme <- update_treatment_scheme(scenario_dose_ini)
#'
#' scenario_with_logreduc_and_co <- scenario_scheme |> 
#'   mutate( log_reduction = map(config, simulate_treatment))
#'
#' scenario_final_dose_and_co_test <- final_dose_calculation(scenario_with_logreduc_and_co)
#'
#' scenario_inf_proba_and_co <- infection_probability_calculation(scenario_final_dose_and_co_test)
#'
#' scenario_illness_proba_and_co <- illness_probability_calculation(scenario_inf_proba_and_co)
#'
#' scenario_dalys_and_co <- dalys_calculation(scenario_illness_proba_and_co)
#'
#' get_risk_total(scenario_dalys_and_co)
get_risk_total <- function(scenario){
  total_risk <- map(.x = scenario$risk, ~.x |> 
    group_by(repeatID, 
            PathogenID, 
            PathogenName.x, 
            PathogenGroup) |> 
    summarise(events = dplyr::n(), 
            inflow_median = median(inflow), 
            effluent_median = median(effluent), 
            logreduction_median = median(logreduction), 
            volume_sum = sum(volume_perEvent), 
            dose_sum = sum(dose_perEvent), 
            infectionProb_sum = 1 - prod(1 - proba_inf), 
            illnessProb_sum = 1 - prod(1 - illnessProb_per_event), 
            dalys_sum = sum(dalys_per_event)))

  scenario_updated <- scenario |> mutate(risk_total = total_risk)
  
  return(scenario_updated)
    
}

# WARNING - Generated by {fusen} from dev/flat_kwb_infection.Rmd: do not edit by hand # nolint: line_length_linter.

#' Get infection probability values
#' 
#' Simulate infection probability according to pathogen consider
#' @param tbl_risk data.frame containing PathogenID and dose_perEvent columns
#' @param dose_response data.frame database of dose_response factors presents in config_ambre
#' @param health data.frame database of health factors presents in config_ambre
#' @return result_vector
#' @references Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @export
get_infection_prob <- function(tbl_risk, dose_response, health)
{
  result_vector <- rep(NA_real_, nrow(tbl_risk))
  
  for (pathogenID in unique(tbl_risk$PathogenID)) {
    
    condition <- tbl_risk$PathogenID == pathogenID
    
    this_pathogen <- dose_response$PathogenID == pathogenID
    
    values <- get_dose_response_values(
      dose = tbl_risk$dose_perEvent[condition], 
      k = dose_response$k[this_pathogen], 
      n50 = dose_response$N50[this_pathogen],
      alpha = dose_response$alpha[this_pathogen],
      pathogen_name = health$PathogenName[health$PathogenID == pathogenID]
    )

    result_vector[condition] <- values$infectionProbability
  }
  
  return(result_vector)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)

## ----setup--------------------------------------------------------------------
library(ambre)
set.seed(2024)

## ----attach-dplyr-------------------------------------------------------------
library(dplyr)

## ----overview-str-------------------------------------------------------------
str(config_ambre, max.level = 1)

## ----overview-subtables-------------------------------------------------------
lapply(config_ambre[c("treatment", "path", "economic")], names)

## ----mapping-table------------------------------------------------------------
mapping <- tibble::tribble(
  ~`CSV in data-raw/`,                  ~`Slot in config_ambre`,
  "exposure.csv",                       "exposure",
  "ambre_pathogene.csv",                "inflow",
  "ambre_barriere_general.csv",         "treatment$processes",
  "treatment_schemes.csv",              "treatment$schemes",
  "ambre_barriere_voie.csv",            "treatment$barriere_path",
  "ambre_barriere_specifique.csv",      "treatment$barriere_specific",
  "ambre_barriere_deperissement.csv",   "treatment$barriere_decay",
  "ambre_doseresponse.csv",             "doseresponse",
  "ambre_voie_description.csv",         "path$description",
  "ambre_voie_frequence.csv",           "path$frequency",
  "ambre_voie_volume.csv",              "path$volume",
  "ambre_culture_hauteur.csv",          "crop",
  "ambre_sante.csv",                    "health",
  "ambre_culture_eau.csv",              "economic$water_need",
  "ambre_population.csv",               "economic$population",
  "ambre_barriere_cout.csv",            "economic$cost"
)
knitr::kable(mapping)

## ----er-path------------------------------------------------------------------
config_ambre$path$description |>
  select(PathID, PopulationName, MatrixName, PathDescription) |>
  head(4)

## ----er-pathogen--------------------------------------------------------------
config_ambre$health |>
  select(PathogenID, PathogenName, dalys_per_case) |>
  inner_join(
    config_ambre$inflow |> select(PathogenID, PathogenGroup),
    by = "PathogenID"
  ) |>
  head(4)

## ----barrier-groups-----------------------------------------------------------
config_ambre$treatment$processes |>
  distinct(TreatmentID, TreatmentName, TreatmentGroup) |>
  count(TreatmentGroup, name = "n_barriers")

## ----barrier-prefix-----------------------------------------------------------
config_ambre$treatment$processes |>
  distinct(TreatmentID, TreatmentName) |>
  mutate(family = sub("[.].*", "", TreatmentName)) |>
  count(family, name = "n_barriers")

## ----dist-legend--------------------------------------------------------------
knitr::kable(tibble::tribble(
  ~type,           ~`columns read`,   ~draws,
  "value",         "value",           "a constant, repeated",
  "uniform",       "min, max",        "runif(min, max)",
  "triangle",      "min, max, mode",  "EnvStats::rtri(min, max, mode)",
  "norm",          "mean, sd",        "rnorm(mean, sd)",
  "lognorm",       "meanlog, sdlog",  "rlnorm(meanlog, sdlog)",
  "log10_uniform", "min, max",        "10^runif(min, max)",
  "log10_norm",    "mean, sd",        "10^rnorm(mean, sd)"
))

## ----dist-row-barrier---------------------------------------------------------
config_ambre$treatment$processes |>
  filter(TreatmentName == "Q.1 - Activated Sludge") |>
  select(TreatmentName, PathogenGroup, type, value, min, max, mode)

## ----dist-row-inflow----------------------------------------------------------
config_ambre$inflow |>
  filter(PathogenName == "Campylobacter jejuni") |>
  select(PathogenName, PathogenGroup, type, value, min, max)

## ----dist-exposure------------------------------------------------------------
config_ambre$exposure

## ----customize, eval = FALSE--------------------------------------------------
#  # 1. add a new quality barrier to the treatment CSV
#  csv_path <- file.path("data-raw", "ambre_barriere_general.csv")
#  new_barrier <- data.frame(
#    TreatmentID = 99, TreatmentName = "Q.8 - Ozonation",
#    TreatmentGroup = "Quality", PathogenGroup = "Bacteria",
#    type = "uniform", value = NA, min = 2, max = 4,
#    mode = NA, mean = NA, sd = NA, meanlog = NA, sdlog = NA
#  )
#  readr::write_csv(
#    rbind(readr::read_csv(csv_path), new_barrier),
#    csv_path
#  )
#  
#  # 2. rebuild data/config_ambre.rda from the 16 CSVs
#  source(file.path("data-raw", "config_ambre.R"))
#  
#  # 3. verify the new barrier resolves to its ID
#  query_barrier(barrierName = "Q.8 - Ozonation")

## ----query-helpers------------------------------------------------------------
query_pathogen(pathogenName = "Campylobacter jejuni")
query_barrier(barrierName = "Q.3 - UV Reactor")
query_crop(cropName = "Tomato")

## ----query-path---------------------------------------------------------------
query_volume(pathID = 1)
query_frequency(pathID = 1)

## ----pitfall-class------------------------------------------------------------
class(config_ambre$crop)

## ----pitfall-crop-------------------------------------------------------------
config_ambre$crop |>
  select(CropName, CropHeight)

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Enumerate Elements
#'
#' This function does the same thing as kwb.utils::enumeration(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#'
#' @param x vector of elements to be enumerated
#' @param type type of element to appear in the message. Default: "element"
#' @param sorted logical. Whether or not to print the available elements in
#'   lexical order. Default: \code{FALSE}
#' @param suffix suffix to be appended to \code{type}. Can be used to
#'   distinguish between singular and plural form. Default: \code{""}
#' @return character string containing the enumeration
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @export
#' @examples
#'
#' # Example usage of enumeration
#' cat(enumeration(c("a", "b", "c"), type = "element"))
#' cat(enumeration(c("x", "y"), type = "element", suffix = "s"))
enumeration <- function(x, type = "element", sorted = FALSE, suffix = "")
{
  sprintf(
    "%s%s%s%s:\n- %s\n", 
    type,
    ifelse(length(x) > 1L, "s", ""), # plural "s"
    ifelse(length(x) > 1L && sorted, " (sorted)", ""), 
    suffix,
    stringList(if (sorted) sort(x) else x, collapse = "\n- ")
  )
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("enumeration works correctly", {
  # Test with default parameters
  expect_equal(
    enumeration(c("a", "b", "c")),
    "elements:\n- 'a'\n- 'b'\n- 'c'\n"
  )

  # Test with custom type
  expect_equal(
    enumeration(c("x", "y"), type = "item"),
    "items:\n- 'x'\n- 'y'\n"
  )

  # Test with sorted = TRUE
  expect_equal(
    enumeration(c("c", "a", "b"), sorted = TRUE),
    "elements (sorted):\n- 'a'\n- 'b'\n- 'c'\n"
  )

  # Test with suffix
  expect_equal(
    enumeration(c("x", "y"), suffix = "s"),
    "elementss:\n- 'x'\n- 'y'\n"
  )

  # Test with single element
  expect_equal(
    enumeration(c("a")),
    "element:\n- 'a'\n"
  )

  # Test with empty vector
  expect_equal(
    enumeration(character(0)),
    "element:\n- ''\n"
  )

  # Test with numeric vector
  expect_equal(
    enumeration(1:3),
    "elements:\n- 1\n- 2\n- 3\n"
  )

  # Test with logical vector
  expect_equal(
    enumeration(c(TRUE, FALSE)),
    "elements:\n- TRUE\n- FALSE\n"
  )
})

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

#' Log reduction matrix calculation
#' 
#' Calculate the difference of maximal log reduction achieve according to regulation 
#' @param scenario data.frame with log_reduction column obtained with simulate_treatment function
#' @param regulation data.frame regulation value to compare to simulation results
#' @importFrom formattable formattable
#' @importFrom purrr map
#' @import dplyr
#' @importFrom tidyr unnest pivot_wider
#' @return formattable 
#' 
#' @export
#' @examples
#' \donttest{
#' library(dplyr)
#' library(purrr)
#' scenario_example <- create_scenario(system.file("input_1culture_2pop.xlsx", 
#'                                                 package = "ambre"))
#'
#' scenario_exposure <- scenario_example |> mutate(
#'      volume = map(
#'          .x = config,                                         
#'          .f = ~ simulate_exposure(config = .x)))
#'
#' scenario_patho <- update_pathogen(scenario = scenario_exposure, 
#'                                   pathoName = c("Rotavirus", "Campylobacter jejuni"))
#' concentration = map(.x = scenario_patho$config,
#'                    .f= ~ simulate_inflow(.x))
#' scenario_concentration <- scenario_patho |> mutate(inflow_concentration = concentration)
#' scenario_ini_dose <- initial_dose_calculation(scenario = scenario_concentration)
#'
#' scenario_barrier <- update_treatment_scheme(scenario = scenario_ini_dose, initial_situation = TRUE)
#'
#' scenario_barrier$config <- purrr::pmap(
#'   list(
#'     config = scenario_barrier$config,
#'     barrierID = scenario_barrier$SupplementaryProcessID,
#'     cropHeight = scenario_barrier$CropHeight,
#'     matrixID = scenario_barrier$MatrixID,
#'     popID = scenario_barrier$PopulationID,
#'     decay = scenario_barrier$nb_day_decay
#'   ),
#'   function(config, barrierID, cropHeight, matrixID, popID, decay) {
#'
#'     if (barrierID %in% c(24, 26, 27, 28) &&
#'         matrixID %in% c(3, 5)) {
#'
#'       update_logreduction_specific_barrier(
#'         config = config,
#'         cropHeight = cropHeight,
#'         barrierID = barrierID
#'       )
#'
#'     } else if (barrierID %in% c(12)) {
#'
#'       update_logreduction_decay(
#'         config = config,
#'         cropHeight = cropHeight,
#'         barrierID = barrierID,
#'         popID = popID,
#'         nb_day_decay = decay
#'       )
#'
#'     } else if (barrierID %in% c(9,13,14,15,16,17,18,24,26,27,28)) {
#'
#'       update_logreduction_path(
#'         config = config,
#'         matrixID = matrixID,
#'         barrierID = barrierID,
#'         popID = popID
#'       )
#'
#'     } else {
#'       config
#'     }
#'   }
#' )
#'   
#' scenario_logreduction <- scenario_barrier |> 
#'     mutate(log_reduction = map(config, simulate_treatment))
#'   
#'
#' regulation_reduction <- config_ambre$regulation$regulation_value |> 
#'   dplyr::filter(Country == "France") |>
#'   dplyr::select(-c(Concentration, Country, RegulationID))
#'
#' regulation_matrix_log(scenario = scenario_logreduction,
#'                       regulation = regulation_reduction)
#' }
regulation_matrix_log <- function(scenario, regulation){
  
  if (!"log_reduction" %in% names(scenario)) {
  stop("The log_reduction column is missing from the scenario object; run simulate_treatment function to create it")
  }
  
  total_logreduction_regulation <- map(
    scenario$log_reduction,
    ~ .x$events_long |>
        filter(TreatmentID %in% 1:7) |>
        group_by(repeatID, eventID, PathogenGroup) |>
        summarise(total_logreduction = round(sum(logreduction),0), .groups = "drop")
  )
    
  scenario$log_reduction_regulation <- total_logreduction_regulation
  
  
  regulation_evaluation <- map2(.x= scenario$inflow_concentration, 
                                .y = scenario$log_reduction_regulation,
                                .f= ~left_join(.x$events, .y, by = c("repeatID", "eventID", "PathogenGroup")) |>
                          mutate(effluent = 10^(log10(inflow) - total_logreduction))
                          )
    
  
  scenario$regulation <- regulation_evaluation  
  

  abattement_total <- scenario |> select(CropName, regulation)
  reduction_achieve <- map(
    abattement_total$regulation,
    ~ .x |>
      group_by(PathogenGroup) |>
      summarise(
        max_logreduction = round(max(total_logreduction, na.rm = TRUE),0),
        .groups = "drop"
      )
  )
  pathogen_group <- map(.x = reduction_achieve, .f = ~.x$PathogenGroup)
  logreduc <- map(.x = reduction_achieve, .f = ~.x$max_logreduction)
  
  abattement_total$PathogenGroup <- pathogen_group
  abattement_total$reduction_achieve <- logreduc
  
  abattement <- abattement_total |> select(-regulation) |> unnest(c(PathogenGroup, reduction_achieve))
  
  
  abattement_class <- abattement |> 
    left_join(config_ambre$regulation$regulation_crop, by = "CropName")
  
  
  abattement_total_class_regulation <- abattement_class |> 
    left_join(regulation, by= c("Class","PathogenGroup")) |>
    unique() |>
    rename(reduction_regulation = Reduction) |>
    rename(target_class = Class)|>
                          mutate(color = case_when(
                                            reduction_regulation > reduction_achieve ~ "#e74c3c",
                                            reduction_regulation <= reduction_achieve ~ "#2ecc71",   # vert clair
                                             TRUE         ~ "#bdbdbd"
                                          ))
  tableau_valeur <- abattement_total_class_regulation |>
    select(CropName, PathogenGroup, reduction_achieve, target_class) |>
    rename("Crop name" = CropName) |> 
    rename("Target water class" = target_class) |>
    pivot_wider(names_from=PathogenGroup, values_from = reduction_achieve)
  
  tableau_couleur <- abattement_total_class_regulation |>
    rename("Crop name" = CropName) |> 
    rename("Target water class" = target_class) |>
    select("Crop name", "PathogenGroup", "color", "Target water class") |>
    pivot_wider(names_from=PathogenGroup, values_from = color)
  
  tableau_name <- tableau_valeur |> select(-c("Crop name", "Target water class"))
  colonnes_pathogenes <- names(tableau_name)
   
  liste_formats <- setNames(
    lapply(colonnes_pathogenes, function(col) {
      format_color(tableau_couleur[[col]])
    }),
    colonnes_pathogenes
  )
  
  table <- formattable(tableau_valeur, liste_formats)
  return(table)
    
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Create random distribution
#' 
#' This function does the same thing as kwb.utils::create_random_distribution(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' @param type "uniform" calls runif(), "log10_uniform" calls 
#' 10^runif(number_of_events, log10_min, log10_max), "triangle" calls 
#' EnvStats::rtri(), "lognorm" calls rlnorm(), "norm" calls rnorm() and 
#' "log10_norm" calls 10^rnorm(number_of_events, mean = log10_mean, 
#' sdev = log10_sdev), (default: "uniform") 
#' @param number_of_repeatings how often should the random distribution with the
#' same parameters be generated (default: 1)
#' @param  number_of_events number of events
#' @param value constant value (no random number), gets repeated number_of_events
#' times (if 'type' = 'value')
#' @param min minimum value (default: 10), only used if 'type' is "runif" or
#' "triangle"
#' @param max maximum value (default: 1000), only used if 'type' is "runif" or
#' "triangle"
#' @param percent_within_minmax percent of data point within min/max (default: 
#' 0.9 i.e. 90 percent
#' @param min_zero  only used if 'type' is "log10_uniform" or 
#' "log10_norm", "norm" or "lognorm" and "min" value equal zero. 
#' In this case the zero is replaced by this value (default: 0.01), see also
#' \code{default_min()}
#' @param log10_min minimum value (default: default_min(min, max, 
#' min_zero, f = log10)), only used if 'type' is "log10_uniform" or "log10_norm"
#' @param log10_max maximum value (default: ifelse(max > 0, log10(max), 
#' log10_zero_threshold), only used if 'type' is "log10_uniform" or "log10_norm"
#' @param log10_mean mean value (default: (log10_min + log10_max)/2), only used 
#' if 'type' is "log10_norm"
#' @param log10_sdev standard deviation (default: abs((log10_max- log10_mean) / 
#' get_percentile(0.95)), only used if 'type' is "log10_norm"
#' @param mean mean value (default: (default_min(min, max, min_zero) / 
#' default_max(max, 10*min_zero)) / 2), only used if 'type' is "norm"
#' @param sdev standard deviation (default: abs((default_max(max, 10*min_zero) - 
#' mean) / get_percentile(0.95))), only used if 'type' is "norm"
#' @param meanlog log mean value (default: mean(log((min + max) / 2))), only
#' used if 'type' is "lognorm"
#' @param sdlog standard deviation (default: abs(sd(c(default_min(min, max, 
#' min_zero, f = log)))) ), only used if 'type' is "lognorm"
#' @param mode (default: default_min(min, max, min_zero) + 
#' default_max(max, 10 * min_zero) / 2), only used if 'type' is "triangle"
#' @param debug print debug information (default: TRUE)
#' @return list with parameters of user defined random distribution and
#' corresponding values
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @export
#' @importFrom stats sd qnorm runif rnorm rlnorm
#' @importFrom dplyr mutate
#' @importFrom rlang .data
#' @importFrom EnvStats rtri
#' 
#' @examples
#'
#' # Example usage of create_random_distribution
#' # Uniform distribution
#' uniform_dist <- create_random_distribution(
#'   type = "uniform",
#'   number_of_repeatings = 2,
#'   number_of_events = 10,
#'   min = 0,
#'   max = 100
#' )
#'
#' # Log10 uniform distribution
#' log10_uniform_dist <- create_random_distribution(
#'   type = "log10_uniform",
#'   number_of_events = 10,
#'   min = 1,
#'   max = 1000
#' )
#'
#' # Triangle distribution
#' triangle_dist <- create_random_distribution(
#'   type = "triangle",
#'   number_of_events = 10,
#'   min = 0,
#'   max = 100,
#'   mode = 50
#' )
#'
#' # Normal distribution
#' norm_dist <- create_random_distribution(
#'   type = "norm",
#'   number_of_events = 10,
#'   mean = 50,
#'   sdev = 10
#' )
#'
#' # Log10 normal distribution
#' log10_norm_dist <- create_random_distribution(
#'   type = "log10_norm",
#'   number_of_events = 10,
#'   log10_mean = 2,
#'   log10_sdev = 0.5
#' )
#'
#' # Lognormal distribution
#' lognorm_dist <- create_random_distribution(
#'   type = "lognorm",
#'   number_of_events = 10,
#'   meanlog = 2,
#'   sdlog = 0.5
#' )
#'
#' # Constant value
#' value_dist <- create_random_distribution(
#'   type = "value",
#'   number_of_events = 10,
#'   value = 42
#' )
create_random_distribution <- function(
  type = "uniform",
  number_of_repeatings = 1,
  number_of_events = 365,
  value = 10,
  min = 10,
  max = 1000,
  percent_within_minmax = 0.9,
  min_zero = 0.01,
  log10_min = default_min(min, max, min_zero, f = log10), 
  log10_max = default_max(max, min_zero * 10, f = log10),
  log10_mean = (log10_min + log10_max) / 2,
  log10_sdev = abs((log10_max - log10_mean) / get_percentile(percent_within_minmax)),
  mean = (default_min(min, max, min_zero) + default_max(max, 10*min_zero)) / 2,
  sdev = abs((default_max(max, 10*min_zero) - mean) / get_percentile(percent_within_minmax)),
  meanlog = mean(log(default_min(min, max, min_zero) + default_max(max, 10 * min_zero)) / 2),
  sdlog = abs(sd(c(default_min(min, max, min_zero, f = log), 
                   default_max(max, 10 * min_zero, f = log)))),
  mode = (default_min(min, max, min_zero) + default_max(max, 10 * min_zero)) / 2,
  debug = TRUE
)
{
  # Inline helper functions 
  
  # Print a message if debug is TRUE
  debug_formatted <- function(fmt, ...) if (debug) cat(sprintf(fmt, ...))

  # Consistent message text
  distribution_text <- function(type_text, ...) paste0(
    "Create ", number_of_repeatings, " random distribution(s): ",
    type_text, " (n: ", number_of_events, ..., ")\n"
  )

  # Creation of paramter data frame
  paras_data_frame <- function(...) data.frame(
    type = type, 
    repeatings = number_of_repeatings,
    events = number_of_events,
    ...
  )
  
  # Creation of event data frame
  event_data_frame <- function(...) distribution_repeater(
    number_of_repeatings = number_of_repeatings,
    number_of_events = number_of_events,
    ...
  )
  
  # Product of numbers, to be reused
  number_product <- number_of_events * number_of_repeatings
  
  if (type == "value") {
    
    debug_formatted(
      "Replicate %d times constant value %f\n", number_product, value
    )

    events <- data.frame(
      repeatID = unlist(lapply(
        seq_len(number_of_repeatings), rep, number_of_events
      )),
      eventID = rep(seq_len(number_of_events), number_of_repeatings),
      values = rep(value, number_product)
    )
    
    paras <- paras_data_frame(value = value)
    
  } else if (type == "uniform") {
    
    debug_formatted(
      distribution_text("uniform", ", min: %f, max: %f"), 
      min, max
    )
    
    events <- event_data_frame(func = runif, min = min, max = max)
    paras <- paras_data_frame(min = min, max = max)
    
  } else if (type == "log10_uniform") {
    
    debug_formatted(
      distribution_text("10^runif", ", min: %f, max: %f"), 
      log10_min, log10_max
    )

    events <- event_data_frame(
      func = runif, min = log10_min, max = log10_max
    ) |> 
      dplyr::mutate(values = 10^.data$values)
    
    paras <- paras_data_frame(min = log10_min, max = log10_max)
    
  } else if (type == "norm") {
    
    debug_formatted(
      distribution_text("norm", ", mean: %f, sd: %f"), mean, sdev
    )
    
    events <- event_data_frame(func = rnorm, mean = mean, sd = sdev)
    paras <- paras_data_frame(mean = mean, sd = sdev)
    
  } else if (type == "log10_norm") {
    
    debug_formatted(
      distribution_text("10^rnorm ", ", mean: %f, sd: %f"),
      log10_mean, log10_sdev
    )
    
    events <- event_data_frame(
      func = rnorm, mean = log10_mean, sd = log10_sdev
    ) |> 
      dplyr::mutate(values = 10^.data$values)
    
    paras <- paras_data_frame(mean = log10_mean, sd = log10_sdev)
    
  } else if (type == "lognorm") {
    
    debug_formatted(
      distribution_text("lognorm", ", meanlog: %f, sdlog: %f"), 
      meanlog, sdlog
    )
    
    events <- event_data_frame(func = rlnorm, meanlog = meanlog, sdlog = sdlog)
    paras <- paras_data_frame(meanlog = meanlog, sdlog = sdlog)
    
  } else if (type == "triangle") {
    
    debug_formatted(
      distribution_text("triangle", ", min: %f, max: %f, mode = %f"),
      min, max, mode
    )
    
    events <- event_data_frame(
      func = EnvStats::rtri, min = min, max = max, mode = mode
    )
    paras <- paras_data_frame(min = min, max = max, mode = mode)
    
  } else {
    
    stop(sprintf(get_stop_text("no_such_distribution"), type), call. = FALSE)
  }

  list(events = events, paras = paras)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("create_random_distribution works correctly", {
  # Deterministic RNG so the range checks below are reproducible. Bounds are set
  # to +/- 5 sd: wide enough that valid draws never fall outside them, so the
  # test asserts the distribution's location/scale family, not an exact draw.
  set.seed(42)

  # Test with default parameters (uniform)
  result <- create_random_distribution()
  expect_equal(nrow(result$events), 365)
  expect_equal(ncol(result$events), 3)
  expect_equal(ncol(result$paras), 5)
  expect_true(all(result$events$values >= 10 & result$events$values <= 1000, na.rm = TRUE))

  # Test with log10_uniform
  result <- create_random_distribution(type = "log10_uniform", min = 1, max = 100)
  expect_true(all(result$events$values >= 1 & result$events$values <= 100, na.rm = TRUE))

  # Test with triangle
  result <- create_random_distribution(type = "triangle", min = 0, max = 100, mode = 50)
  expect_true(all(result$events$values >= 0 & result$events$values <= 100, na.rm = TRUE))

  # Test with norm (mean +/- 5 sd)
  result <- create_random_distribution(type = "norm", mean = 50, sdev = 10)
  expect_true(all(result$events$values >= 50 - 5 * 10 & result$events$values <= 50 + 5 * 10, na.rm = TRUE))

  # Test with log10_norm (exponent within mean +/- 5 sd)
  result <- create_random_distribution(type = "log10_norm", log10_mean = 2, log10_sdev = 0.5)
  expect_true(all(result$events$values >= 10^(2 - 5 * 0.5) & result$events$values <= 10^(2 + 5 * 0.5), na.rm = TRUE))

  # Test with lognorm (log within meanlog +/- 5 sdlog)
  result <- create_random_distribution(type = "lognorm", meanlog = 2, sdlog = 0.5)
  expect_true(all(result$events$values >= exp(2 - 5 * 0.5) & result$events$values <= exp(2 + 5 * 0.5), na.rm = TRUE))

  # Test with value
  result <- create_random_distribution(type = "value", value = 42)
  expect_true(all(result$events$values == 42, na.rm = TRUE))

  # Test with custom number_of_repeatings and number_of_events
  result <- create_random_distribution(number_of_repeatings = 2, number_of_events = 10)
  expect_equal(nrow(result$events), 20)
  expect_equal(length(unique(result$events$repeatID)), 2)
  expect_equal(length(unique(result$events$eventID)), 10)

  # Test with debug = FALSE
  result <- create_random_distribution(debug = FALSE)
  expect_equal(nrow(result$events), 365)

  # Test with invalid type
  expect_error(create_random_distribution(type = "invalid_type"), "is not an implemented distribution")
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' isNullOrEmpty
#' 
#' This function does the same thing as kwb.utils::isNullOrEmpty(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' @param x object to be tested for NULL or being empty (vector or list of
#'   length 0 or data frame with no rows)
#' @return TRUE if x is NULL or x is a vector of length 0 or x is a data frame
#'   with no rows.
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' 
#' @examples
#' # Example usage of isNullOrEmpty
#'
#' isNullOrEmpty(NULL)
#' isNullOrEmpty(c(10))
#'
isNullOrEmpty <- function(x)
{
  is.null(x) || 
    (is.vector(x) && (length(x) == 0)) || 
    (is.list(x) && (length(x) == 0)) ||
    (is.data.frame(x) && (nrow(x) == 0))
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("isNullOrEmpty works correctly", {
  # Test NULL
  expect_true(isNullOrEmpty(NULL))

  # Test empty vector
  expect_true(isNullOrEmpty(c()))

  # Test empty list
  expect_true(isNullOrEmpty(list()))

  # Test empty data frame
  expect_true(isNullOrEmpty(data.frame()))

  # Test non-empty vector
  expect_false(isNullOrEmpty(c(1, 2, 3)))

  # Test non-empty list
  expect_false(isNullOrEmpty(list(1, 2, 3)))

  # Test non-empty data frame
  expect_false(isNullOrEmpty(data.frame(a = 1:3, b = letters[1:3])))
})

# WARNING - Generated by {fusen} from dev/flat_simulate_exposure.Rmd: do not edit by hand # nolint: line_length_linter.

#' Simulate: exposure
#' 
#' This function does the same thing as kwb.qmra::get_treatment_data(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' @param config as retrieved by config_read() 
#' @param debug print debug information (default: TRUE)
#' @return list with parameters of user defined exposure scenario (number of events 
#' and volumes per Event)
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @export
#' @examples
#' \donttest{
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#' scenario_updated <- update_pathogen(scenario = scenario_test,
#'  pathoName = c("Campylobacter jejuni", "Rotavirus"))
#'
#' simulate_exposure(config = scenario_updated$config[[1]])
#'}
simulate_exposure <- function(config, debug = TRUE){
  catIf(debug, sprintf("Simulated exposure: volume per event\n"))

  n_events <- number_of_exposures(config)
  
  volumes <-  generate_random_values(
    config = config$exposure[config$exposure$name == "volume_perEvent", ], 
    number_of_repeatings = number_of_repeatings(config),
    number_of_events = n_events,
    debug = debug
  )
  
  list(
    #number_of_events = n_events,
    volumes = list(
      events = renameColumns(volumes$events, list(
        values = "volume_perEvent"
      ))
    )
  )
}

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Run QMRA custom 
#' 
#' Run the full QMRA workflow using custom data for volume and frequency of exposure (not the default databases)
#' 1) simulate inflow concentration, volume of exposure 
#' 2) calculate initial dose
#' 3) simulate log reduction for treatment processes
#' 4) calculate final dose
#' 5) calculate infection probability, illness probability and dalys reduction
#' 6) calculate annual total risk
#' @param scenario data.frame obtain with create_scenario function
#' @param concentration data.frame of pathogenName, min, max and distribution law to simulate
#' @param volume data.frame of min and max value, same size as scenario
#' @param frequency array of integer same size as scenario
#' @param objective numeric reference dalys threshold to compare simulation result on dalys plot. default value is OMS threshold 1e-6 dalys
#' @param regulationLog data.frame regulation value of the log reduction to compare with simulation result
#' @param regulationConcentration data.frame regulation value of the log reduction to compare with simulation result
#' @param initialSituation boolean, TRUE to consider InitialTrainID for simulation, FALSE to consider SupplementaryTrainID for simulation
#' @importFrom purrr pmap map
#' @importFrom dplyr mutate
#' @return all_plots list 
#' 
#' @export
#' @examples
#' \donttest{
#' sc <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#'
#' regulation_reduction <- config_ambre$regulation$regulation_value |> 
#'   dplyr::filter(Country == "France") |>
#'   dplyr::select(-c(Concentration, Country, RegulationID))
#' regulation_concentration <- config_ambre$regulation$regulation_value |> 
#'   dplyr::filter(Country == "France") |>
#'   dplyr::select(-c(Country, RegulationID, Reduction))
#'
#' concentration_custom <- data.frame(PathogenName = c("Rotavirus"),
#'                                    min = c(1),
#'                                    max = c(2),
#'                                    type = c("uniform"))
#' run_qmra_custom(scenario = sc,
#'                   concentration = concentration_custom,
#'                   volume = data.frame(min=c(0.5,1), max = c(1,2), type= c("triangle", "triangle")),
#'                   frequency = c(200L, 300L),
#'                   regulationLog = regulation_reduction,
#'                   regulationConcentration = regulation_concentration,
#'                   initialSituation = TRUE)
#'
#' run_qmra_custom(scenario = sc,
#'                   concentration = concentration_custom,
#'                   volume = data.frame(min=c(0.5,1), max = c(1,2), type= c("triangle", "triangle")),
#'                   frequency = c(200L, 300L),
#'                   regulationLog = regulation_reduction,
#'                   regulationConcentration = regulation_concentration,
#'                   initialSituation = FALSE)
#'}
run_qmra_custom <- function(scenario,  
                              volume, 
                              frequency,
                              concentration, 
                              objective = 1e-6, 
                              regulationLog, 
                              regulationConcentration,
                              initialSituation) {
  
  
  if (initialSituation == TRUE){
    if(all(is.na(scenario$InitialTrainID))){
      stop("No value in column InitialTreatmentName of the scenario")
    }
  } else if (initialSituation == FALSE) {
    if(all(is.na(scenario$SupplementaryTrainID))){
      stop("No value in column SupplementaryTreatmentName of the scenario")
    }
  }

  sc  <- update_volume_with_desired_value(scenario = scenario, volume = volume)
  
  sc <- update_frequency_with_desired_value(scenario = sc, frequency = frequency)
  
  sc <- inflow_concentration_custom(scenario = sc, concentration = concentration)
  
  sc <- sc |> mutate(volume = map(config, ~ simulate_exposure(config = .x)))
  sc <- initial_dose_calculation(scenario = sc)
  sc <- update_treatment_scheme(scenario = sc, 
                                initial_situation = initialSituation)
  
  if (initialSituation == TRUE){
    sc$config <- pmap(
    list(
      config = sc$config,
      barrierID = sc$InitialProcessID,
      cropHeight = sc$CropHeight,
      matrixID = sc$MatrixID,
      popID = sc$PopulationID,
      decay = sc$nb_day_decay
    ),
    retrieve_specific_barrier
  )
  } else if (initialSituation == FALSE){
    sc$config <- pmap(
    list(
      config = sc$config,
      barrierID = sc$SupplementaryProcessID,
      cropHeight = sc$CropHeight,
      matrixID = sc$MatrixID,
      popID = sc$PopulationID,
      decay = sc$nb_day_decay
    ),
    retrieve_specific_barrier
  )
  }
  
  
  sc <- sc |> mutate(log_reduction = map(config, ~ simulate_treatment(.x)))
  sc <- final_dose_calculation(scenario = sc)
  sc <- infection_probability_calculation(scenario = sc)
  sc <- illness_probability_calculation(scenario = sc)
  sc <- dalys_calculation(scenario = sc)
  
  
  
  log_reduction_plot <- plot_logreduction(scenario = sc)
  
  matrix_log <- regulation_matrix_log(scenario = sc,
                                      regulation = regulationLog)
  
  matrix_concentration <- regulation_matrix_concentration(scenario = sc,
                                                          regulation = regulationConcentration)
  
  scenario_risk_total <- get_risk_total(sc)
  
  dalys_plot <- plot_dalys(scenario = scenario_risk_total,
                           objective = objective)
  
  
  
  all_plots <- list(
    logreduction = log_reduction_plot,
    dalys = dalys_plot,
    regul_matrix_log = matrix_log,
    regul_matrix_concentration = matrix_concentration
  )
  
  return(all_plots)
  
}

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("run_qmra_custom works", {
  scenario <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

regulation_reduction <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Country, RegulationID, Reduction))
concentration_custom <- data.frame(PathogenName = c("Rotavirus"),
                                   min = c(1),
                                   max = c(2),
                                   type = c("uniform"))
  result <- run_qmra_custom(scenario = scenario,
                  concentration = concentration_custom,
                  volume = data.frame(min=c(0.5,1), max = c(1,2), type= c("triangle", "triangle")),
                  frequency = c(200L, 300L),
                  regulationLog = regulation_reduction,
                  regulationConcentration = regulation_concentration,
                  initialSituation = TRUE)
  
  expect_equal(object = class(result),
               expected = "list")
  
  scenario = create_scenario(system.file("input_cas_apprentissage_complet.xlsx", package = "ambre"))
  n <- nrow(scenario)
  expect_error(object = run_qmra_custom(scenario = scenario,
                  concentration = concentration_custom,
                  volume = data.frame(min=rep(1, n), max = rep(2, n), type = rep("triangle", n)),
                  frequency = c(200L, 300L),
                  regulationLog = regulation_reduction,
                  regulationConcentration = regulation_concentration,
                  initialSituation = FALSE),
               regexp = "No value in column SupplementaryTreatmentName of the scenario") 
  
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Population Query
#' 
#' Query to return population ID from population Name using the database
#' @param config list configuration file default value is config_ambre dataset
#' @param popName character Name of the population for which you want to find the ID
#' @importFrom dplyr filter pull
#' @return popID integer ID of the population define
#' 
#' @export
#' @examples
#' query_pop(popName = "Irrigation staff")
query_pop <- function(config = config_ambre, popName){
  allowed_pop <- c("Irrigation staff", "Maintenance staff", "Harvesting staff", "Local residents", "Passerby", "Consumers")
  if (any(!popName %in% allowed_pop)) {
    stop("Population name doesn't exist")}
  
  popID <- config_ambre$path$description |> filter(PopulationName %in% popName) |> 
                                          pull(PopulationID) |> unique()
  return(popID)
    
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("query_pop works", {
  expect_equal(object = query_pop(popName = "Irrigation staff"), 
               expected = 1) 
})

test_that("query_pop errors", {
  expect_error(object = query_pop(popName = "Maintainer"),
               regexp = "Population name doesn't exist")
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Annual cost
#' 
#' Calculate annual cost of the scenario simulated
#' @param scenario data.frame scenario obtained with create_scenario function
#' @param price_per_m3 numeric, water price in euro per cubic meter
#' @param membership_fee numeric annual membership fee in euro per hectare
#' @param grant numeric amount of the grant in percentage 
#' @param initialSituation boolean TRUE to simulate initial situation cost, FALSE to simulation supplementary process cost
#' @param allocation_key numeric allocation key, NULL by default
#' @importFrom tidyr pivot_longer
#' @import ggplot2
#' @return result list
#' 
#' @export
#' @examples
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' annual_cost(scenario = scenario_apprentissage,
#'             price_per_m3 = 0.1,
#'             membership_fee = 200,
#'             grant = 0.5,
#'             initialSituation = TRUE)
#'
#' annual_cost(scenario = scenario_apprentissage,
#'             price_per_m3 = 0.1,
#'             membership_fee = 200,
#'             grant = 0.5,
#'             initialSituation = FALSE)
#'
#' crop <- scenario_apprentissage$CropName
#' allocation_custom <- data.frame(CropName = crop,
#'                                 allocation = c(0.5, 0.5))
#' annual_cost(scenario = scenario_apprentissage,
#'             price_per_m3 = 0.1,
#'             membership_fee = 200,
#'             grant = 0,
#'             initialSituation = TRUE,
#'             allocation_key = allocation_custom)
#'
#' annual_cost(scenario = scenario_apprentissage,
#'             price_per_m3 = 0.1,
#'             membership_fee = 200,
#'             grant = 0,
#'             initialSituation = FALSE,
#'             allocation_key = allocation_custom)
#'
annual_cost <- function(scenario, price_per_m3, membership_fee, grant, initialSituation, allocation_key = NULL){
  irrigation_need <- irrigation_need_calculation(scenario = scenario)

  # 1. Water price

  irrigation_need_with_waterprice <- water_price(scenario = irrigation_need,
              price_per_m3 = price_per_m3,
              membership_fee = membership_fee)
  
  
  # 2. Mesure collective
  if (all(!is.na(irrigation_need_with_waterprice$CollectiveTreatmentID))) {
    if (is.null(allocation_key)){
      irrigation_need_with_collectivecost <- collective_treatment_cost_calculation(
                        scenario = irrigation_need_with_waterprice,
                        grant = grant
                        )
    } else {
      irrigation_need_with_collectivecost <- collective_treatment_cost_calculation(
                        scenario = irrigation_need_with_waterprice,
                        grant = grant,
                        allocation_key = allocation_key)
    }
  
} else {
  irrigation_need_with_collectivecost <- list(scenario = irrigation_need_with_waterprice,
                                                    allocation_key = NA)
  irrigation_need_with_collectivecost$scenario <-
  irrigation_need_with_collectivecost$scenario %>%
  mutate(
    capex_collective_treatment = NA,
    collective_opex = NA,
    loan_asa = NA
  )
}
  
  
  # 3. Mesure individuelle : opex et capex
  if(initialSituation == TRUE){
    irrigation_need_with_mesurecost <- process_cost_calculation(scenario = irrigation_need_with_collectivecost$scenario, 
                                                              membership_fee = membership_fee, 
                                                              charge = price_per_m3,
                                                              initialSituation = TRUE)
  }
  else if(initialSituation == FALSE){
    irrigation_need_with_mesurecost <- process_cost_calculation(scenario = irrigation_need_with_collectivecost$scenario, 
                                                              membership_fee = membership_fee, 
                                                              charge = price_per_m3,
                                                              initialSituation = FALSE)
  }
  
  
  
  irrigation_need_long <- irrigation_need_with_mesurecost |>
    select(c(CropName, annual_membership_fee, loan_asa, 
             water_charge, collective_opex, opex_supplementaryProcess, capex_supplementaryProcess_20y)) |>
    rename("Annual membership fee" = annual_membership_fee,
           "Water price" = water_charge,
           "Operational cost collective process" = collective_opex,
           "Investment collective process (20-year loan)" = loan_asa,
           "Operational cost individual process" = opex_supplementaryProcess,
           "Investment individual process (20-year loan)" = capex_supplementaryProcess_20y)|>
    pivot_longer(cols = !CropName, names_to = "cost_type", values_to = "value") |>
    mutate(
    cost_type = factor(
      cost_type,
      levels = c(
        "Water price",
        "Annual membership fee",
        "Operational cost collective process",
        "Investment collective process (20-year loan)",
        "Operational cost individual process",
        "Investment individual process (20-year loan)"
      )
    )
  )|>
    unique()
  
  plot <- ggplot(irrigation_need_long, aes(x = CropName, 
                                   y = value, 
                                   fill = cost_type, 
                                   label = value))+
    geom_bar(stat = "identity", position = position_stack(reverse = TRUE)) +
    geom_text(position = position_stack(vjust = 0.5, reverse = TRUE))+
    scale_fill_manual(
      values = c(
        "Water price" = "palegreen4",
        "Annual membership fee" = "palegreen",
        "Operational cost collective process"  = "#fedc7f",
        "Investment collective process (20-year loan)" = "#d28433",
        "Operational cost individual process"  = "#9bdde1",
        "Investment individual process (20-year loan)" = "#167779"
      )
    )+
    labs(fill = "Cost Type", 
         y = "Annual cost (EUR/year)",
         x = "Crop name") + 
    theme_minimal() + 
    theme(legend.position = "bottom")
  
  result <- list(
    plot_annual_cost = plot,
    allocation_key = irrigation_need_with_collectivecost$allocation_key
  )
  return(result)
    
}

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("annual_cost works", {
  scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
  result <- annual_cost(scenario = scenario_apprentissage,
            price_per_m3 = 0.1,
            membership_fee = 200,
            grant = 0.5,
            initialSituation = TRUE)
  expect_true(all(c("plot_annual_cost", "allocation_key") %in% names(result)))
  expect_equal(object = result$allocation_key,
               expected = structure(list(grant = c(0.5, 0.5), CropName = c("Tomato", "Corn seed"
), allocation = c(0.1111, 0.3889)), row.names = 1:2, class = "data.frame"))
  
  crop <- scenario_apprentissage$CropName
  allocation_custom <- data.frame(CropName = crop,
                                  allocation = c(0.5, 0.5))
  result <- annual_cost(scenario = scenario_apprentissage,
              price_per_m3 = 0.1,
              membership_fee = 200,
              grant = 0,
              initialSituation = TRUE,
              allocation_key = allocation_custom)
  
  expect_equal(object = result$allocation_key,
               expected = structure(list(grant = c(0, 0), CropName = c("Tomato", "Corn seed"
), allocation = c(0.5, 0.5)), row.names = 1:2, class = "data.frame"))
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Update monte carlo value
#' 
#' Update number_of_repeating in config$exposure database
#' @param scenario data.frame scenario obtained with create_scenario function
#' @param monte_carlo integer number of simulation iterations for Monte Carlo
#' @return scenario_updated
#' 
#' @export
#' @examples
#' scenario_example <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' scenario_updated <- update_monte_carlo(scenario = scenario_example, 
#'                    monte_carlo = 2000)
#' scenario_updated$config[[1]]$exposure
update_monte_carlo <- function(scenario, monte_carlo = 1000) {
  
  new_frequency <- map(.x = scenario$PathID, .f = ~ query_frequency(.x))   # $min et $max

  scenario_updated <- scenario |>
    mutate(
      config = map(
        .x = config,
        .f = ~{
          .x$exposure$value[[1]] <- monte_carlo 
          .x                          
        }
      )
    )
  return(scenario_updated)
    
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("update_monte_carlo works", {
  scenario_example <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  scenario_updated <- update_monte_carlo(scenario = scenario_example, 
                     monte_carlo = 2000)
  
  difference <- scenario_updated$config[[1]]$exposure$value[[1]] - scenario_example$config[[1]]$exposure$value[[1]]
  scenario_updated$config[[1]]$exposure
  expect_equal(object = difference,
               expected = 1000) 
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' get_stop_text
#' 
#' This function does the same thing as kwb.utils::get_stop_text(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' Description
#' @param keyword character text to evaluate
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @return text
#' 
#' @noRd
#' @examples
#' get_stop_text("must_be_value")
get_stop_text <- function(keyword = NULL)
{
  texts <- list(
    doseresponse_config_incomplete = paste0(
      "Doseresponse configuration incomplete for pathogen %s.\n",
      "Define required parameter(s), either k (for exponential model) or\n",
      "alpha & N50 (for beta-poisson model)"
    ),
    must_be_value = paste0(
      "%s in configuration 'exposure' MUST be of 'type' ", 
      "= 'value' and column 'value' has to be numeric"
    ),
    no_such_distribution = paste0(
      "Your value for parameter 'type' = %s is not an implemented distribution.\n",
      "Valid values of 'type' are:\n",
      "'uniform', 'triangle', 'norm', 'lognorm', 'log10_uniform' or 'log10_norm'"
    )
  )
  
  if (is.null(keyword)) {
    return(texts)
  }
  
  if (! keyword %in% names(texts)) {
    stop(
      "No such keyword: ", keyword, ". Available keywords:\n",
      paste(names(texts), collapse = ", "), call. = FALSE
    )
  }
  
  texts[[keyword]]
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("get_stop_text works", {
  expect_equal(object = get_stop_text("must_be_value"),
               expected = "%s in configuration 'exposure' MUST be of 'type' = 'value' and column 'value' has to be numeric") 
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Update volume with desired value
#' 
#' Update the min and max value of the volume in the simulated configuration according to a value defined by the user
#' 
#' @param scenario data.frame
#' @param volume numeric, named list of min and max value to simulate
#' @importFrom purrr map2
#' @importFrom dplyr mutate
#' @return scenario_updated
#' 
#' @export
#' @examples
#' scenario_example <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' volumes <- data.frame("min" = c(1.1, 1.2), "max" = c(1.3,1.4), "type" = c("triangle", "triangle"))
#' scenario_example_updated <- update_volume_with_desired_value(scenario = scenario_example,
#'                                  volume = volumes)
#'
#' print(scenario_example_updated$config[[1]]$exposure)
#' print(scenario_example_updated$config[[2]]$exposure)
update_volume_with_desired_value <- function(scenario, volume){
  
  if (isFALSE("min" %in% names(volume))){
    stop("List of volume must contain min values")
  }
  
  if (isFALSE("max" %in% names(volume))){
    stop("List of volume must contain min values")
  }
  
  if (isFALSE("type" %in% names(volume))){
    stop("List of volume must contain type of the distribution")
  }
  
  if (isFALSE(length(volume$min == length(scenario$config)))){
    stop("A min volume value is required for all lines in the scenario")
  } 
  
  if (isFALSE(length(volume$max == length(scenario$config)))){
    stop("A max volume value is required for all lines in the scenario")
  } 
  
  
  scenario_updated <- scenario |>
    mutate(
      config = map2(
        .x = config,
        .y = volume$min,
        .f = ~{
          .x$exposure$min[[3]] <- .y
          .x                          
        }
      )
    ) |>
    mutate(
      config = map2(
        .x = config,
        .y = volume$max,
        .f = ~{
          .x$exposure$max[[3]] <- .y
          .x
        }
      )
    ) |>
    mutate(
      config = map2(
        .x = config,
        .y = volume$type,
        .f = ~{
          .x$exposure$type[[3]] <- .y
          .x
        }
      )
    )
  return(scenario_updated)
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("update_volume_with_desired_value works", {
  scenario_example <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  volumes <- data.frame("min" = c(1.1, 1.2), "max" = c(1.3,1.4), "type" = c("triangle", "triangle"))
  result <- update_volume_with_desired_value(scenario = scenario_example,
                                 volume = volumes)
  expect_equal(object = result$config[[1]]$exposure$min[[3]], 
               expected = 1.1) 
  expect_equal(object = result$config[[1]]$exposure$max[[3]], 
               expected = 1.3) 
  
  expect_equal(object = result$config[[2]]$exposure$min[[3]], 
               expected = 1.2) 
  expect_equal(object = result$config[[2]]$exposure$max[[3]], 
               expected = 1.4)
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Quote Character Strings
#' 
#' quotes objects of type character with quoting character
#' 
#' This function does the same thing as kwb.utils::hsQuoteChr(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' @param x vector or list of character strings
#' @param qchar quoting character to be used. Default: single quote "'"
#' @param escapeMethod one of \code{"double", "backslash", "none"} deciding
#'   how to treat the quote character if it occurs within the string to be
#'   quoted
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' 
#' @examples
#' # Example usage of hsQuoteChr
#' hsQuoteChr("Hello, world!", qchar = "'")
#' hsQuoteChr("He said, 'Hello!'", qchar = "'", escapeMethod = "backslash")
#' hsQuoteChr("He said, 'Hello!'", qchar = "'", escapeMethod = "double")
hsQuoteChr <- function(
  x, qchar = "'", escapeMethod = c("double", "backslash", "none")
) 
{
  if (mode(x) == "list") {
    
    len <- length(x)
    
    if (len == 0) return (NULL)
    
    if (len == 1) return (hsQuoteChr(x[[1]]))
    
    return (c(hsQuoteChr(x[[1]]), hsQuoteChr(x[-1])))
  }
  
  escapeChars <- list(double = qchar, backslash = '\\', none = "")
  
  if (mode(x) == "character") {
    
    escapeChar <- escapeChars[[escapeMethod[1]]]
    
    x.escaped <- gsub(qchar, paste0(escapeChar, qchar), x)
    
    return (paste0(qchar, x.escaped, qchar))
  }
  
  x
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("hsQuoteChr works correctly with single quotes", {
  expect_equal(hsQuoteChr("Hello", qchar = "'"), "'Hello'")
  expect_equal(hsQuoteChr("He said, 'Hello!'", qchar = "'", escapeMethod = "backslash"), "'He said, 'Hello!''")
  expect_equal(hsQuoteChr("He said, 'Hello!'", qchar = "'", escapeMethod = "double"), "'He said, ''Hello!'''")
})

test_that("hsQuoteChr works correctly with double quotes", {
  expect_equal(hsQuoteChr("Hello", qchar = "\""), "\"Hello\"")
  expect_equal(hsQuoteChr("He said, \"Hello!\"", qchar = "\"", escapeMethod = "backslash"), "\"He said, \"Hello!\"\"")
  expect_equal(hsQuoteChr("He said, \"Hello!\"", qchar = "\"", escapeMethod = "double"), "\"He said, \"\"Hello!\"\"\"")
})

test_that("hsQuoteChr works correctly with lists", {
  expect_equal(hsQuoteChr(list("Hello", "world"), qchar = "'"), c("'Hello'", "'world'"))
  expect_equal(hsQuoteChr(list("He said, 'Hello!'"), qchar = "'", escapeMethod = "backslash"), "'He said, ''Hello!'''")
  expect_equal(hsQuoteChr(list(), qchar = "'"), NULL)
})

test_that("hsQuoteChr handles non-character inputs", {
  expect_equal(hsQuoteChr(123, qchar = "'"), 123)
  expect_equal(hsQuoteChr(c(TRUE, FALSE), qchar = "'"), c(TRUE, FALSE))
})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Dalys calculation
#' 
#' Calculate dalys lose per event according to illness probability
#' @param scenario data.frame scenario with risk column
#' @return scenario_updated
#' 
#' @export
#' @examples
#' library(dplyr)
#' library(purrr)
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#'
#' scenario_conc <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni"))
#'
#' scenario_volume <-
#'   scenario_conc |>                     
#'   mutate(
#'     volume = map(
#'       .x = config,                                         
#'       .f = ~ simulate_exposure(config = .x)              
#'     )
#'   )
#'
#' scenario_dose_ini <- initial_dose_calculation(scenario_volume)
#'
#' scenario_scheme <- update_treatment_scheme(scenario_dose_ini)
#'
#' scenario_with_logreduc_and_co <- scenario_scheme |> 
#'   mutate( log_reduction = map(config, simulate_treatment))
#'
#' scenario_final_dose_and_co_test <- final_dose_calculation(scenario_with_logreduc_and_co)
#'
#' scenario_inf_proba_and_co <- infection_probability_calculation(scenario_final_dose_and_co_test)
#'
#' scenario_illness_proba_and_co <- illness_probability_calculation(scenario_inf_proba_and_co)
#'
#' dalys_calculation(scenario_illness_proba_and_co)
dalys_calculation <- function(scenario){
  risk_illness <- map(.x = scenario$risk, .f = ~mutate(.x, dalys_per_event = .x$illnessProb_per_event * .x$dalys_per_case * .$fraction_susceptibilite))
  
  scenario_updated <- scenario |> 
    mutate(risk = risk_illness)
    
}

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

#' Plot volume
#' 
#' Boxplot of the volume of exposition for each population exposed
#' @param scenario data.frame with volume column obtained with simulate_exposure function
#' @importFrom dplyr mutate
#' @importFrom tidyr unnest
#' @import ggplot2
#' @import colorspace
#' @importFrom stringr str_wrap
#' @importFrom purrr map
#' @return plot
#' 
#' @export
#' @examples
#' library(dplyr)
#' library(purrr)
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#'
#' scenario_conc <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni"))
#'
#' scenario_volume <-
#'   scenario_conc |>                     
#'   mutate(
#'     volume = map(
#'       .x = config,                                         
#'       .f = ~ simulate_exposure(config = .x)              
#'     )
#'   )
#'
#' plot_volumes(scenario_volume)
plot_volumes <- function(scenario){
  if (!"volume" %in% names(scenario)) {
  stop("The volume column is missing from the scenario object; run the simulate_exposure function to create it")
  }
  
  plot_vol <- scenario |> select(CropName, PopulationName, PathName) |>
            mutate(volume_perEvent = map(.x = scenario$volume, 
                                         .f = ~.x$volumes$events$volume_perEvent))


  plot_vol_long <- plot_vol |> unnest(volume_perEvent)
  
  n_path <- length(unique(plot_vol_long$PathName))

  plot <- ggplot(plot_vol_long, aes(x = PopulationName, y = volume_perEvent*1000, fill = PathName)) +
              geom_boxplot() +
              facet_grid(~CropName) +
              theme_minimal() +
              labs(x = "Population exposed",
                   y = "(mL/event)",
                   fill = "Exposition path"
              )+
              scale_fill_manual(
                values = rainbow_hcl(n_path),
                labels = \(x) str_wrap(x, width = 60)
              ) +
               guides(
                fill = guide_legend(
                  nrow = length(unique(plot_vol_long$PathName)),
                  byrow = TRUE
                )
              ) +
              theme_minimal() +
              theme(
                legend.position = "bottom",
                legend.direction = "horizontal",
                axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1)
              )
  
  return(plot)
    
}

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

#' Plot comparison QMRA calculation with initial vs supplementary processes
#' 
#' Compare the result of the QMRA simulation with reduction_log from initial vs supplementary processes
#' @param scenario data.frame, scenario obtained with create_scenario function
#' @param pathogen character, Name of the pathogen to simulate
#' @param regulationLog data.frame regulation value of the log reduction to compare with simulation result
#' @param regulationConcentration data.frame regulation value of the log reduction to compare with simulation result
#' @import cowplot
#' @import ggplot2
#' @importFrom purrr map2
#' @return combine_plots
#' 
#' @export
#' @examples
#' \donttest{
#' library(dplyr)
#' scenario_example <- create_scenario(
#'   filepath = system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#'
#'
#' regulation_reduction <- config_ambre$regulation$regulation_value |> 
#'   dplyr::filter(Country == "France") |>
#'   dplyr::select(-c(Concentration, Country, RegulationID))
#' regulation_concentration <- config_ambre$regulation$regulation_value |> 
#'   dplyr::filter(Country == "France") |>
#'   dplyr::select(-c(Country, RegulationID, Reduction))
#'
#' plot_comparison_qmra_initial_vs_supplementary_processes(
#'   scenario = scenario_example,
#'   pathogen = c("Campylobacter jejuni", "Norovirus"),
#'   regulationLog = regulation_reduction,
#'   regulationConcentration = regulation_concentration
#' )
#' }
plot_comparison_qmra_initial_vs_supplementary_processes <- function(scenario, pathogen, regulationLog, regulationConcentration){

  qmra_initial <- run_qmra_initial_situation(scenario = scenario, 
                       pathogen = pathogen,
                       regulationLog = regulationLog,
                       regulationConcentration = regulationConcentration)
  qmra_supplementary <- run_qmra_supplementary_process(scenario = scenario, 
                               pathogen = pathogen,
                               regulationLog = regulationLog,
                               regulationConcentration = regulationConcentration )

  combine_log_reduction <- map2(
              qmra_initial$logreduction,
              qmra_supplementary$logreduction,
              ~ plot_grid(
      .x +
        labs(subtitle = "Initial Situation") +
        theme(plot.subtitle = element_text(hjust = 0.5)),
      .y +
        labs(subtitle = "Supplementary Process") +
        theme(plot.subtitle = element_text(hjust = 0.5)),
      ncol = 2,
      align = "h"
    )
)
  combine_dalys <- map2(
  qmra_initial$dalys,
  qmra_supplementary$dalys,
  ~ plot_grid(
      .x +
        labs(subtitle = "Initial Situation") +
        theme(plot.subtitle = element_text(hjust = 0.5)),
      .y +
        labs(subtitle = "Supplementary Process") +
        theme(plot.subtitle = element_text(hjust = 0.5)),
      ncol = 2,
      align = "h"
    )
)
    
  combine_plots <- list(
    log_reduction = combine_log_reduction,
    dalys = combine_dalys)
  
  return(combine_plots)
}

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("plot_comparison_qmra_treatment_vs_barrier works", {
  scenario_test <- create_scenario(
    filepath = system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  regulation_reduction <- config_ambre$regulation$regulation_value |> dplyr::filter(Country == "France") |>
                          dplyr::select(-c(Concentration, Country, RegulationID))
  regulation_concentration <- config_ambre$regulation$regulation_value |> dplyr::filter(Country == "France") |>
                          dplyr::select(-c(Country, RegulationID, Reduction))
  
  result <- plot_comparison_qmra_initial_vs_supplementary_processes(
    scenario = scenario_test,
    pathogen = c("Campylobacter jejuni"),
    regulationLog = regulation_reduction,
    regulationConcentration = regulation_concentration)
  expect_equal(object = names(result),
               expected = c("log_reduction", "dalys")) 
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Run economic analysis
#' 
#' Calculate annual cost and total investment fees for simulated management risk scenario
#' @param scenario data.frame scenario obtained with create_scenario function
#' @param price_per_m3 numeric, water price in euro per cubic meter
#' @param membership_fee numeric annual membership fee in euro per hectare
#' @param grant numeric amount of the grant in percentage
#' @param initialSituation boolean, TRUE to simulate initial situation cost, FALSE to simulate supplementary process cost
#' @param allocation_key numeric allocation key, NULL by default
#' @return all_plots
#' 
#' @export
#' @examples
#' scenario <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#' run_economic_analysis(scenario = scenario,
#'                       price_per_m3 = 0.1, 
#'                       membership_fee = 200, 
#'                       grant = 0.5,
#'                       initialSituation = TRUE)
#' run_economic_analysis(scenario = scenario,
#'                       price_per_m3 = 0.1, 
#'                       membership_fee = 200, 
#'                       grant = 0.5,
#'                       initialSituation = FALSE)
#'
#' crop <- scenario$CropName
#' allocation_custom <- data.frame(CropName = crop,
#'                                 allocation = c(0.5, 0.5))
#' run_economic_analysis(scenario = scenario,
#'                       price_per_m3 = 0.1, 
#'                       membership_fee = 200, 
#'                       grant = 0,
#'                       initialSituation = TRUE,
#'                       allocation_key = allocation_custom)
run_economic_analysis <- function(scenario, price_per_m3, membership_fee, grant, initialSituation, allocation_key = NULL){
  irrigation_need <- irrigation_need_calculation(scenario = scenario)
  
  # 1. Water price

  irrigation_need_with_waterprice <- water_price(scenario = irrigation_need,
              price_per_m3 = price_per_m3,
              membership_fee = membership_fee)
  
  
  # 2. Mesure collective
  if (all(!is.na(irrigation_need_with_waterprice$CollectiveTreatmentID))) {
    if (is.null(allocation_key)){
      irrigation_need_with_collectivecost <- collective_treatment_cost_calculation(
                        scenario = irrigation_need_with_waterprice,
                        grant = grant
                        )
    } else {
      irrigation_need_with_collectivecost <- collective_treatment_cost_calculation(
                        scenario = irrigation_need_with_waterprice,
                        grant = grant,
                        allocation_key = allocation_key)
    }
  
} else {
  irrigation_need_with_collectivecost <- list(scenario = irrigation_need_with_waterprice,
                                                    allocation_key = NA)
  irrigation_need_with_collectivecost$scenario <-
  irrigation_need_with_collectivecost$scenario |>
  mutate(
    capex_collective_treatment = NA,
    collective_opex = NA,
    loan_asa = NA
  )
}
  
  
  # 3. Mesure individuelle : opex et capex
  
  if(initialSituation == TRUE){
    irrigation_need_with_mesurecost <- process_cost_calculation(scenario = irrigation_need_with_collectivecost$scenario, 
                                                              membership_fee = membership_fee, 
                                                              charge = price_per_m3,
                                                              initialSituation = TRUE)
  }
  else if(initialSituation == FALSE){
    irrigation_need_with_mesurecost <- process_cost_calculation(scenario = irrigation_need_with_collectivecost$scenario, 
                                                              membership_fee = membership_fee, 
                                                              charge = price_per_m3,
                                                              initialSituation = FALSE)
  }
  
  
  
  irrigation_need_long <- irrigation_need_with_mesurecost |>
    select(c(CropName, annual_membership_fee, loan_asa, 
             water_charge, collective_opex, opex_supplementaryProcess, capex_supplementaryProcess_20y)) |>
    rename("Annual membership fee" = annual_membership_fee,
           "Water price" = water_charge,
           "Operational cost collective process" = collective_opex,
           "Investment collective process (20-year loan)" = loan_asa,
           "Operational cost individual process" = opex_supplementaryProcess,
           "Investment individual process (20-year loan)" = capex_supplementaryProcess_20y)|>
    pivot_longer(cols = !CropName, names_to = "cost_type", values_to = "value") |>
    mutate(
    cost_type = factor(
      cost_type,
      levels = c(
        "Water price",
        "Annual membership fee",
        "Operational cost collective process",
        "Investment collective process (20-year loan)",
        "Operational cost individual process",
        "Investment individual process (20-year loan)"
      )
    )
  )|>
    unique()
  
  
    plot_annual <- ggplot(irrigation_need_long, aes(x = CropName, 
                                     y = value, 
                                     fill = cost_type, 
                                     label = value))+
      geom_bar(stat = "identity") +
      geom_text(size = 3, position = position_stack(vjust = 0.5)) +
      scale_fill_manual(
        values = c(
          "Water price" = "palegreen4",
        "Annual membership fee" = "palegreen",
        "Operational cost collective process"  = "#fedc7f",
        "Investment collective process (20-year loan)" = "#d28433",
        "Operational cost individual process"  = "#9bdde1",
        "Investment individual process (20-year loan)" = "#167779"
        )
      )+
      labs(fill = "Cost Type", y = "Annual cost (EUR/year)") + 
      theme_minimal()
  if(initialSituation == TRUE){
    result <- list(annual = plot_annual,
                      allocation_key = irrigation_need_with_collectivecost$allocation_key)
  }
  
  else if(initialSituation == FALSE){
    total_investment <- irrigation_need_with_mesurecost |>
    select(c(CropName, capex_supplementaryProcess, capex_collective_treatment)) |>
    rename("Investment collective process" = capex_collective_treatment,
           "Investment individual process" = capex_supplementaryProcess) |>
    pivot_longer(cols = !CropName, names_to = "cost_type", values_to = "value") |>
    unique()
  
    plot_total <- ggplot(total_investment, aes(x = CropName, 
                                   y = value, 
                                   fill = cost_type, 
                                   label = value))+
    geom_bar(stat = "identity") +
    geom_text(size = 3, position = position_stack(vjust = 0.5)) +
    scale_fill_manual(
      values = c(
        "Investment collective process" = "#d28433",
        "Investment individual process" = "#167779"
      )
    )+ 
    labs(fill = "Cost Type", y = "Total cost (EURO)") + 
    theme_minimal() + 
    theme(legend.position = "bottom")
  
  
  result <- list(annual = plot_annual,
                   total_investement = plot_total,
                   allocation_key = irrigation_need_with_collectivecost$allocation_key)
  }
  return(result)
    
}

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("run_economic_analysis works", {
  scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
  result <- run_economic_analysis(scenario = scenario_apprentissage,
                      price_per_m3 = 0.1, 
                      membership_fee = 200, 
                      grant = 0.5,
                      initialSituation = TRUE)
  expect_equal(object = (names(result)),
               expected = c("annual", "allocation_key"))
  expect_false("total_investement" %in% names(result))
  
  result <- run_economic_analysis(scenario = scenario_apprentissage,
                      price_per_m3 = 0.1, 
                      membership_fee = 200, 
                      grant = 0.5,
                      initialSituation = FALSE)
  expect_equal(object = (names(result)),
               expected = c("annual", "total_investement", "allocation_key"))
})

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

#' Plot inflow concentration
#' 
#' Violin plot of inflow concentration according to pathogen simulated
#' @param scenario data.frame with inflow_concentration column obtained from inflow_concentration function
#' @import ggplot2
#' @importFrom purrr map
#' @return plot
#' 
#' @export
#' @examples
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#' scenario_with_inflow <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni", "Norovirus"))
#' plot_inflow(scenario_with_inflow)
plot_inflow <- function(scenario){
  pathoName <- map(.x = scenario$inflow_concentration, .f = ~.x$events$PathogenName)
  pathoGroup <- map(.x = scenario$inflow_concentration, .f = ~.x$events$PathogenGroup)
  inflow_concentration <- map(.x = scenario$inflow_concentration, .f = ~.x$events$inflow)

  plot_data <- data.frame(PathogenName = unlist(pathoName),
                          PathogenGroup = unlist(pathoGroup),
                          inflow = unlist(inflow_concentration))
  
  fill_color <- c("Bacteria" = "palegreen3", 
                  "Viruses"  = "orchid", 
                  "Protozoa" = "darkorange")
  
  plot <- ggplot(
          data = plot_data,
          mapping = aes(x = PathogenName,
                                 y = inflow / 10,
                                 fill = PathogenGroup)
       ) +
    geom_violin(trim = FALSE,
                         alpha = 0.7,
                         colour = "black") +
    scale_y_log10() +
    scale_fill_manual(values = fill_color,
                               name = "Pathogen group") +
    theme_minimal() +
    labs(
      x = "Names of the pathogen simulated",
      y = "log/100mL"
    )
  
  return(plot)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' ID columns to integer
#' 
#' This function does the same thing as kwb.utils::id_columns_to_integer(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' ID conversion to integer
#' @param data list
#' @return data integer
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' @references 
#'          https://github.com/KWB-R/kwb.qmra/blob/master/R/utils.R 
#' @examples
#' data_test <- data.frame(PathogenID = c(5.5))
#' id_columns_to_integer(data_test)
id_columns_to_integer <- function(data)
{
  # Which ID columns require conversion?
  to_convert <- grepl("ID$", names(data)) & ! sapply(data, is.integer)
  
  data[to_convert] <- lapply(data[to_convert], as.integer)
  
  data
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("id_columns_to_integer works", {
  data_test <- data.frame(PathogenID = c(5.5))

  expect_equal(object = id_columns_to_integer(data_test), 
               expected = structure(list(PathogenID = 5L), row.names = c(NA, -1L), class = "data.frame")) 
})

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

#' Query irrigation need
#' 
#' Retrieve irrigation need according to crop ID 
#' @param scenario data.frame scenario obtained with create_scenario function
#' @importFrom dplyr left_join
#' @return scenario_updated
#' 
#' @export
#' @examples
#' scenario_apprentissage <- create_scenario(
#'   system.file("input_1culture_2pop.xlsx", package = "ambre")
#' )
#'
#' irrigation_need <- query_irrigation_need(scenario_apprentissage)
#' print(irrigation_need$IrrigationNeed)
query_irrigation_need <- function(scenario){
  scenario$CropID <- unlist(scenario$CropID)
  scenario$CropName <- unlist(scenario$CropName)

  scenario_updated <- scenario |>
                      left_join(config_ambre$economic$water_need,
                                by = c("CropID", "CropName"))
  
  return(scenario_updated)
    
}

# WARNING - Generated by {fusen} from dev/flat_economic.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("query_irrigation_need works", {
  scenario_apprentissage <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  result <- query_irrigation_need(scenario_apprentissage)
  expect_true("IrrigationNeed" %in% names(result)) 
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Create Exposure Scenario
#'
#' This function creates exposure scenarios during the two game turns.
#'
#' @param filepath Path to the Excel file containing input data
#' @param config list config file default value config_ambre
#' @importFrom readxl read_excel
#' @importFrom dplyr mutate pick
#' @importFrom purrr map
#' @return A list containing the scenario data and a placeholder for future data
#' @export
#' @examples
#' scenario <- create_scenario(system.file("input_cas_apprentissage_complet.xlsx", package = "ambre"))
create_scenario <- function(filepath, config = config_ambre) {
  if (!is.character(filepath)) {
    stop("filename must be a string")
  }
  if (!file.exists(filepath)) {
    stop("filename must exist")
  }
  
  # load user input
  user_input <- read_excel(filepath)
  
  
  if (!is.numeric(user_input$Area)) {
    stop("Area must be a numeric value")
  }
  
  if (!is.numeric(user_input$nb_population)) {
    stop("nb_population must be a numeric value")
  }
  
  if (any(user_input$Area <= 0) || any(user_input$Area >= 10000)) {
    stop("Area must be strictly greater than 0 ha and smaller than 10,000 ha")
  }
  
  if (any(user_input$nb_population <= 0) ||
      any(user_input$nb_population >= 5000)) {
    stop("nb_population must be strictly greater than 0 and smaller than 5,000")
  }
  
  # add config file to each line
  scenario <- user_input |>  mutate(config = list(config))
  
  # retrieve ID
  cropID <- as.numeric(map(.x = scenario$CropName,
                .f = ~ query_crop(cropName = .x)))
  cropHeight <- as.numeric(map(.x = scenario$CropName,
                .f = ~ query_crop_height(cropName = .x)))
  popID <- as.numeric(map(.x = scenario$PopulationName,
               .f = ~ query_pop(popName = .x)))
  pathID <- as.numeric(map(.x = scenario$PathName,
                .f = ~ query_exp_path(pathName = .x)))
  matrixID <- as.numeric(map(.x = scenario$PathName,
                  .f = ~ query_matrix(pathName = .x)))
  STEPtttID <- map(.x = scenario$STEPtreatmentName,
                   .f = ~ query_barrier(barrierName = .x))
  collectiveTreatmentID <- map(
    .x = scenario$CollectiveTreatmentName,
    .f = ~ query_barrier(barrierName = .x)
  )
  initialProcessID <- map(
    .x = scenario$InitialProcessName,
    .f = ~ query_barrier(barrierName = .x)
  )
  supplementaryProcessID <- map(
    .x = scenario$SupplementaryProcessName,
    .f = ~ query_barrier(barrierName = .x)
  )
  
  scenario <- scenario |> mutate(
    CropID = cropID,
    CropHeight = cropHeight,
    PopulationID = popID,
    PathID = pathID,
    MatrixID = matrixID,
    STEPtreatmentID = STEPtttID,
    CollectiveTreatmentID = collectiveTreatmentID,
    InitialProcessID = initialProcessID,
    SupplementaryProcessID = supplementaryProcessID
  )
  
  scenario <- scenario |> mutate(
    InitialTrainName = pmap(
      pick(
        STEPtreatmentName,
        CollectiveTreatmentName,
        InitialProcessName
      ),
      ~ unname(
        trimws(
          unlist(
            strsplit(
              c(...)[!is.na(c(...))],
              ","
            )
          )
        ) |>
          (\(x) x[x != ""])()
      )
    )
  )
  
  scenario <- scenario |> mutate(InitialTrainID = pmap(
    pick(
      STEPtreatmentID,
      CollectiveTreatmentID,
      InitialProcessID
    ),
    ~ unname(c(...)[!is.na(c(...))])
  ))
  
  if (all(is.na(scenario$SupplementaryProcessName))) {

  scenario <- scenario |>
    mutate(SupplementaryTrainName = NA) |>
    mutate(SupplementaryTrainID = NA)

} else {
  
    scenario <- scenario |> mutate(SupplementaryTrainName = pmap(
      pick(STEPtreatmentName, CollectiveTreatmentName, InitialProcessName, SupplementaryProcessName),
      ~ unname(
          trimws(
            unlist(
              strsplit(
                c(...)[!is.na(c(...))],
                ","
              )
            )
          ) |>
            (\(x) x[x != ""])()
        )
      )
    )
    
    scenario <- scenario |> mutate(SupplementaryTrainID = pmap(
      pick(STEPtreatmentID, CollectiveTreatmentID, InitialProcessID, SupplementaryProcessID),
      ~ unname(c(...)[!is.na(c(...))])
  ))}
  
  
  
  return(scenario)
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

library(tibble)
test_that("update_frequency works", {
  scenario_example <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  scenario_updated <- update_frequency(scenario_example)
  
  # check the value is updated
  expect_equal(object = scenario_updated$config[[1]]$exposure$value[[2]],
               expected = 60)
  
  # check the other values remain unchanged
  example_exposure_unchanged_values <- scenario_example$config[[1]]$exposure |> slice(-2)
  test_exposure_unchanged_values <- scenario_updated$config[[1]]$exposure |> slice(-2)
  
  expect_equal(object = test_exposure_unchanged_values, 
               expected = example_exposure_unchanged_values)
  
  # check the type
  expect_true(is_tibble(scenario_updated$config[[1]]$exposure))
})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("run_qmra_supplementary_proces works", {
  scenario <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
  
  regulation_reduction <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Country, RegulationID, Reduction))
  
  result <- run_qmra_supplementary_process(scenario = scenario, 
                                           pathogen = c("Campylobacter jejuni"),
                                           regulationLog = regulation_reduction,
                               regulationConcentration = regulation_concentration)

  expect_equal(
    object = names(result),
    expected = c(
      "logreduction",
      "dalys",
      "regul_matrix_log",
      "regul_matrix_concentration"
    )
  )

  expect_true(inherits(result, "list"))

})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("scenario works", {
  expect_true(inherits(create_scenario, "function"))
})

test_that("scenario returns a dataframe", {
  expect_true(is.data.frame(create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))))
})

test_that("create_scenario stop conditions work", {
  # Test non-character filepath
  expect_error(create_scenario(123), "filename must be a string")

  # Test non-existent filepath
  expect_error(create_scenario("nonexistent_file.xlsx"), "filename must exist")

  # Test invalid Area (not numeric)
  temp_file <- tempfile(fileext = ".xlsx")
  writexl::write_xlsx(data.frame(Area = "invalid", nb_population = 100), temp_file)
  expect_error(create_scenario(temp_file), "Area must be a numeric value")

  # Test Area out of range (too small)
  writexl::write_xlsx(data.frame(Area = 0, nb_population = 100), temp_file)
  expect_error(create_scenario(temp_file), "Area must be strictly greater than 0 ha and smaller than 10,000 ha")

  # Test Area out of range (too large)
  writexl::write_xlsx(data.frame(Area = 10000, nb_population = 100), temp_file)
  expect_error(create_scenario(temp_file), "Area must be strictly greater than 0 ha and smaller than 10,000 ha")
  
   # Test invalid nb_population (not numeric)
  temp_file <- tempfile(fileext = ".xlsx")
  writexl::write_xlsx(data.frame(Area = 0, nb_population = "invalid"), temp_file)
  expect_error(create_scenario(temp_file), "nb_population must be a numeric value")

  # Test nb_population out of range (too small)
  writexl::write_xlsx(data.frame(Area = 100, nb_population = 0), temp_file)
  expect_error(create_scenario(temp_file), "nb_population must be strictly greater than 0 and smaller than 5,000")

  # Test nb_population out of range (too large)
  writexl::write_xlsx(data.frame(Area = 100, nb_population = 5000), temp_file)
  expect_error(create_scenario(temp_file), "nb_population must be strictly greater than 0 and smaller than 5,000")

  # Clean up
  unlink(temp_file)
})

# Test that column names is correct
test_that("scenario column names are correct", {
  expect_equal(
    object = names(create_scenario(
      system.file("input_1culture_2pop.xlsx", package = "ambre")
    )),
    expected = c("CropName", "Area", "PopulationName", "nb_population", "PathName",                                                                                                                   
"STEPtreatmentName", "CollectiveTreatmentName", "InitialProcessName", 
"SupplementaryProcessName", "nb_day_decay", "config", "CropID", 
"CropHeight", "PopulationID", "PathID", "MatrixID", "STEPtreatmentID", 
"CollectiveTreatmentID", "InitialProcessID", "SupplementaryProcessID", 
"InitialTrainName", "InitialTrainID", "SupplementaryTrainName", 
"SupplementaryTrainID")
  )
})

# Test usage
test_that("return the right IDs associated to queries", {
  scenario_tested <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )
  expect_equal(object = scenario_tested$CropID,
               expected = c(6, 1))
  
  expect_equal(object = scenario_tested$PopulationID,
               expected = c(1, 2))
  
  expect_equal(object = scenario_tested$PathID,
               expected = c(1, 4))
  
  expect_equal(object = scenario_tested$STEPtreatmentID,
               expected = list(1, 1))
  
  expect_equal(object = scenario_tested$SupplementaryProcessID,
               expected = list(26, 12))
})

# WARNING - Generated by {fusen} from dev/flat_simulate_treatment.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("update_logreduction_path works", {
  scenario_example <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )

  result <- purrr::pmap(
  list(
    config = scenario_example$config,
    barrierID = scenario_example$SupplementaryProcessID,
    matrixID = scenario_example$MatrixID,
    popID = scenario_example$PopulationID
  ),
  function(config, barrierID, matrixID, popID) {

    if (barrierID %in% c(9,13,14,15,16,17,18,24,26,27,28)) {

      update_logreduction_path(
        config = config,
        matrixID = matrixID,
        barrierID = barrierID,
        popID = popID
      )

    } else {
      config
    }
  }
)
  
  difference_min_value <- scenario_example$config[[1]]$treatment$processes[76, ]$min - result[[1]]$treatment$processes[76, ]$min
  difference_max_value <- scenario_example$config[[1]]$treatment$processes[76, ]$max - result[[1]]$treatment$processes[76, ]$max
  
  expect_equal(object = difference_min_value,
               expected = 0) 
  
  expect_equal(object = difference_max_value,
               expected = 3)

})

# WARNING - Generated by {fusen} from dev/flat_simulate_treatment.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("update_logreduction_decay works", {
  scenario_example <- create_scenario(
    system.file("input_1culture_2pop.xlsx", package = "ambre")
  )

  result <- purrr::pmap(
    list(
      config = scenario_example$config,
      barrierID = scenario_example$SupplementaryProcessID,
      cropHeight = scenario_example$CropHeight,
      popID = scenario_example$PopulationID,
      decay = scenario_example$nb_day_decay
      
    ),
    function(config, barrierID, cropHeight, popID, decay) {
  
      if (barrierID %in% c(12)) {
  
        update_logreduction_decay(
          config = config,
          cropHeight = cropHeight,
          barrierID = barrierID,
          popID = popID,
          nb_day_decay = decay
        )
  
      } else {
        config
      }
    }
  )
  
  difference_min_value <- scenario_example$config[[2]]$treatment$processes[34, ]$min - result[[2]]$treatment$processes[34, ]$min
  difference_max_value <- scenario_example$config[[2]]$treatment$processes[34, ]$max - result[[2]]$treatment$processes[34, ]$max
  
  expect_equal(object = difference_min_value,
               expected = -3) 
  
  expect_equal(object = difference_max_value,
               expected = -4)
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Crop Query
#' 
#' Query to return crop ID from crop Name in crop database
#' @param config list configuration file default value is config_ambre dataset
#' @param cropName character Name of crop for which you want to find the identifier
#' @importFrom dplyr filter pull
#' @return cropID integer
#' 
#' @export
#' @examples
#' query_crop(cropName = "Corn seed")
query_crop <- function(config = config_ambre, cropName) {
  allowed_crop <- c("Corn seed", "Potato", "Tomato", "Onion", "Apple", "Salad")
  if (any(!cropName %in% allowed_crop)){
    stop("Crop name doesn't exist")}

  config$crop |>
    filter(CropName %in% cropName) |>   # filtre sur la valeur fournie
    pull(CropID) 
    
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Print Debug Messages Before and After Running Code
#'
#' This function does the same thing as kwb.utils::catIf(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' @param messageText text to be printed before running the code
#' @param expr expressions to be run. Enclose more than one expression in
#'   curly braces
#' @param newLine integer controlling new lines. 0: no extra new line, 1:
#'   new line after \code{messageText}, 2: new line after "ok.", 3: new line
#'   after both, \code{messageText} and "ok."
#' @param dbg logical. If \code{FALSE}, output is suppressed.
#' @param log_time logical. If \code{TRUE}, the time elapsed during the 
#'   evaluation of \code{expr} is printed.
#' @return This function returns the evaluation of \code{expr}. The result is
#'   returned invisibly so that the result of an assignment as the last 
#'   expression in \code{exprt} does not show up on the console.
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' @examples
#' for (newLine in 0:3) {
#'   catAndRun("work hard", newLine = newLine, {
#'      cat("hard\nworking\n")
#'     })
#'
#'   cat("here.\n\n")
#' } 
catAndRun <- function(
  messageText = "Running code", expr, newLine = 2L, dbg = TRUE, log_time = TRUE
)
{
  catIf(dbg, messageText, "... ")
  
  catNewLineIf(dbg && bitwAnd(newLine, 1))
  
  start_time <- Sys.time()
  
  result <- eval(expr, envir = -1)

  catIf(dbg, "ok. ")
  
  dt <- Sys.time() - start_time
  
  catIf(dbg && log_time, sprintf("(%0.2f %s) ", dt, attr(dt, "units")))
  
  catNewLineIf(dbg && bitwAnd(newLine, 2))
  
  invisible(result)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("catAndRun works correctly", {
  # Test basic functionality
  output <- capture.output(catAndRun("Test message", {cat("Running test")}, dbg = TRUE, log_time = FALSE))
  expect_true(grepl("Test message ... Running testok.", paste(output, collapse = " ")))

  # Test with newLine = 0
  output <- capture.output(catAndRun("Test message", {cat("Running test")}, newLine = 0L, dbg = TRUE, log_time = FALSE))
  expect_true(grepl("Test message ... Running testok.", paste(output, collapse = " ")))

  # Test with newLine = 1
  output <- capture.output(catAndRun("Test message", {cat("Running test")}, newLine = 1L, dbg = TRUE, log_time = FALSE))
  combined <- paste(output, collapse = " ")
  expect_true(grepl("Test message ...", combined))
  expect_true(grepl("Running testok.", combined))

  # Test with dbg = FALSE
  output <- capture.output(catAndRun("Test message", {cat("Running test")}, dbg = FALSE))
  expect_true(grepl(c("Running test"), paste(output, collapse = " ")))

  # Test return value
  result <- catAndRun("Test message", {1 + 1}, dbg = FALSE)
  expect_equal(result, 2)
})

utils::globalVariables(unique(c(
# update_concentration : 
"max.x", "max.y", "min.x", "min.y", "type.x", "type.y", 
# previously declared:
".x", "allocation", "annual_cost", "annual_fee", "annual_membership_fee", "Area", "capex", "capex_barrier", "capex_barrier_20y", "capex_collective_treatment", "capex_supplementaryProcess", "capex_supplementaryProcess_20y", "Class", "collective_opex", "CollectiveTreatmentID", "CollectiveTreatmentName", "color", "Concentration", "config", "config_ambre", "cost_type", "CostType", "Country", "CropHeight", "CropID", "CropName", "dalys_total", "infectionProb_total", "inflow", "initial_dose", "InitialProcessID", "InitialProcessName", "irrigation_need_m3an", "IrrigationNeed", "loan_asa", "logReduction", "MatrixID", "max_reductionC_achieve", "new_membership_fee", "new_water_charge", "opex", "opex_barrier", "opex_supplementaryProcess", "PathDescription", "PathID", "PathName", "PathogenGroup", "PathogenID", "PathogenName", "PopulationID", "PopulationName", "Reduction", "RegulationID", "risk_total", "STEPtreatmentID", "STEPtreatmentName", "SupplementaryBarrierID", "SupplementaryBarrierName", "SupplementaryProcessID", "SupplementaryProcessName", "SupplementaryTreatmentID", "SupplementaryTreatmentName", "target_class", "total_annual_cost", "total_capex", "TreatmentID", "TreatmentName", "value", "volume", "volume_perEvent", "volume_to_treat", "water_charge"
)))

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Helper function: distribution repeater
#' 
#' This function does the same thing as kwb.utils::distribution_repeater(),
#' but it is bundled with *ambre* to avoid a hard dependency on the
#' unmaintained package.
#' 
#' @param number_of_repeatings how often should the random distribution with the
#' same parameters be generated (default: 1)
#' @param  number_of_events number of events (a single positive integer, >= 1)
#' @param func distribution function to be repeated (e.g. runif, rlnorm, rnorm)
#' @param ... further parameters passed to func
#' @importFrom stats setNames
#' @importFrom tidyr gather
#' @importFrom dplyr mutate select
#' @return data.frame with columns repeatID, eventID and values
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @export
#' @examples
#'  distribution_repeater(
#'    number_of_repeatings = 2,
#'    number_of_events = 10,
#'    func = runif,
#'    min = 1,
#'    max = 10
#' )
distribution_repeater <- function(
  number_of_repeatings = 10, number_of_events = 365, func, ...
)
{
  if (
    ! is.numeric(number_of_events) || length(number_of_events) != 1 ||
    is.na(number_of_events) || number_of_events < 1
  ) {
    stop("number_of_events must be a single positive integer (>= 1)")
  }

  seq_repeatings <- seq_len(number_of_repeatings)

  repl <- vapply(
    seq_repeatings,
    function(i) func(n = number_of_events, ...),
    FUN.VALUE = numeric(number_of_events)
  )

  # vapply() collapses the result to a plain vector when number_of_events == 1;
  # coerce back to a matrix so every case flows through the same pipeline and
  # the function always returns the documented long data.frame.
  repl <- matrix(repl, nrow = number_of_events)

  repl |>
    as.data.frame() |>
    cbind(eventID = seq_len(number_of_events)) |>
    stats::setNames(c(seq_repeatings, "eventID")) |>
    tidyr::gather(key = "repeatID", value = "values", -.data$eventID) |>
    dplyr::mutate(repeatID = as.integer(.data$repeatID)) |>
    dplyr::select(.data$repeatID, .data$eventID, .data$values)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.


test_that("distribution_repeater works correctly", {
  # Test with default parameters
  result <- distribution_repeater(func = runif)
  expect_equal(nrow(result), 3650)
  expect_equal(length(unique(result$repeatID)), 10)
  expect_equal(length(unique(result$eventID)), 365)

  # Test with custom number_of_repeatings and number_of_events
  result <- distribution_repeater(
    number_of_repeatings = 2,
    number_of_events = 10,
    func = runif
  )
  expect_equal(nrow(result), 20)
  expect_equal(length(unique(result$repeatID)), 2)
  expect_equal(length(unique(result$eventID)), 10)

  # Test with number_of_events = 1: must still return the long data.frame
  # (number_of_repeatings defaults to 10, so 10 rows / 10 repeatIDs / 1 eventID)
  result <- distribution_repeater(number_of_events = 1, func = runif)
  expect_s3_class(result, "data.frame")
  expect_named(result, c("repeatID", "eventID", "values"))
  expect_equal(nrow(result), 10)
  expect_equal(length(unique(result$repeatID)), 10)
  expect_equal(length(unique(result$eventID)), 1)

  # number_of_events < 1 is meaningless and must fail early and clearly
  expect_error(
    distribution_repeater(number_of_events = 0, func = runif),
    "positive integer"
  )
  expect_error(
    distribution_repeater(number_of_events = -1, func = runif),
    "positive integer"
  )

  # Test with different distribution function (rnorm)
  result <- distribution_repeater(func = rnorm, mean = 0, sd = 1)
  expect_equal(nrow(result), 3650)
  expect_equal(length(unique(result$repeatID)), 10)
  expect_equal(length(unique(result$eventID)), 365)

  # Test with additional parameters for distribution function
  result <- distribution_repeater(func = runif, min = 1, max = 10)
  expect_equal(nrow(result), 3650)
  expect_equal(length(unique(result$repeatID)), 10)
  expect_equal(length(unique(result$eventID)), 365)
  expect_true(all(result$values >= 1 & result$values <= 10, na.rm = TRUE))
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)

## ----setup--------------------------------------------------------------------
library(ambre)
set.seed(2024)

## ----libs---------------------------------------------------------------------
library(dplyr)
library(purrr)

## ----chain, results = "hide"--------------------------------------------------
scenario <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario <- inflow_concentration(scenario, pathogenName = "Campylobacter jejuni")
scenario <- scenario |> mutate(volume = map(config, ~ simulate_exposure(config = .x)))
scenario <- initial_dose_calculation(scenario)
scenario <- update_treatment_scheme(scenario, initial_situation = TRUE)
scenario <- scenario |> mutate(log_reduction = map(config, ~ simulate_treatment(.x)))
scenario <- final_dose_calculation(scenario)
scenario <- infection_probability_calculation(scenario)
scenario <- illness_probability_calculation(scenario)
scenario <- dalys_calculation(scenario)
scenario <- get_risk_total(scenario)

## ----unnest-------------------------------------------------------------------
rt <- bind_rows(scenario$risk_total)
nrow(rt)         # 1000 runs x 2 scenario rows

## ----quantiles----------------------------------------------------------------
quantile(rt$dalys_sum, c(0.05, 0.5, 0.95))
mean(rt$dalys_sum > 1e-6)     # share of Monte-Carlo runs above the WHO target

## ----dalys-plot, results = "hide"---------------------------------------------
library(dplyr)
scenario_example <- create_scenario(filepath = system.file("input_1culture_2pop.xlsx", package = "ambre"))
regulation_reduction <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                          select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                        select(-c(Country, RegulationID, Reduction))

plots <- plot_comparison_qmra_initial_vs_supplementary_processes(
  scenario = scenario_example,
  pathogen = c("Campylobacter jejuni"),
  regulationLog = regulation_reduction,
  regulationConcentration = regulation_concentration
)

## ----dalys-show---------------------------------------------------------------
plots$dalys

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)

## ----setup--------------------------------------------------------------------
library(ambre)
set.seed(2024)

## ----libs---------------------------------------------------------------------
library(dplyr)
library(purrr)

## ----install, eval = FALSE----------------------------------------------------
#  remotes::install_git(
#    "https://forge.inrae.fr/reversaal/reut/ambre-package.git"
#  )

## ----example-files------------------------------------------------------------
list.files(system.file(package = "ambre"), pattern = "\\.xlsx$")

## ----scenario-----------------------------------------------------------------
scenario <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
dim(scenario)

## ----scenario-glimpse---------------------------------------------------------
dplyr::glimpse(
  scenario[, c("CropName", "PopulationName", "PathName",
               "CropID", "PopulationID", "PathID", "config")]
)

## ----run, results = "hide"----------------------------------------------------
library(dplyr)
scenario <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
regulation_reduction <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                          select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                          select(-c(Country, RegulationID, Reduction))
plots <- run_qmra_initial_situation(scenario = scenario, 
                     pathogen = c("Campylobacter jejuni", "Norovirus"),
                     regulationLog = regulation_reduction,
                     regulationConcentration = regulation_concentration)

## ----run-names----------------------------------------------------------------
names(plots)

## ----fig-logreduction---------------------------------------------------------
plots$logreduction

## ----fig-dalys----------------------------------------------------------------
plots$dalys

## ----fig-matrix-log-----------------------------------------------------------
plots$regul_matrix_log

## ----fig-matrix-concentration-------------------------------------------------
plots$regul_matrix_concentration

## ----plot-inflow--------------------------------------------------------------
scenario_with_inflow <- inflow_concentration(scenario = scenario,
 pathogenName = c("Campylobacter jejuni", "Norovirus"))
plot_inflow(scenario_with_inflow)

## ----plot-volume--------------------------------------------------------------
scenario_volume <-
  scenario_with_inflow |>                     
  mutate(
    volume = map(
      .x = config,                                         
      .f = ~ simulate_exposure(config = .x)              
    )
  )

plot_volumes(scenario_volume)

## ----plot-infection-probability-----------------------------------------------
scenario_dose_ini <- initial_dose_calculation(scenario_volume)

scenario_scheme <- update_treatment_scheme(scenario_dose_ini)

scenario_with_logreduc_and_co <- scenario_scheme |> 
  mutate( log_reduction = map(config, simulate_treatment))

scenario_final_dose_and_co_test <- final_dose_calculation(scenario_with_logreduc_and_co)

scenario_inf_proba_and_co <- infection_probability_calculation(scenario_final_dose_and_co_test)

scenario_illness_proba_and_co <- illness_probability_calculation(scenario_inf_proba_and_co)

scenario_dalys_and_co <- dalys_calculation(scenario_illness_proba_and_co)

scenario_risk_total_and_co <-get_risk_total(scenario_dalys_and_co)

plot_infection_probability(scenario_risk_total_and_co)

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' String of Comma Separated Quoted Strings
#' 
#' Create a string of comma separated quoted strings
#' 
#' This function does the same thing as kwb.utils::stringList(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' @param x vector of character
#' @param qchar character to be used for quoting, default: single quote
#'   character
#' @param collapse characters used to separate the strings. Default: ", "
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' 
#' @examples
#'
#' # cat(stringList(c("apple", "banana", "cherry"), qchar = "\""), "\n")
#' # cat(stringList(c("red", "green", "blue"), qchar = "'", collapse = "; "), "\n")
stringList <- function(x, qchar = "'", collapse = ", ") 
{
  collapsed(hsQuoteChr(x, qchar = qchar), collapse = collapse)
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("stringList works correctly", {
  # Test with default parameters
  expect_equal(stringList(c("a", "b", "c")), "'a', 'b', 'c'")

  # Test with custom quote character
  expect_equal(stringList(c("a", "b", "c"), qchar = "\""), "\"a\", \"b\", \"c\"")

  # Test with custom collapse
  expect_equal(stringList(c("a", "b", "c"), collapse = "; "), "'a'; 'b'; 'c'")

  # Test with single element
  expect_equal(stringList(c("a")), "'a'")

  # Test with empty vector
  expect_equal(stringList(character(0)), "''")

  # Test with numeric vector
  expect_equal(stringList(1:3), "1, 2, 3")

  # Test with logical vector
  expect_equal(stringList(c(TRUE, FALSE)), "TRUE, FALSE")
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Restore Object Attributes
#' 
#' Restore given attributes that are not object attributes any more
#' 
#' This function does the same thing as kwb.utils::catIf(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' @param x object
#' @param attribs former attributes of x (as retrieved by attributes(x)) to be
#'   restored
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' 
#' @examples
#' # Example usage of hsRestoreAttributes
#' x <- 1:5
#' attr(x, "my_attr") <- "test attribute"
#' attr(x, "another_attr") <- 123
#'
#' # Remove an attribute
#' attr(x, "my_attr") <- NULL
#'
#' # Restore the original attributes
#' original_attrs <- list(my_attr = "test attribute", another_attr = 123)
#' x_restored <- hsRestoreAttributes(x, original_attrs)
#'
#' # Print results
#' print("Original object with attributes:")
#' print(attributes(x))
#'
#' print("Restored object with attributes:")
#' print(attributes(x_restored))
hsRestoreAttributes <- function(x, attribs)
{
  for (attrib in setdiff(names(attribs), names(attributes(x)))) {
    
    attr(x, attrib) <- attribs[[attrib]]
  }
  
  x
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("hsRestoreAttributes works correctly", {
  # Create a test object with attributes
  x <- 1:5
  attr(x, "attr1") <- "value1"
  attr(x, "attr2") <- 123

  # Store original attributes
  original_attrs <- attributes(x)

  # Remove an attribute
  attr(x, "attr1") <- NULL

  # Restore the removed attribute
  x_restored <- hsRestoreAttributes(x, original_attrs)

  # Test that the attribute was restored
  expect_equal(attr(x_restored, "attr1"), "value1")
  expect_equal(attr(x_restored, "attr2"), 123)

  # Test that the object values remain unchanged
  expect_true(all(x_restored == 1:5))

  # Test with no attributes to restore
  x_no_restore <- hsRestoreAttributes(x_restored, list())
  expect_equal(x_no_restore, x_restored)
  expect_equal(attributes(x_no_restore), attributes(x_restored))
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Query Matrix
#' 
#' Retrieve Matrix ID according to path 
#' @param config list configuration file default value is config_ambre dataset
#' @param pathName character Name of the path for which you want to find the ID
#' @importFrom dplyr filter pull
#' @return matrixID integer
#' 
#' @export
#' @examples
#' query_matrix(pathName = "Consumption of the final product potatoes")
query_matrix <- function(config = config_ambre, pathName){
  matrixID <- config_ambre$path$description |> filter(PathDescription %in% pathName) |> 
                                          pull(MatrixID)
  return(matrixID)
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("query_matrix works", {
  result <- query_matrix(pathName = "Consumption of the final product potatoes")
  expect_equal(object = result, 
               expected = 5) 
})

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Create Exposure Scenario from a data.frame
#'
#' Like [create_scenario()] but takes an in-memory `data.frame` instead of an
#' Excel file, and each of the four treatment roles
#' (STEP / Collective / Initial / Supplementary) may hold a *vector* of process
#' names (a list-column), so a train can contain more than one process per role.
#'
#' Single-value role columns are also accepted (they behave like
#' [create_scenario()]); `NA` / empty roles are dropped from the train.
#'
#' @param user_input A data.frame / tibble with columns `CropName`, `Area`,
#'   `PopulationName`, `nb_population`, `PathName`, `nb_day_decay`, and the four
#'   treatment columns `STEPtreatmentName`, `CollectiveTreatmentName`,
#'   `InitialProcessName`, `SupplementaryProcessName`. Each treatment cell may be
#'   a single name, a character vector of names, `NA`, or empty.
#' @param config list config file, default value `config_ambre`.
#' @importFrom dplyr mutate pick
#' @importFrom purrr map pmap
#' @return A scenario tibble, same structure as [create_scenario()].
#' @export
#' @examples
#' df <- data.frame(
#'   CropName = "Tomato", Area = 35,
#'   PopulationName = "Maintenance staff", nb_population = 1,
#'   PathName = "Consumption of the final product tomato",
#'   nb_day_decay = NA_real_
#' )
#' df$STEPtreatmentName <- list(c("Q.1 - Activated Sludge", "Q.2 - Maturation Pond"))
#' df$CollectiveTreatmentName <- list(NA_character_)
#' df$InitialProcessName <- list("Q.6 - Chlorination")
#' df$SupplementaryProcessName <- list(NA_character_)
#' create_scenario_from_df(df)
create_scenario_from_df <- function(user_input, config = config_ambre) {
  if (!is.data.frame(user_input)) {
    stop("user_input must be a data.frame")
  }
  if (!is.numeric(user_input$Area)) {
    stop("Area must be a numeric value")
  }
  if (!is.numeric(user_input$nb_population)) {
    stop("nb_population must be a numeric value")
  }
  if (any(user_input$Area <= 0) || any(user_input$Area >= 10000)) {
    stop("Area must be strictly greater than 0 ha and smaller than 10,000 ha")
  }
  if (any(user_input$nb_population <= 0) ||
      any(user_input$nb_population >= 5000)) {
    stop("nb_population must be strictly greater than 0 and smaller than 5,000")
  }

  # add config file to each line
  scenario <- user_input |> mutate(config = list(config))

  # Single-value context columns (one crop / population / path per usage).
  scenario <- scenario |> mutate(
    CropID = as.numeric(map(.x = CropName, .f = ~ query_crop(cropName = .x))),
    CropHeight = as.numeric(map(.x = CropName, .f = ~ query_crop_height(cropName = .x))),
    PopulationID = as.numeric(map(.x = PopulationName, .f = ~ query_pop(popName = .x))),
    PathID = as.numeric(map(.x = PathName, .f = ~ query_exp_path(pathName = .x))),
    MatrixID = as.numeric(map(.x = PathName, .f = ~ query_matrix(pathName = .x)))
  )

  # Treatment roles: each cell may hold a vector of names -> a vector of IDs.
  # Resolve name-by-name (dropping NA) so names and IDs stay aligned and ordered
  # (query_barrier() on a vector would re-order to the processes-table order).
  # An empty role stays a scalar NA (like create_scenario()), so downstream code
  # that reads a single per-role ID (e.g. run_qmra_*() reading
  # SupplementaryProcessID) keeps working; a filled role becomes a vector of IDs.
  resolve_ids <- function(names) {
    names <- names[!is.na(names)]
    if (length(names) == 0) {
      return(NA_real_)
    }
    vapply(names, function(nm) query_barrier(barrierName = nm), numeric(1))
  }
  scenario <- scenario |> mutate(
    STEPtreatmentID = map(STEPtreatmentName, resolve_ids),
    CollectiveTreatmentID = map(CollectiveTreatmentName, resolve_ids),
    InitialProcessID = map(InitialProcessName, resolve_ids),
    SupplementaryProcessID = map(SupplementaryProcessName, resolve_ids)
  )

  # Trains: concatenate the role vectors, drop NA, keep order.
  drop_na_vec <- function(...) unname(c(...)[!is.na(c(...))])
  scenario <- scenario |> mutate(
    InitialTrainName = pmap(
      pick(STEPtreatmentName, CollectiveTreatmentName, InitialProcessName),
      drop_na_vec
    ),
    InitialTrainID = pmap(
      pick(STEPtreatmentID, CollectiveTreatmentID, InitialProcessID),
      drop_na_vec
    ),
    SupplementaryTrainName = pmap(
      pick(STEPtreatmentName, CollectiveTreatmentName, InitialProcessName, SupplementaryProcessName),
      drop_na_vec
    ),
    SupplementaryTrainID = pmap(
      pick(STEPtreatmentID, CollectiveTreatmentID, InitialProcessID, SupplementaryProcessID),
      drop_na_vec
    )
  )

  return(scenario)
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("create_scenario_from_df returns a data.frame", {
  df <- data.frame(
    CropName = "Tomato", Area = 35,
    PopulationName = "Maintenance staff", nb_population = 1,
    PathName = "Consumption of the final product tomato",
    nb_day_decay = NA_real_
  )
  df$STEPtreatmentName <- list("Q.1 - Activated Sludge")
  df$CollectiveTreatmentName <- list(NA_character_)
  df$InitialProcessName <- list("Q.6 - Chlorination")
  df$SupplementaryProcessName <- list(NA_character_)

  expect_true(is.data.frame(create_scenario_from_df(df)))
})

test_that("create_scenario_from_df builds a multi-process initial train", {
  df <- data.frame(
    CropName = "Tomato", Area = 35,
    PopulationName = "Maintenance staff", nb_population = 1,
    PathName = "Consumption of the final product tomato",
    nb_day_decay = NA_real_
  )
  df$STEPtreatmentName <- list(c("Q.1 - Activated Sludge", "Q.2 - Maturation Pond"))
  df$CollectiveTreatmentName <- list(NA_character_)
  df$InitialProcessName <- list("Q.6 - Chlorination")
  df$SupplementaryProcessName <- list(NA_character_)

  scenario <- create_scenario_from_df(df)

  # Roles are concatenated (order kept, NAs dropped); names and IDs stay aligned.
  expect_identical(
    scenario$InitialTrainName[[1]],
    c("Q.1 - Activated Sludge", "Q.2 - Maturation Pond", "Q.6 - Chlorination")
  )
  expect_identical(scenario$InitialTrainID[[1]], c(1, 2, 6))
})

test_that("create_scenario_from_df stop conditions work", {
  base <- data.frame(
    CropName = "Tomato", PopulationName = "Maintenance staff",
    PathName = "Consumption of the final product tomato", nb_day_decay = NA_real_
  )

  expect_error(create_scenario_from_df(123), "user_input must be a data.frame")
  expect_error(
    create_scenario_from_df(cbind(base, Area = "x", nb_population = 1)),
    "Area must be a numeric value"
  )
  expect_error(
    create_scenario_from_df(cbind(base, Area = 0, nb_population = 1)),
    "Area must be strictly greater"
  )
})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Inflow concentration custom
#' 
#' To calculate inflow concentration for specified pathogen
#' @param scenario list scenario created with create_scenario
#' @param concentration data.frame min, max and distribution law of pathogen concentration
#' @param monteCarlo integer number of simulation iterations for Monte Carlo
#' @importFrom purrr map
#' @importFrom dplyr mutate
#' 
#' @return scenario_with_concentration
#' 
#' @export
#' @examples
#' sc <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#' sc  <- update_volume_with_desired_value(sc, 
#'                                         volume = data.frame(min = c(0.5,1), 
#'                                                             max = c(1,2), 
#'                                                             type = c("triangle", "triangle")))
#'   
#' sc <- update_frequency_with_desired_value(scenario = sc, frequency = c(200L, 300L))
#'
#' concentration_custom <- data.frame(PathogenName = c("Campylobacter jejuni", "Norovirus"),
#'                                    min = c(1, 10),
#'                                    max = c(2,20),
#'                                    type = c("uniform", "uniform"))
#'   
#' inflow_concentration_custom(scenario = sc,
#'                             concentration = concentration_custom)
inflow_concentration_custom <- function(scenario, concentration, monteCarlo = 1000){
  if (isFALSE("PathogenName" %in% names(concentration))){
    stop("List of concentration must contain PathogenName")
  }
  scenario_updated_patho <- update_pathogen(scenario = scenario, 
                                            pathoName = concentration$PathogenName)
  scenario_updated_monte_carlo <- update_monte_carlo(scenario = scenario_updated_patho,
                                                     monte_carlo = monteCarlo)
  scenario_update_concentration <- update_concentration(scenario = scenario_updated_monte_carlo ,
                                                        concentration = concentration)
  
  concentration = map(.x = scenario_update_concentration$config,
                      .f= ~ simulate_inflow(.x))

  scenario_with_concentration <- scenario_update_concentration |>
     mutate(inflow_concentration = concentration)
  
  return(scenario_with_concentration)
    
}

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Apply barriers' specific log-reduction to a config
#'
#' For one or more supplementary barriers, updates the per-config log-reduction
#' according to each barrier's type: pathway-specific barriers use
#' [update_logreduction_path()], natural die-off uses
#' [update_logreduction_decay()], and product/soil-specific barriers use
#' [update_logreduction_specific_barrier()]. Barriers are applied in turn so
#' their reductions accumulate; any other barrier leaves the config unchanged.
#' Extracted from `run_qmra_initial_situation()` /
#' `run_qmra_supplementary_process()` (was an inline anonymous function).
#'
#' @param config A per-row config (list) from the scenario.
#' @param barrierID One or more supplementary barrier ids (a scalar or a vector);
#'   each is applied in turn. `NA` matches no branch and is skipped.
#' @param cropHeight The crop height.
#' @param matrixID The exposure-matrix id.
#' @param popID The exposed-population id.
#' @param decay The number of decay days (for natural die-off barriers).
#'
#' @return The updated config.
#' @export
retrieve_specific_barrier <- function(config, barrierID, cropHeight, matrixID, popID, decay) {
  # Apply each barrier in turn so their log-reductions accumulate (handles a
  # single id, a vector of ids, or NA -- NA matches no branch and is skipped).
  for (bid in barrierID) {
    if (bid %in% c(24, 26, 27, 28) && matrixID %in% c(3, 5)) {
      config <- update_logreduction_specific_barrier(
        config = config,
        cropHeight = cropHeight,
        barrierID = bid
      )
    } else if (bid %in% c(12)) {
      config <- update_logreduction_decay(
        config = config,
        cropHeight = cropHeight,
        barrierID = bid,
        popID = popID,
        nb_day_decay = decay
      )
    } else if (bid %in% c(9, 13, 14, 15, 16, 17, 18, 24, 26, 27, 28)) {
      config <- update_logreduction_path(
        config = config,
        matrixID = matrixID,
        barrierID = bid,
        popID = popID
      )
    }
  }
  config
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

#' Call cat If Condition Is Met
#' 
#' This function does the same thing as kwb.utils::catIf(),
#' but it is bundled with *ambre* to avoid a hard dependency on the 
#' unmaintained package.
#' 
#' @param condition if TRUE, cat is called, else not
#' @param \dots arguments passed to cat
#' @references
#'   Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
#'   URL: \url{https://github.com/kwb-r/kwb.utils}
#'   (MIT Licence)
#' @noRd
#' 
#' @examples
#'
#' catIf(TRUE, "This will be printed\n")
#' catIf(FALSE, "This will not be printed\n")
catIf <- function(condition, ...)
{
  if (condition) {
    
    cat(...)
  }
}

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("catIf works correctly", {
  # Test that catIf prints when condition is TRUE
  output <- capture.output(catIf(TRUE, "Test message"))
  expect_equal(output, "Test message")

  # Test that catIf doesn't print when condition is FALSE
  output <- capture.output(catIf(FALSE, "Test message"))
  expect_equal(output, character(0))
})

# WARNING - Generated by {fusen} from dev/flat_plots.Rmd: do not edit by hand # nolint: line_length_linter.

#' Plot infection probability
#' 
#' Boxplot of infection probability per year according to population exposed and pathogen simulated
#' @param scenario with risk_total column obtained with get_risk_total
#' @import ggplot2
#' @importFrom tibble tibble
#' @importFrom tidyr unnest
#' @importFrom purrr map imap
#' @return plots
#' 
#' @export
#' @examples
#' \donttest{
#' library(dplyr)
#' library(purrr)
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#'
#' scenario_conc <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni", "Escherichia coli", "Norovirus", "Rotavirus"))
#'
#' scenario_volume <-
#'   scenario_conc |>                     
#'   mutate(
#'     volume = map(
#'       .x = config,                                         
#'       .f = ~ simulate_exposure(config = .x)              
#'     )
#'   )
#'
#' scenario_dose_ini <- initial_dose_calculation(scenario_volume)
#'
#' scenario_scheme <- update_treatment_scheme(scenario_dose_ini)
#'
#' scenario_with_logreduc_and_co <- scenario_scheme |> 
#'   mutate( log_reduction = map(config, simulate_treatment))
#'
#' scenario_final_dose_and_co_test <- final_dose_calculation(scenario_with_logreduc_and_co)
#'
#' scenario_inf_proba_and_co <- infection_probability_calculation(scenario_final_dose_and_co_test)
#'
#' scenario_illness_proba_and_co <- illness_probability_calculation(scenario_inf_proba_and_co)
#'
#' scenario_dalys_and_co <- dalys_calculation(scenario_illness_proba_and_co)
#'
#' scenario_risk_total_and_co <-get_risk_total(scenario_dalys_and_co)
#'
#' plot_infection_probability(scenario_risk_total_and_co)
#' }
plot_infection_probability <- function(scenario){
  plot_inf_proba <- tibble(
    PopulationName = scenario$PopulationName,
    CropName = scenario$CropName,
    PathogenGroup = map(scenario$risk_total, ~.x$PathogenGroup),
    PathogenName = map(scenario$risk_total, ~.x$PathogenName.x),
    infectionProb_total = map(scenario$risk_total, ~.x$infectionProb_sum))

  plot_inf_proba_long <- plot_inf_proba |> unnest(c(infectionProb_total,PathogenGroup, PathogenName))
  
  n_pathogens <- plot_inf_proba_long %>% 
    distinct(PathogenName) %>% 
    nrow()
  
  colors <- c("Tomato" = "#E53935",
              "Apple" ="#2E7D32",
              "Salad" = "#AEEA00",
              "Potato" = "#C49A6C",
              "Corn seed" = "#FDD835",
              "Onion" = "#9C27B0")
  
  
  plots <- plot_inf_proba_long |>
    split(~ PathogenName) |>
    imap(function(df, pathogen){

      ggplot(df, aes(
        x = PopulationName,
        y = infectionProb_total,
        fill = CropName
      )) +
        geom_boxplot() +
        scale_fill_manual(values = colors, name = "Crop name") +
        theme_minimal() +
        labs(x = "Population expopsed",
            y = "infection probability per year",
             title = pathogen)
    })
  
    return(plots)
    
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

#' Query volume
#' 
#' Retrieve min and max volume values according to path defined in the scenario (database ambre_volume)
#' @param pathID integer ID of the path for which we want to retrieve the volume min and max values
#' @importFrom dplyr filter select
#' @return volume
#' 
#' @export
#' @examples
#' query_volume(pathID = 1)
query_volume <- function(pathID){
    volume <- config_ambre$path$volume |> dplyr::filter(PathID %in% pathID) |>
        dplyr::select(min, max)
    return(volume)
}

# WARNING - Generated by {fusen} from dev/flat_scenario.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("query_volume works", {
  expected_result <- c("min", "max")
  expect_equal(object = colnames(query_volume(pathID = 1)),
               expected = expected_result)
})

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

#' Infection probability calculation
#' 
#' Simulate infection probability for each events
#' @param scenario data.frame with columns inflow concentration, final dose and volume
#' @importFrom purrr map
#' @importFrom dplyr mutate
#' @return scenario_updated
#' 
#' @export
#' @examples
#'
#' library(dplyr)
#' library(purrr)
#' scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
#'
#' scenario_conc <- inflow_concentration(scenario = scenario_test,
#'  pathogenName = c("Campylobacter jejuni", "Escherichia coli"))
#'
#' scenario_volume <-
#'   scenario_conc |>                     
#'   mutate(
#'     volume = map(
#'       .x = config,                                         
#'       .f = ~ simulate_exposure(config = .x)              
#'     )
#'   )
#'
#' scenario_dose_ini <- initial_dose_calculation(scenario_volume)
#'
#' scenario_scheme <- update_treatment_scheme(scenario_dose_ini)
#'
#' scenario_with_logreduc_and_co <- scenario_scheme |> 
#'   mutate( log_reduction = map(config, simulate_treatment))
#'
#' scenario_final_dose_and_co_test <- final_dose_calculation(scenario_with_logreduc_and_co)
#'
#' infection_probability_calculation(scenario_final_dose_and_co_test)
#'
infection_probability_calculation <- function(scenario){
  tbl_risk <- get_tbl_risk(scenario)

  infection_proba <- map(tbl_risk, ~get_infection_prob(.x, dose_response = config_ambre$doseresponse, health = config_ambre$health))
  tbl_risk <- map2(tbl_risk, infection_proba, ~ mutate(.x, proba_inf = .y))
  
  scenario_updated <- scenario |> 
    mutate(risk = tbl_risk)
  
  return(scenario_updated)
    
}

# WARNING - Generated by {fusen} from dev/flat_qmra.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("inflow_concentration_custom works", {
  expect_true(inherits(inflow_concentration_custom, "function")) 
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("number_of_repeatings works", {
  expect_equal(object = number_of_repeatings(config_ambre),
               expected = 1000)
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("noSuchElements works correctly", {
  # Test with single missing element
  expect_equal(
    noSuchElements("x", c("a", "b", "c")),
    "No such element:\n- 'x'\nAvailable elements (sorted):\n- 'a'\n- 'b'\n- 'c'\n"
  )

  # Test with multiple missing elements
  expect_equal(
    noSuchElements(c("x", "y"), c("a", "b", "c"), suffix = "s"),
    "No such elements (sorted)s:\n- 'x'\n- 'y'\nAvailable elements (sorted):\n- 'a'\n- 'b'\n- 'c'\n"
  )

  # Test with sorted = FALSE
  expect_equal(
    noSuchElements("b", c("a", "c"), type = "item", sorted = FALSE),
    "No such item:\n- 'b'\nAvailable items:\n- 'a'\n- 'c'\n"
  )

  # Test with numeric vectors
  expect_equal(
    noSuchElements(4, 1:3),
    "No such element:\n- 4\nAvailable elements (sorted):\n- 1\n- 2\n- 3\n"
  )

  # Test with logical vectors
  expect_equal(
    noSuchElements(TRUE, c(FALSE, TRUE)),
    "No such element:\n- TRUE\nAvailable elements (sorted):\n- FALSE\n- TRUE\n"
  )
})

# WARNING - Generated by {fusen} from dev/flat_kwb_simulate_inflow.Rmd: do not edit by hand # nolint: line_length_linter.

test_that("default_min works correctly", {
  # Test with org_min > 0 and org_max > 1
  expect_equal(default_min(2, 100, 0.01, f = log10), log10(2))

  # Test with org_min == 0 and org_max < 1
  expect_equal(default_min(0, 0.5, 0.01, f = log10), log10(0.01 * 0.5))

  # Test with org_min == 0 and org_max > 1
  expect_equal(default_min(0, 100, 0.01, f = log10), log10(0.01))

  # Test with org_min > 0 and org_max < 1
  expect_equal(default_min(0.1, 0.5, 0.01, f = log10), log10(0.01))

  # Test with no transformation function
  expect_equal(default_min(2, 100, 0.01), 2)

  # Test with log transformation
  expect_equal(default_min(2, 100, 0.01, f = log), log(2))

  # Test with org_min == 0 and org_max == 0 (degenerate input never reached in the
  # pipeline; matches kwb.utils: org_max < 1 branch -> f(0.01 * 0) = f(0) = -Inf)
  expect_equal(default_min(0, 0, 0.01, f = log10), log10(0))
})

---
title: "What-if: calibrating exposure"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{What-if: calibrating exposure}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)
```

```{r setup}
library(ambre)
set.seed(2024)
```

```{r libs}
library(dplyr)
library(purrr)
```

Two numbers decide how much of the water each person actually meets:

- the **volume** swallowed or inhaled per exposure event (litres per event), and
- the **frequency** of those events (how many times it happen per year).

Together with the pathogen concentration, these drive the *dose*, and the dose
drives the risk. This vignette is about the exposure side of the model: where
those two numbers come from by default, how to read them, and how to **override**
them to ask "what if?" — including the two most common management measures,
personal protective equipment and reduced-contact irrigation.

If you are new to the pipeline, read `vignette("a-get-started", package = "ambre")`
first. The barrier-side levers (treatment plants, on-field practices, their
log-reductions) are a different story, told in
`vignette("b-initial-vs-new-scenario", package = "ambre")`.

## How exposure is set by default

You never set volume, frequency or concentration by hand in a normal run. When you call
`inflow_concentration()` (which `run_qmra_intial_situation()` and 
`run_qmra_supplementary_process()` call for you), it runs three updates in 
order:

```
inflow_concentration()
  ├─ update_pathogen()    # pathogen concentration
  ├─ update_frequency()   # number of events / year, per path
  └─ update_volume()      # litres per event, per path
```

```{r default-exposure, results = "hide"}
scenario <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario_conc <- inflow_concentration(
  scenario = scenario,
  pathogenName = "Campylobacter jejuni"
)
```

`update_pathogen()` look up **inflow** database. 
`update_frequency()` and `update_volume()` look up the **exposure path** of each
scenario row and copy the path-specific numbers out of the shipped database
`config_ambre$path` into that row's embedded `config$exposure` table. So the 
model does **not** use one generic volume for every path — each crop × population
× path combination gets its own exposure profile.

You can see the result. The `config$exposure` table has three rows —
`number_of_repeatings` (the Monte-Carlo count), `number_of_exposures` (events per
year) and `volume_perEvent`:

```{r default-exposure-show}
# Row 1 of this two-row scenario: irrigation-staff droplet ingestion
scenario_conc$config[[1]]$exposure
```

Two quirks are worth knowing up front, and we
return to them at the end:

- `update_frequency()` writes only the path's **`max`** frequency, and it writes
  it into the `number_of_exposures` slot — the annual event count *is* the
  frequency.
- both updates use **positional** writes (`min[[3]]`, `max[[3]]`, `value[[2]]`)
  that assume the exposure table keeps its current row order.

## Inspect the base numbers

Before changing anything, look at what the model is assuming. 

Start from the PathogenName, you can see the default value of concentration for the simulated pathogen : 

```{r default-concentration-show}
# Row 1 of this two-row scenario: Campylobacter jejuni concentration
dplyr::filter(scenario_conc$config[[1]]$inflow, PathogenName == "Campylobacter jejuni")
```

Then start from a path
description and get its `PathID` with `query_exp_path()`:

```{r inspect-pathid}
pid <- query_exp_path(
  pathName = "Ingestion of water droplets during maintenance of the irrigation system"
)
pid
```

Then read the per-path volume and frequency straight from the database. Both
helpers accept a vector of IDs, so you can inspect a whole scenario at once:

```{r inspect-values}
scenario$PathID          # the two paths in this input file

query_volume(scenario$PathID)     # litres per event: min / max
query_frequency(scenario$PathID)  # events per year: min / max
```

These are exactly the numbers `update_volume()` and `update_frequency()` inject.
The volumes here are tiny — `0.001` litre is one millilitre of incidentally
swallowed droplets — the frequency `40`–`60` becomes `60` because only the
maximum is used and the concentration is between 100 and 500 log/L.

## Override for a what-if

To explore sensitivity you replace those numbers with your own. Two helpers do
this, and both expect **one value per scenario row**.

`update_volume_with_desired_value()` takes a `volume` argument that is a
data frame (or list) with a `min`, a `max` and a `type` column. Use `min == max` for a
fixed volume, or `min < max` for a distribution spread according to the `type` specified:

```{r override-volume}
scenario_ppe <- update_volume_with_desired_value(
  scenario = scenario,
  volume = data.frame(
    min = c(0.0005, 0.0005),   # 0.5 mL per event, one entry per row
    max = c(0.0005, 0.0005),
    type = c("triangle", "triangle")
  )
)

scenario_ppe$config[[1]]$exposure
```

`update_frequency_with_desired_value()` takes a plain numeric **vector** of 
**integer**, one number of events per year per row:

```{r override-frequency}
scenario_night <- update_frequency_with_desired_value(
  scenario = scenario,
  frequency = c(30L, 30L)   # cap both paths at 30 events / year
)

scenario_night$config[[1]]$exposure
```

The `volume_perEvent` row now reads `0.0005`, and `number_of_exposures` reads
`30`: your what-if values have replaced the ones the database would have injected.


`update_concentration()` takes a **data.frame** with **PathogenName**, **min**, **max** and **type** of ditribution law values.

```{r override-concentration}
concentration_custom <- data.frame(PathogenName = c("Campylobacter jejuni"),
                                   min = c(1),
                                   max = c(2),
                                   type = c("uniform"))

scenario_pathogen <- update_pathogen(scenario = scenario, pathoName = concentration_custom$PathogenName)
scenario_low_pathogen <- update_concentration(scenario = scenario_pathogen ,
                     concentration = concentration_custom)

dplyr::filter(scenario_low_pathogen$config[[1]]$inflow, PathogenName == "Campylobacter jejuni")

```

## Modelling management measures

Read those two overrides as **interventions**. This is the exposure-side
complement to the barrier log-reductions in
`vignette("b-treatment-vs-multibarrier", package = "ambre")`: instead of removing
pathogens from the water, you change how much of the water each person meets.

- **Personal protective equipment (gloves, masks, hygiene rules)** lowers the
  **volume** ingested or inhaled per event. Model it by setting
  `volume_perEvent` below the default — a gloved worker who swallows a fraction
  of a millilitre instead of several.
- **Night-time irrigation, signage, fencing, keeping workers off wet fields**
  lowers the **frequency** of contact. Model it by setting `number_of_exposures`
  to a smaller annual count. Because the annual risk aggregates as
  $1 - \prod(1 - p)$ over events, fewer events mean lower annual risk even when
  each event is unchanged.

Both are assumptions *you* choose and should defend — they are not credited from
a barrier database. (The `barriere_path` / `barriere_specific` tables in
`config_ambre` that *would* encode route-specific exposure reductions are shipped
data, not yet wired into the engine.)

## A mini sensitivity study

Let us quantify how much the volume assumption matters. One catch first:
`run_qmra_initial_situation()` and `run_qmra_supplementary_process()`
re-derives the per-path volumes from the database on every
call (via `inflow_concentration()`), so it would overwrite any override you set.
To inject a what-if value you should use `run_qmra_custom` function:

Run it twice on the same file and pathogen — once with a pessimistic bare-hand
volume, once with a PPE volume ten times smaller:

```{r qmra-custom, results = "hide"}
sc <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

regulation_reduction <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Country, RegulationID, Reduction))

concentration_custom <- data.frame(PathogenName = c("Rotavirus"),
                                   min = c(1000),
                                   max = c(2000),
                                   type = c("uniform"))

set.seed(2024)
bare <- run_qmra_custom(scenario = sc,
                  concentration = concentration_custom,
                  volume = data.frame(min=c(0.005,0.005), max = c(0.01, 0.01), type = c("triangle", "triangle")),
                  frequency = c(48L, 60L),
                  regulationLog = regulation_reduction,
                  regulationConcentration = regulation_concentration,
                  initialSituation = TRUE) # min 5 mL, max 10 mL

set.seed(2024)
ppe  <- run_qmra_custom(scenario = sc,
                  concentratio = concentration_custom,
                  volume = data.frame(min=c(0.0005,0.0005), max = c(0.001, 0.001), type = c("triangle", "triangle")),
                  frequency = c(48L, 60L),
                  regulationLog = regulation_reduction,
                  regulationConcentration = regulation_concentration,
                  initialSituation = TRUE) # min 0.5 mL min 0.1 mL

```

As the the other functions `run_qmra_*` this custom function return 2 plots and to formattable. See `d-interpreting-risk`for more detail on the output.

```{r qmra-custom-comparison, results = "hide"}
library(ggplot2)
cowplot::plot_grid(
      bare$dalys$Rotavirus +
        labs(subtitle = "min 5 mL, max 10 mL") +
        theme(plot.subtitle = element_text(hjust = 0.5)),
      ppe$dalys$Rotavirus +
        labs(subtitle = "min 0.5 mL min 0.1 mL") +
        theme(plot.subtitle = element_text(hjust = 0.5)),
      ncol = 2,
      align = "h"
    )
```


The risk tracks the volume almost proportionally: a tenfold cut in swallowed
volume gives roughly a tenfold cut in DALYs. More tellingly, the 95th percentile
crosses the line — the bare-hand upper tail sits **above** 1e-6 while the PPE tail
drops **below** it. Deciding whether a scenario meets the target can come down to
this single exposure assumption. For how to read these ranges against the
regulatory line, see
`vignette("d-interpreting-risk", package = "ambre")`; for why every number is a
distribution rather than a point, see
`vignette("g-monte-carlo-engine", package = "ambre")`.

## Technical notes and caveats

- **Positional writes are fragile.** `update_volume()` writes `config$exposure$min[[3]]`
  and `max[[3]]`; `update_frequency()` writes `config$exposure$value[[2]]`; the
  `_with_desired_value()` helpers do the same. The indices assume `volume_perEvent`
  is the third row and `number_of_exposures` the second. If the exposure table's
  row order ever changed, these would silently target the wrong parameter — so
  always re-inspect `config$exposure` after an override, as we did above.
- **Frequency uses the maximum only.** `update_frequency()` copies the path's
  `max` frequency (ignoring `min`) into `number_of_exposures`. The default run is
  therefore conservative on contact frequency. Your own
  `update_frequency_with_desired_value()` value is used exactly as given.
- **Volume units.** `volume_perEvent` multiplies the inflow concentration
  (organisms per litre) to yield organisms per event, so it is a volume in
  **litres**. The per-path values (roughly `1e-4` to `1.5e-2` L) are fractions of
  a millilitre up to a few millilitres of incidentally ingested water — or, for
  consumption paths, an equivalent mass of product treated as a volume. The CSVs
  carry no explicit unit column, so keep your overrides on the same litre basis.
- **Overrides and the one-call entry points.** Because `run_qmra_initial_situation()` and
  `run_qmra_supplementary_process()` rebuild the exposure profile from the database each time,
  the `_with_desired_value()` helpers only take effect in a hand-assembled chain
  like the one above. There is no exposure-override argument on the one-call
  functions.

---
title: "Initial situation VS New scenario"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Initial situation VS New scenario}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)
```

```{r setup}
library(ambre)
set.seed(2024)
```

There are many ways to reduce the health risk associated with water reuse. A
scheme can be improved by adding a treatment process, introducing an additional
field barrier, changing irrigation practices, or modifying exposure
conditions.

`ambre` allows you to compare **an initial situation** with **a modified
scenario** on the same quantitative microbial risk assessment (QMRA) scale.
Both scenarios are simulated with the same Monte Carlo engine, making it easy
to evaluate the effect of a proposed change on pathogen concentrations,
infection risk and DALYs.

New to the pipeline? Read `vignette("a-get-started", package = "ambre")` first.

## Treatments and field barriers live in one table

`ambre` does not consider "treatment" and "practices" as different kinds of thing.
Both are just **log-reductions** -- a number of `log10` units of pathogen removed
-- stored side by side in one table, `config_ambre$treatment$processes`. Its
entries are coded by barrier types : **Q.** for water *quality* (treatment processes),
**E.** for on-field *equipment*, **P.** for cultivation and irrigation
*practices*.

```{r families}
tn <- sort(unique(config_ambre$treatment$processes$TreatmentName))
head(tn, 15)
```

Each process is characterised by a distribution of pathogen log-reductions and
identified by its `TreatmentID`. Depending on the scenario, one or more
processes can be added to the existing treatment train to represent an
improvement of the reuse scheme.

## How the Excel prepares two trains

The Excel input file describes the initial reuse scheme `InitialProcessName`. Additional processes
can then be introduced to build a modified scenario while keeping every other
parameter identical `SupplementaryProcessName`.

This approach makes it possible to quantify the benefit of a proposed
intervention under exactly the same exposure assumptions.

Your input file carries four columns describing the barrier chain:

```{r input}
readxl::read_excel(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)[, c("STEPtreatmentName", "CollectiveTreatmentName",
      "InitialProcessName", "SupplementaryProcessName")]
```

From these, `create_scenario()` assembles **two candidate trains**:

- the **initial train** -- wastewater treatment plant + collective treatment + *initial process*;
- the **supplementary train** -- wastewater treatment plant + collective treatment + initial process + * supplementary process.


## The switch is a single line

`run_qmra_initial_situation()` and `run_qmra_supplementary_processs()` are the *same* pipeline. The
only difference is one argument passed deep inside them,
`update_treatment_scheme(initial_situation = ...)`, which decides which train
is injected into the config before the log-reductions are simulated:

- `run_qmra_treatment()` uses `initial_situation = TRUE` -> the initial
  train;
- `run_qmra_barrier()` uses `initial_situation = FALSE` -> the supplementary train.

Everything else -- inflow, exposure volume, dose, dose-response, DALYs -- is
computed identically. In particular, **only the concentration log-reduction
differs** between the two runs; the exposure side is unchanged.

## Run and overlay the two strategies

`plot_comparison_qmra_initial_vs_supplementary_processes()` runs both pipelines on the same 
input scenario file and pathogen and returns three side-by-side comparisons 
-- log-reduction, and DALYs -- each a `cowplot` panel 
with the initial result on the left and the supplementary result on the right.

```{r compare, results = "hide"}
library(dplyr)
scenario_example <- create_scenario(filepath = system.file("input_1culture_2pop.xlsx", package = "ambre"))
regulation_reduction <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                          select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                        select(-c(Country, RegulationID, Reduction))

comparison <- plot_comparison_qmra_initial_vs_supplementary_processes(
  scenario = scenario_example,
  pathogen = c("Campylobacter jejuni"),
  regulationLog = regulation_reduction,
  regulationConcentration = regulation_concentration
)
```

```{r compare-names}
names(comparison)
```

**Log-reduction** -- how much each train removes. This is where the two
strategies visibly diverge, because it is the only step that differs.

```{r compare-logreduction, fig.height = 4}
comparison$log_reduction
```


**DALYs** -- the health burden per person per year, both panels sharing the red
**WHO 1e-6** reference line. Reading the two boxplots against that line tells you
whether either strategy -- or which one -- brings the scheme under the tolerable
target.

```{r compare-dalys, fig.height = 4}
comparison$dalys
```

For how to read these ranges and conclude, see
`vignette("d-interpreting-risk", package = "ambre")`.

## Scope and current limits

The multi-barrier idea is powerful, but be honest about what the engine credits
*today*:

- **Barrier credits add in log space.** Within a train, the log-reductions of
  the successive barriers are summed. Combining a modest treatment with a good
  field practice can reach the target without ever-heavier treatment -- exactly
  the argument behind the WHO multi-barrier guidance and **EU Regulation
  2020/741** (which lets a scheme meet a reclaimed-water class through additional
  barriers rather than more treatment).
- **Some data relating to barriers vary depending on the route of exposure, the 
crop and agricultural practices** `config_ambre$treatment`  contains the tables 
`barriere_path`, `barriere_specific` and `barriere_decay`. These are used to 
update the source database `config_ambre$processes` according to the simulated 
scenario; with function `retrieve_specific_barrier`

To see the underlying database for yourself, read
`vignette("h-config-ambre", package = "ambre")`.

---
title: "The config_ambre database"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{The config_ambre database}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)
```

```{r setup}
library(ambre)
set.seed(2024)
```

```{r attach-dplyr}
library(dplyr)
```

Every risk figure `ambre` produces is ultimately read out of one bundled object:
`config_ambre`. It is the *knowledge base* of the package -- pathogen
concentrations, dose-response models, log-reduction credits for treatment and
field barriers, exposure paths, crops and costs, all in one nested list. When
you run `create_scenario()`, a full copy of `config_ambre` is embedded in each
row of the scenario and mutated in place as the pipeline runs (see
`vignette("a-get-started", package = "ambre")`).

This vignette is the practical reference for that object: what is inside it,
how the tables join, and how you would adapt it to your own case.

## Overview: 8 slots, 16 CSVs

`config_ambre` has **eight** top-level slots. Three of them (`treatment`,
`path`, `economic`) are themselves lists of sub-tables, so the object holds
**16 tables** in total. Each table is assembled from one CSV in `data-raw/` by
the build script `data-raw/config_ambre.R`.

```{r overview-str}
str(config_ambre, max.level = 1)
```

The sub-tables live under the three list slots:

```{r overview-subtables}
lapply(config_ambre[c("treatment", "path", "economic")], names)
```

Here is the full file-to-slot mapping, read straight from
`data-raw/config_ambre.R`. This is the map you need when you want to edit the
source data:

```{r mapping-table}
mapping <- tibble::tribble(
  ~`CSV in data-raw/`,                  ~`Slot in config_ambre`,
  "exposure.csv",                       "exposure",
  "ambre_pathogene.csv",                "inflow",
  "ambre_barriere_general.csv",         "treatment$processes",
  "treatment_schemes.csv",              "treatment$schemes",
  "ambre_barriere_voie.csv",            "treatment$barriere_path",
  "ambre_barriere_specifique.csv",      "treatment$barriere_specific",
  "ambre_barriere_deperissement.csv",   "treatment$barriere_decay",
  "ambre_doseresponse.csv",             "doseresponse",
  "ambre_voie_description.csv",         "path$description",
  "ambre_voie_frequence.csv",           "path$frequency",
  "ambre_voie_volume.csv",              "path$volume",
  "ambre_culture_hauteur.csv",          "crop",
  "ambre_sante.csv",                    "health",
  "ambre_culture_eau.csv",              "economic$water_need",
  "ambre_population.csv",               "economic$population",
  "ambre_barriere_cout.csv",            "economic$cost"
)
knitr::kable(mapping)
```

In one sentence per slot: `exposure` holds the Monte-Carlo controls (how many
runs, how many events, the per-event volume); `inflow` is the raw pathogen
concentration in the treated wastewater; `treatment` carries the log-reduction
credits; `doseresponse` and `health` turn a dose into an infection, an illness
and a DALY; `path` describes the 19 exposure routes; `crop` and `economic`
support the agronomic and costing calculations.

## The entity-relation keys

The tables are relational: they share a small set of ID columns and are joined
on them during the pipeline. Knowing the keys is what lets you cross-reference
(and safely extend) the database.

| Key | Meaning | Appears in |
|-----|---------|------------|
| `PathogenID` / `PathogenGroup` | one of 38 pathogens (Bacteria / Viruses / Protozoa) | `inflow`, `doseresponse`, `health`, `treatment$processes` |
| `TreatmentID` | one barrier (treatment step or field practice) | `treatment$*`, `economic$cost` |
| `PathID` | one of 19 exposure routes | `path$description`, `path$frequency`, `path$volume` |
| `PopulationID` | one of 6 exposed populations | `path$description`, `economic$population`, `barriere_*` |
| `MatrixID` | the environmental compartment (Water / Air / Plant / Soil / Product) | `path$description`, `barriere_path` |
| `CropID` / `CropHeight` | one of 6 crops and its height above the irrigation system | `crop`, `economic$water_need` |

`path$description` is the hub that ties a route to *who* is exposed and to
*which* matrix:

```{r er-path}
config_ambre$path$description |>
  select(PathID, PopulationName, MatrixName, PathDescription) |>
  head(4)
```

And `PathogenID` is what links a pathogen's concentration to its health burden:

```{r er-pathogen}
config_ambre$health |>
  select(PathogenID, PathogenName, dalys_per_case) |>
  inner_join(
    config_ambre$inflow |> select(PathogenID, PathogenGroup),
    by = "PathogenID"
  ) |>
  head(4)
```

## The barrier families

The multi-barrier philosophy of `ambre` rests on the PhD thesis of Alice-Rose
Thomas (INSA Lyon, 2024), carried out in the Multiware research context, which
catalogues up to **49 candidate log-reduction barriers** covering both the
treatment plant and the field. `config_ambre$treatment$processes` ships a
curated subset of that catalogue. Every barrier has a `TreatmentName` prefixed
by its type:

* **`Q.`** -- *Quality*: the wastewater-treatment process (Activated Sludge,
  Maturation Pond, UV Reactor, Chlorination, ...).
* **`E.`** -- *Equipment*: on-field hardware (drip irrigation, signage, fences,
  personal protective equipment, ...).
* **`P.`** -- *Practices*: cultivation and irrigation practices (non-edible
  crop, night-time irrigation, cooking, peeling, washing, ...).

The shipped `TreatmentGroup` labels and the number of distinct barriers in each
family are:

```{r barrier-groups}
config_ambre$treatment$processes |>
  distinct(TreatmentID, TreatmentName, TreatmentGroup) |>
  count(TreatmentGroup, name = "n_barriers")
```

```{r barrier-prefix}
config_ambre$treatment$processes |>
  distinct(TreatmentID, TreatmentName) |>
  mutate(family = sub("[.].*", "", TreatmentName)) |>
  count(family, name = "n_barriers")
```

So the bundled database currently encodes 28 distinct barriers across three
families -- a subset of the PhD's 49, and an intentionally extensible one.
Credits from several barriers combine by simple **log-additivity** when a
scenario stacks a treatment scheme and a field barrier
(`vignette("b-treatment-vs-multibarrier", package = "ambre")`).

## Anatomy of a distribution

Eight of the sixteen tables (`exposure`, `inflow`, `treatment$processes`, the
three `barriere_*` tables, `path$frequency` and `path$volume`) share the same
nine-column **distribution block**:

```
type | value | min | max | mode | mean | sd | meanlog | sdlog
```

This is what makes `ambre` a Monte-Carlo engine: instead of a single number, a
barrier or a concentration is a *distribution* that gets resampled on every run.
The `type` column decides which of the other columns are actually read:

```{r dist-legend}
knitr::kable(tibble::tribble(
  ~type,           ~`columns read`,   ~draws,
  "value",         "value",           "a constant, repeated",
  "uniform",       "min, max",        "runif(min, max)",
  "triangle",      "min, max, mode",  "EnvStats::rtri(min, max, mode)",
  "norm",          "mean, sd",        "rnorm(mean, sd)",
  "lognorm",       "meanlog, sdlog",  "rlnorm(meanlog, sdlog)",
  "log10_uniform", "min, max",        "10^runif(min, max)",
  "log10_norm",    "mean, sd",        "10^rnorm(mean, sd)"
))
```

Here is a real barrier row. `Q.1 - Activated Sludge` removes a `uniform`
0.5-to-1 log of protozoa; every other distribution column is `NA` because a
uniform draw only needs `min` and `max`:

```{r dist-row-barrier}
config_ambre$treatment$processes |>
  filter(TreatmentName == "Q.1 - Activated Sludge") |>
  select(TreatmentName, PathogenGroup, type, value, min, max, mode)
```

And the matching inflow row: the raw *Campylobacter jejuni* concentration is a
`uniform` draw between 100 and 5000 organisms per litre:

```{r dist-row-inflow}
config_ambre$inflow |>
  filter(PathogenName == "Campylobacter jejuni") |>
  select(PathogenName, PathogenGroup, type, value, min, max)
```

The `exposure` slot uses the same block for the sampling controls themselves --
1000 runs, 365 events, and a triangular per-event volume:

```{r dist-exposure}
config_ambre$exposure
```

How these rows are turned into 1000 x 365 random values is the subject of
`vignette("g-monte-carlo-engine", package = "ambre")`, via
`create_random_distribution()` and `generate_random_values()`.

## Customizing the database for your case

Because the object is rebuilt from plain CSVs, adapting it is a three-step
loop: **edit the right CSV, rebuild, verify**.

1. **Edit** the CSV that maps to the slot you want to change (see the mapping
   table above). Add a *coherent* row -- a fresh `TreatmentID`, a `type` from
   the legend above, and the columns that `type` requires.
2. **Rebuild** by re-running `data-raw/config_ambre.R`, which re-reads the 16
   CSVs and writes `data/config_ambre.rda`.
3. **Verify** the new entry is reachable with the `query_*` helpers.

The chunk below writes to `data-raw/` and rebuilds the dataset, so it is shown
for reference only (`eval = FALSE`):

```{r customize, eval = FALSE}
# 1. add a new quality barrier to the treatment CSV
csv_path <- file.path("data-raw", "ambre_barriere_general.csv")
new_barrier <- data.frame(
  TreatmentID = 99, TreatmentName = "Q.8 - Ozonation",
  TreatmentGroup = "Quality", PathogenGroup = "Bacteria",
  type = "uniform", value = NA, min = 2, max = 4,
  mode = NA, mean = NA, sd = NA, meanlog = NA, sdlog = NA
)
readr::write_csv(
  rbind(readr::read_csv(csv_path), new_barrier),
  csv_path
)

# 2. rebuild data/config_ambre.rda from the 16 CSVs
source(file.path("data-raw", "config_ambre.R"))

# 3. verify the new barrier resolves to its ID
query_barrier(barrierName = "Q.8 - Ozonation")
```

The verification helpers each translate a *name* into the *ID* the engine works
with (and error early if the name is unknown), so they are the quickest way to
confirm an edit landed. They run against the live `config_ambre`:

```{r query-helpers}
query_pathogen(pathogenName = "Campylobacter jejuni")
query_barrier(barrierName = "Q.3 - UV Reactor")
query_crop(cropName = "Tomato")
```

`query_volume()` and `query_frequency()` take a `PathID` instead and return the
per-path `min`/`max` block that gets injected into a scenario's exposure (this
is what `update_volume()` / `update_frequency()` do automatically inside the
pipeline):

```{r query-path}
query_volume(pathID = 1)
query_frequency(pathID = 1)
```

## Known pitfalls

A few sharp edges to keep in mind before you rely on or extend the database.

**The tables are `spec_tbl_df`.** Because they are read with `readr`, each slot
carries a `readr` column specification and an extra class:

```{r pitfall-class}
class(config_ambre$crop)
```

This is harmless -- `dplyr` verbs work as usual -- but if you compare tables
with `identical()` or write your own I/O you may trip over the `spec`
attribute. Filtering or mutating usually drops it.

**`economic$cost` is the least polished table.** `mean` / `meanlog` are 
currently all `NA`, so a normally- or lognormally-distributed cost you add expecting
`mean`/`meanlog` will not be picked up, see
`vignette("g-monte-carlo-engine", package = "ambre")`.

**`CropHeight` is a signed height in centimetres** above the irrigation
outlet: `-1` means underground (Onion), `0` means at ground level
(Potato, Salad), and positive values mean the edible part sits above the spray
(Tomato 25, Corn seed / Apple 50). It is the intended key for height-dependent
barrier credits.

```{r pitfall-crop}
config_ambre$crop |>
  select(CropName, CropHeight)
```


## See also

* `vignette("a-get-started", package = "ambre")` -- how `config_ambre` is embedded
  in a scenario and mutated through the pipeline.
* `vignette("g-monte-carlo-engine", package = "ambre")` -- how the distribution
  block is sampled into 1000 x 365 values.
* `vignette("c-input-file", package = "ambre")` -- the Excel catalogues (crops,
  populations, paths, barriers) whose allowed values come from these tables.
* `vignette("e-economic-analysis", package = "ambre")` -- how `economic$cost`,
  `water_need` and `population` drive the costing.

---
title: "Get started with ambre"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get started with ambre}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)
```

```{r setup}
library(ambre)
set.seed(2024)
```

```{r libs}
library(dplyr)
library(purrr)
```

## What ambre does, and for whom

If you plan to irrigate crops with **treated wastewater** (water reuse,
or *REUT*), you need to know whether the water is safe. `ambre` answers
that question with **QMRA** -- Quantitative Microbial Risk Assessment --
estimating the health risk of a programme for the reuse of treated wastewater 
for exposed populations: field workers, local residents and consumers of the 
products.

The benchmark is the World Health Organization target of **no more than
1e-6 DALY per person per year** (one micro disability-adjusted life-year).
`ambre` follows the **multi-barrier** idea: pathogens are cut down not
only by the wastewater-treatment plant but also by on-field practices
(drip instead of sprinklers, buffer distances, washing, peeling...), all
modelled from one shared log-reduction database. This article is the
guided tour; the other vignettes go deeper on each piece.

## Installation and the bundled example data

`ambre` lives on the INRAE forge. Install it once with `remotes`:

```{r install, eval = FALSE}
remotes::install_git(
  "https://forge.inrae.fr/reversaal/reut/ambre-package.git"
)
```

You do not need your own data to follow along. The package ships four
example input files -- ready-made Excel scenarios -- that every function
in this tour reads through `system.file()`:

```{r example-files}
list.files(system.file(package = "ambre"), pattern = "\\.xlsx$")
```

They grow in size from a single crop and population
(`input_1culture_1pop.xlsx`) to a full learning case of 35 exposed populations
(`input_cas_apprentissage_complet.xlsx`). We will use the two-row file
`input_1culture_2pop.xlsx` throughout.

## The `scenario` object

Everything in `ambre` revolves around one table, the **scenario**. You
build it from an Excel file with `create_scenario()`:

```{r scenario}
scenario <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
dim(scenario)
```

Each **row is one combination of crop x exposed population x exposure
path**. The example has two rows, so two situations to assess. Let's look
at the columns that matter most:

```{r scenario-glimpse}
dplyr::glimpse(
  scenario[, c("CropName", "PopulationName", "PathName",
               "CropID", "PopulationID", "PathID", "config")]
)
```

The human-readable names you typed in Excel (`CropName`,
`PopulationName`, `PathName`) are each resolved to an integer key --
`CropID`, `PopulationID`, `PathID` -- that `ambre` uses to look values up
in its internal database.

The last column, `config`, is the interesting one: it is a **list-column**
holding, on every row, a full private copy of the bundled `config_ambre`
database. As the QMRA pipeline runs, each step writes its results back
into that per-row config, so a scenario carries both its inputs and its
growing outputs in a single object. You rarely touch `config` by hand --
the run functions below do it for you.

## Your first run

The fastest way to a result is `run_qmra_treatment()`. Give it a scenario
file and one or more pathogens; it runs the whole workflow -- simulate inflow
concentration and exposure volume to calculate the initial dose, simulate 
log-reduction from the treatment scheme to calculate the final dose of exposure, 
calculate infection and illness probability, and DALYs -- and
returns a set of figures.

The function prints a running commentary as it works, so we send that
noise to nowhere with `results = "hide"` and look at the pictures
afterwards:

```{r run, results = "hide"}
library(dplyr)
scenario <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
regulation_reduction <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                          select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                          select(-c(Country, RegulationID, Reduction))
plots <- run_qmra_initial_situation(scenario = scenario, 
                     pathogen = c("Campylobacter jejuni", "Norovirus"),
                     regulationLog = regulation_reduction,
                     regulationConcentration = regulation_concentration)
```

What comes back is a **named list of five `ggplot` objects**:

```{r run-names}
names(plots)
```

These five steps trace the journey of raw water through to its effects on 
health. (This run takes into account only treatment-type barriers; to also 
include agricultural practice-type barriers, see `run_qmra_barrier()` and the 
comparative vignette below.)

## Reading the four figures

Print each element to see it. Here is what each one tells you.

**Log-reduction** -- how many log10 units each barrier in the train
removes according to pathogen group simulated (Bacteria, Viruses, Protozoa); a taller box means a more effective barrier.

```{r fig-logreduction}
plots$logreduction
```

**DALYs** -- the disability-adjusted life-years lost per person per year,
on a log scale. The **red line marks the objective**: boxes
sitting below it meet the tolerable-risk goal, boxes crossing above it do
not. The default value of 1e-6 DALY corresponds to the WHO target.

```{r fig-dalys}
plots$dalys
```

Each plot is a boxplot for a reason: `ambre` is a **Monte-Carlo** engine
that repeats the calculation many times, so a result is a *range* of
plausible values, not a single number. Reading that spread is the subject
of its own vignette.

**Regulation matrix**

The outcome of an exposure scenario can be assessed in terms of regulatory 
compliance with regard to:
- Log‑reduction (the decrease in pathogen concentration expressed in log10 
units: regulation_matrix_log()

```{r fig-matrix-log}
plots$regul_matrix_log
```

- Residual concentration of the contaminant in the effluent: 
regulation_matrix_concentration()

```{r fig-matrix-concentration}
plots$regul_matrix_concentration
```

The background color indicates whether the reduction results are sufficient in relation to the regulations (green: complies with or falls below regulatory requirements; red: fails to meet regulatory requirements)


## Other figures

Other figures can be generated :

**Inflow pathogen concentration**

The `plot_inflow()` function generates a violin plot showing the distribution of pathogen inflow concentrations simulated in a scenario. Each violin represents the variability of the inflow concentration for one pathogen, while colors distinguish pathogen groups.

```{r plot-inflow}
scenario_with_inflow <- inflow_concentration(scenario = scenario,
 pathogenName = c("Campylobacter jejuni", "Norovirus"))
plot_inflow(scenario_with_inflow)
```

A violin plot combines the features of a boxplot and a kernel density plot to visualize the distribution of simulated values.

- The **height** of each violin represents the range of simulated inflow concentrations for a given pathogen.
- The **width** of the violin at a given y-value is proportional to the density of simulations at that concentration:
  - **wider sections** indicate that many simulations produced similar inflow concentrations;
  - **narrower sections** indicate that fewer simulations fall within that range.
- The **shape** of the violin can reveal important characteristics of the distribution, such as skewness, multimodality, or the presence of multiple concentration regimes.
- Colors identify the pathogen group (Bacteria, Viruses, or Protozoa), allowing comparisons both within and between groups.

Because the y-axis is displayed on a **log₁₀ scale**, equal vertical distances correspond to multiplicative (rather than additive) differences in concentration. This scaling is particularly useful for microbial concentrations, which often span several orders of magnitude.

When interpreting the figure, wider violins indicate a higher probability of observing concentrations within a given range, while taller violins indicate a broader overall variability in simulated inflow concentrations.



**Volume of exposure**

The `plot_volumes()` function generates boxplots of the exposure volumes simulated for each combination of exposed population and exposure pathway. The plots are faceted by crop, making it possible to compare exposure volumes across crops, populations, and transmission pathways.

```{r plot-volume}
scenario_volume <-
  scenario_with_inflow |>                     
  mutate(
    volume = map(
      .x = config,                                         
      .f = ~ simulate_exposure(config = .x)              
    )
  )

plot_volumes(scenario_volume)
```
Each boxplot summarizes the distribution of simulated exposure volumes:

- the **horizontal line** inside the box represents the median exposure volume;
- the **box** spans the interquartile range (IQR), containing the middle 50% of simulated values;
- the **whiskers** extend to the most extreme observations within 1.5 × IQR from the box;
- observations beyond the whiskers are displayed as outliers;
- if a boxplot appears as a **single horizontal line**, all simulated values are identical. In this case, the minimum, first quartile, median, third quartile, and maximum are equal, indicating that no variability was observed in the simulated exposure volume.

Comparing boxplots allows users to assess differences in the typical exposure volume, the variability of simulated events, and the presence of extreme values among populations, crops, and exposure pathways.


**Probability of infection**

The `plot_infection_probability()` function generates boxplots of the simulated annual probability of infection for each pathogen. Separate plots are produced for each pathogen, allowing comparisons of infection probabilities across exposed populations and crops.

```{r plot-infection-probability}
scenario_dose_ini <- initial_dose_calculation(scenario_volume)

scenario_scheme <- update_treatment_scheme(scenario_dose_ini)

scenario_with_logreduc_and_co <- scenario_scheme |> 
  mutate( log_reduction = map(config, simulate_treatment))

scenario_final_dose_and_co_test <- final_dose_calculation(scenario_with_logreduc_and_co)

scenario_inf_proba_and_co <- infection_probability_calculation(scenario_final_dose_and_co_test)

scenario_illness_proba_and_co <- illness_probability_calculation(scenario_inf_proba_and_co)

scenario_dalys_and_co <- dalys_calculation(scenario_illness_proba_and_co)

scenario_risk_total_and_co <-get_risk_total(scenario_dalys_and_co)

plot_infection_probability(scenario_risk_total_and_co)
```


Each boxplot summarizes the distribution of simulated annual infection probabilities.
Comparing boxplots allows users to identify differences in both the magnitude and the variability of annual infection probabilities among exposed populations and crops for each pathogen simulated.


## Where to go next

You now have the mental map. From here, pick the thread you need -- open
any of these articles with `vignette()`, or browse them on the package
website:

- `vignette("b-treatment-vs-multibarrier", package = "ambre")` -- treatment
  plant only versus adding on-field barriers, side by side.
- `vignette("c-input-file", package = "ambre")` -- how to write your own
  Excel scenario, and the allowed crops, populations, paths and barriers.
- `vignette("d-interpreting-risk", package = "ambre")` -- reading results
  against the WHO 1e-6 DALY target and the EU 2020/741 framing.
- `vignette("g-monte-carlo-engine", package = "ambre")` -- why every result
  is a range, and how the repeat x event simulation works.
- `vignette("e-economic-analysis", package = "ambre")` -- putting a cost on
  a reuse scenario alongside its risk.
- `vignette("h-config-ambre", package = "ambre")` -- a tour of the bundled
  `config_ambre` database that powers every calculation.
- `vignette("f-what-if-exposure", package = "ambre")` -- calibrating
  exposure volume and frequency to test "what if" questions.

---
title: "Costing a reuse scenario"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Costing a reuse scenario}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)
```

```{r setup}
library(ambre)
set.seed(2024)
```

Safety is not the only thing that decides a reuse scheme -- someone has to pay
for it. Alongside the health assessment, `ambre` estimates what a scenario
**costs**: the up-front investment and the yearly bill of the barriers it puts in
place. This vignette walks through that costing. It pairs naturally
with the risk side in `vignette("b-initial-vs-new-scenario", package = "ambre")`
-- the whole point being to weigh what each strategy *buys* against what it
*costs*.

## The three levers

`run_economic_analysis()` takes your scenario plus three financial parameters 
and boolean to indicate to calculate cost for initial situation supplementary 
process:

```{r signature, eval = FALSE}
run_economic_analysis(scenario, membership_fee, 
                      price_per_m3, grant, initialSituation = FALSE)
```

- **`membership_fee`** -- annual membership fee to ASA (Association Syndicale Autorisée).
- **`price_per_m3`** -- a volumetric charge on the water actually used (`€/m3`),
  applied to the scenario's computed irrigation need.
- **`grant`** -- the share of the investment covered by a subsidy, as a
  **fraction between 0 and 1**. A `grant` of 0.5 means public money pays half the
  capital cost, so users finance the remaining `(1 - grant)`.
- **initialSituation** -- TRUE to calculate cost of the processes indicated in 
  InitialProcessName column of the scenario of FALSE to calculate cost of the 
  processes indicated in SupplementaryProcessName column.

## What the model computes

Before any cost, `run_economic_analysis()` calls `irrigation_need_calculation()`
to work out how much water each row needs (from its crop and area). It then
splits the bill along two axes:

- **capex vs opex** -- the one-off capital cost of installing a barrier versus
  its recurring operating cost. Capital costs are spread over a **20-year**
  amortization when turned into an annual figure.
- **collective vs individual** -- treatment shared across the scheme versus
  barriers installed per crop. The shared cost is allocated in proportion to the
  user-financed area, `(1 - grant) x area`.

The per-barrier unit prices come from `config_ambre$economic$cost`, each with a
`unit` string (`€/m3`, `€/ha`, `€/ml` of perimeter, `€/p` per person...):

```{r cost-table}
config_ambre$economic$cost[, c("TreatmentName", "CostType", "value", "unit")] |>
  head(8)
```

## Run it

Use a richer example than the two-row starter -- the learning case has six rows:

```{r scenario}
scenario <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
```

```{r run, results = "hide"}
plots <- run_economic_analysis(
  scenario,
  membership_fee = 200,   # €/ha/year
  price_per_m3   = 0.1,   # €/m3
  grant          = 0.5,    # half the capital cost is subsidised,
  initialSituation = FALSE # calcul the cost of Supplementary process
)
```

It returns two `ggplot`s and the allocation key table. **Annual cost** compares the recurring yearly bill of
the situation considered (initial situation or new scenario):

```{r annual}
plots$annual
```

**Capex** compares their up-front investment:

```{r capex}
plots$total_investement
```

This graph is display only if **initialSituation = FALSE**, as the initial 
situation corresponds to the current situation and therefore does not require 
any investment.


```{r allocation-key}
plots$allocation_key
```

This allocation key is calculated by default in function `collective_treatment_cost`, but it can be customised by the user using parameter `allocation_key`, which is set to **NULL** by default. This parameter accepts a vector of percentage values of the same size as the number of simulated crops.

```{r run-custom}

crop <- scenario$CropName
allocation_custom <- data.frame(CropName = crop,
                                allocation = c(0.5, 0.5))
run_economic_analysis(
  scenario,
  membership_fee = 200,   
  price_per_m3   = 0.1,   
  grant          = 0,   
  initialSituation = FALSE, 
  allocation_key = allocation_custom # No subvention, collective treatment price 50% for each crop
)
```

## Getting the underlying numbers

The `run_` function returns only plots. To get the figures behind them, call the
costing functions yourself. They expect the scenario to carry its irrigation
need first, so run `irrigation_need_calculation()` before them:

```{r numbers, results = "hide"}
scenario_need <- irrigation_need_calculation(scenario)

water_price <- water_price(scenario = scenario_need,
                          price_per_m3 = 0.01,
                          membership_fee = 200)
supplementary_cost <- process_cost_calculation(scenario = scenario_need,
                                              membership_fee = 200,
                                              charge = 0.1,
                                              initialSituation = FALSE)

```

Each returns the scenario augmented with cost columns; the added columns are the
ones to inspect:

```{r numbers-cols}
setdiff(names(water_price), names(supplementary_cost))
```

## Caveats worth flagging

- **The situations are modelled asymmetrically.** The initial and supplementary
  process cases do not carry exactly the same cost structure, so read the 
  comparison as orders of magnitude rather than a precise like-for-like tender.

## Adapting the cost base

The prices are not hard-coded in the functions -- they live in a CSV. To cost a
scheme for your own territory, edit `data-raw/ambre_barriere_cout.csv` (keeping
the `unit` convention), then rebuild the bundled dataset by sourcing
`data-raw/config_ambre.R`. The next `run_economic_analysis()` will use your
numbers. The database and this rebuild step are described in
`vignette("h-config-ambre", package = "ambre")`.

---
title: "Interpreting risk against the WHO target"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Interpreting risk against the WHO target}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)
```

```{r setup}
library(ambre)
set.seed(2024)
```

```{r libs}
library(dplyr)
library(purrr)
```

`ambre` produces figures, but the real deliverable is a **decision**: is this
reuse scenario safe enough? This vignette explains the benchmark, how the
package gets from single exposure events to a yearly figure, and how to read the
resulting distribution against the regulatory objective.

## The objective

A reference is the World Health Organization (WHO) tolerable-risk target of

> **no more than 1e-6 DALY per person per year**.

-- one micro disability-adjusted life-year, i.e. roughly one healthy year of
life lost per million exposed people per year. It underpins the WHO guidelines
for wastewater reuse (2006, 2016), is the health basis behind **EU Regulation
2020/741** on water reuse (with its reclaimed-water classes A-D), the French
**arrêté du 18 décembre 2023**, and **ISO 16075**. 

`ambre`'s DALY plot draws
this target as a red line so you can read acceptability at a glance.
And you can adjust this target to other regulation by changing the value of the
parameter `objective` in `plot_dalys` function.

## From an event to a year

A single splash of water is a tiny risk. What matters is the accumulated risk
over a **year** of repeated exposures. `get_risk_total()` performs that
aggregation. For each Monte-Carlo run it combines the per-event probabilities
into annual ones with the standard independent-events formula

$$P_\text{year} = 1 - \prod_\text{events} (1 - p_\text{event})$$

and sums the per-event DALYs. It writes a `risk_total` table with, per run:

- `infectionProb_sum` -- annual probability of infection, `1 - prod(1 - p)`;
- `illnessProb_sum` -- annual probability of illness (infection weighted by the
  chance it makes you ill);
- `dalys_sum` -- the annual DALY lost, the number you compare to 1e-6.

## Why the result is a distribution

Here is the point most easily missed: **one row of your scenario does not give
one risk number, it gives a thousand of them.** `ambre` repeats the whole
calculation `number_of_repeatings` times (1000 by default), each time redrawing
concentrations, volumes and log-reductions from their distributions. The index
that separates these runs is `repeatID`. So `dalys_sum` is a *sample* of 1000
plausible annual burdens, capturing the dispersion in the inputs. You interpret
its **spread**, not a single point. (For the machinery behind this, see
`vignette("g-monte-carlo-engine", package = "ambre")`.)

## A worked example with the numbers

Let us pull the actual numbers out. `run_qmra_initial_situation()` and
`run_qmra_supplementary_process()` run this exact chain internally and returns 
only plots, so to reach the underlying table we reproduce its steps and stop at 
`get_risk_total()`:

```{r chain, results = "hide"}
scenario <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario <- inflow_concentration(scenario, pathogenName = "Campylobacter jejuni")
scenario <- scenario |> mutate(volume = map(config, ~ simulate_exposure(config = .x)))
scenario <- initial_dose_calculation(scenario)
scenario <- update_treatment_scheme(scenario, initial_situation = TRUE)
scenario <- scenario |> mutate(log_reduction = map(config, ~ simulate_treatment(.x)))
scenario <- final_dose_calculation(scenario)
scenario <- infection_probability_calculation(scenario)
scenario <- illness_probability_calculation(scenario)
scenario <- dalys_calculation(scenario)
scenario <- get_risk_total(scenario)
```

Unnest the `risk_total` list-column and you have one row per Monte-Carlo run:

```{r unnest}
rt <- bind_rows(scenario$risk_total)
nrow(rt)         # 1000 runs x 2 scenario rows
```

Now read the annual DALY distribution against the target. The median, the upper
tail, and -- most usefully for a decision -- the **fraction of runs that breach
1e-6**:

```{r quantiles}
quantile(rt$dalys_sum, c(0.05, 0.5, 0.95))
mean(rt$dalys_sum > 1e-6)     # share of Monte-Carlo runs above the WHO target
```

A median comfortably below 1e-6 with only a small fraction of runs above it is a
very different message from a median that straddles the line: the first says
"safe with margin", the second "safe on average but not robustly". Reporting the
exceed fraction makes that distinction explicit.

## Reading the DALY plot

The same information, visually, is what `plot_dalys()` shows -- and what
`run_qmra_initial_situation$dalys` `run_qmra_supplementary_proocess$dalys` 
return:

```{r dalys-plot, results = "hide"}
library(dplyr)
scenario_example <- create_scenario(filepath = system.file("input_1culture_2pop.xlsx", package = "ambre"))
regulation_reduction <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                          select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> filter(Country == "France") |>
                        select(-c(Country, RegulationID, Reduction))

plots <- plot_comparison_qmra_initial_vs_supplementary_processes(
  scenario = scenario_example,
  pathogen = c("Campylobacter jejuni"),
  regulationLog = regulation_reduction,
  regulationConcentration = regulation_concentration
)
```

```{r dalys-show}
plots$dalys
```

Each box is the distribution of `dalys_sum` for one crop x population, on a
**log10** y-axis. The **red line is the 1e-6 target**: a box sitting entirely
below it meets the goal; a box straddling or above it does not.


## Concluding

Turn the reading into an action:

- **Median and 95th percentile below 1e-6** -> the scenario meets the target;
  document the margin.
- **The line runs through the box** -> not robust. You need to cut the risk.

Two levers do that, each with its own vignette:

- reduce the pathogen **concentration** with more or better barriers --
  `vignette("b-initial-vs-new-scenario", package = "ambre")`;
- reduce the **exposure** itself (PPE, fewer contacts) --
  `vignette("f-what-if-exposure", package = "ambre")`.

---
title: "The Monte-Carlo engine"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{The Monte-Carlo engine}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)
```

```{r setup}
library(ambre)
set.seed(2024)
```

What separates `ambre` from a spreadsheet is that almost nothing in it is a
single number. Concentrations, exposure volumes and log-reductions are all
**probability distributions**, and the package draws from them thousands of
times to report a *range* of risk rather than one fragile point estimate. This
vignette opens up that Monte-Carlo core: the two loops it runs, the distributions
it understands, and how to reproduce a result. It is the machinery behind the
boxplots in `vignette("a-get-started", package = "ambre")` and the spread you
interpret in `vignette("d-interpreting-risk", package = "ambre")`.

## Two dimensions: repeats and events

Every simulated quantity is drawn on a grid of two indices, both controlled by
`config_ambre$exposure`:

```{r exposure-control}
config_ambre$exposure[, c("name", "type", "value", "min", "max")]
```

- **`number_of_repeatings`** (default **1000**) is the number of Monte-Carlo
  **runs**. Each run is one plausible world; together they form the *uncertainty
  range*. This index is called `repeatID`.
- **`number_of_exposures`** (default **365**) is the number of exposure
  **events** in a year. This index is called `eventID`, and it is what the annual
  risk aggregation `1 - prod(1 - p)` runs over.

So a single draw of, say, exposure volume is really a `repeatID x eventID` table
-- 1000 x 365 values -- and the whole pipeline carries that shape through to the
final DALYs.

## The distribution catalog

The third row above, `volume_perEvent`, is a `triangle` with `min` 0.5 and `max`
3. That `type` / `value` / `min` / `max` / `mode` / `mean` / `sd` / `meanlog` /
`sdlog` block is the **shared vocabulary** of the whole database: the same nine
columns describe an inflow concentration, a treatment log-reduction and a cost.
`create_random_distribution()` reads that vocabulary. The supported `type` values
are:

| `type` | drawn with | parameters used |
|---|---|---|
| `value` | (constant) | `value` |
| `uniform` | `runif` | `min`, `max` |
| `log10_uniform` | `10^runif` | `min`, `max` (as log10) |
| `norm` | `rnorm` | `mean`, `sd` |
| `log10_norm` | `10^rnorm` | `mean`, `sd` (as log10) |
| `lognorm` | `rlnorm` | `meanlog`, `sdlog` |
| `triangle` | `EnvStats::rtri` | `min`, `max`, `mode` |

Call it directly to see what a draw looks like. It returns an `events` table
(one row per `repeatID x eventID`) and the `paras` actually used:

```{r crd}
draw <- create_random_distribution(
  type = "uniform",
  number_of_repeatings = 3,
  number_of_events = 5,
  min = 1, max = 10,
  debug = FALSE
)
head(draw$events)
draw$paras
```

In practice you rarely call it by hand. The pipeline calls
`generate_random_values()`, a thin wrapper that reads one row of a config table
and fills in sensible defaults before delegating. Feed it the real
`volume_perEvent` row and you get the exact draw the engine would use for
exposure volume:

```{r grv}
vol_row <- config_ambre$exposure[config_ambre$exposure$name == "volume_perEvent", ]
gv <- generate_random_values(
  vol_row,
  number_of_repeatings = 2,
  number_of_events = 4,
  debug = FALSE
)
summary(gv$events$volume_perEvent)
```

The distribution schema is described further in
`vignette("h-config-ambre", package = "ambre")`.

## From min/max to a spread

For the normal and log-normal families you often supply only a `min` and a
`max`, not a mean and standard deviation. `ambre` derives the missing parameters
so that a given fraction of the mass falls inside `[min, max]`, using
`get_percentile()` (a wrapper around the normal quantile) together with
`default_min()` / `default_max()`. For example, the z-score that places 90% of a
normal distribution within the min-max window is:

```{r percentile}
ambre:::get_percentile(0.9)
```

You do not normally call these yourself, but knowing they exist explains why a
`norm` row with only `min` and `max` still produces a sensible bell curve.

## A special case worth knowing

`generate_random_values()` has one silent behaviour. A `triangle` whose `min`
equals its `max` cannot be a triangle, so it is quietly converted to a
(degenerate) `uniform` -- effectively a constant:

```{r triangle-collapse}
degenerate <- data.frame(
  type = "triangle", value = NA,
  min = 2, max = 2, mode = NA,
  mean = NA, sd = NA, meanlog = NA, sdlog = NA
)
gv2 <- generate_random_values(degenerate, number_of_repeatings = 1,
                              number_of_events = 3, debug = FALSE)
unique(gv2$events$values)
```

You will see the message *"Distribution set from 'triangle' to 'uniform' because
'min' equals 'max'"* in a real run when this happens -- it is expected, not an
error.

## Why barrier credits add up

Log-reductions combine by **addition in log space**. A train that removes 2 log
at the plant and 3 log in the field removes 5 log overall, and 5 log means the
surviving concentration is `10^-5` of the inflow. Internally `ambre` builds a
wide `repeatID x eventID` table of each barrier's draw and sums the columns, so
the additivity holds *run by run*, preserving the correlation structure across
the Monte-Carlo sample. This is why you can compare a treatment strategy and a
barrier strategy on the same footing (see
`vignette("b-initial-vs-new-scenario", package = "ambre")`).

## Reproducibility

Because every draw is random, set a seed before a run you want to reproduce.
Same seed, same numbers:

```{r seed}
set.seed(1)
a <- create_random_distribution(type = "uniform", number_of_repeatings = 1,
                                number_of_events = 4, min = 0, max = 1, debug = FALSE)
set.seed(1)
b <- create_random_distribution(type = "uniform", number_of_repeatings = 1,
                                number_of_events = 4, min = 0, max = 1, debug = FALSE)
identical(a$events$values, b$events$values)
```

To read the uncertainty a run expresses, summarise a quantity across `repeatID`
with quantiles -- exactly what the final DALY boxplots do, and what
`vignette("d-interpreting-risk", package = "ambre")` walks through.

---
title: "Preparing your input file"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Preparing your input file}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 7,
  fig.height = 4.5,
  out.width = "100%"
)
```

```{r setup}
library(ambre)
set.seed(2024)
```

Every `ambre` analysis starts from one Excel file describing your reuse
scenario. Get that file right and the rest of the pipeline just runs; get a
label wrong and `create_scenario()` stops with an error. This vignette is the
reference for building that file: the nine columns, their allowed values, and
the traps to avoid. If you have not seen a run yet, read
`vignette("a-get-started", package = "ambre")` first.

## Anatomy of the file

The file has **ten columns**, one row per situation you want to assess. Here is
a bundled two-row example -- use it as a template:

```{r template}
readxl::read_excel(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
) |>
  knitr::kable()
```

| Column | Meaning |
|---|---|
| `CropName` | the crop being irrigated |
| `Area` | irrigated area, in hectares |
| `PopulationName` | the exposed population for this crop |
| `nb_population` | how many people are exposed |
| `PathName` | the exposure pathway (how the water reaches the population exposed) |
| `STEPtreatmentName` | the Wastewater Treatment Plant (WWTP) process |
| `CollectiveTreatmentName` | additional treatment after the WWTP, common to all crop|
| `InitialProcessName` | individual process (equipment or practice) step currently in place |
| `SupplementaryProcessName` | individual process (equipment or practice) step of a new scenario to evaluate |
| `nb_decay` | an on-field barrier (equipment or practice) |

Each row is one **crop x population x pathway** combination. A scenario with two
crops, each exposing three populations, is six rows.

## Validity bounds and empty cells

Two numeric columns are range-checked when you build the scenario:

- `Area` must be **greater than 0 and below 10000** hectares;
- `nb_population` must be **greater than 0 and below 5000**.

The four barrier columns (`STEPtreatmentName`, `CollectiveTreatmentName`,
`InitialProcessName`, `SupplementaryProcessName`) may be left **empty
(`NA`)** when a scheme does not use that slot -- for instance a row with a plant
step and a field barrier but no collective treatment. On the other hand, a cell 
may contain **several processes separated by commas** (i.e. "P.9 - Peeling, P.8 - Drying").
The name columns, by contrast, **must** match a known value exactly 
(indicated in `instruction` sheet), 
and that is what the rest of this vignette is about.

## The allowed values

The names you type are resolved against `config_ambre`, the bundled database.
Printing the catalogs straight from that object guarantees this list stays true
to the package you actually have installed.

**Crops** -- the `CropName` column:

```{r crops}
config_ambre$crop[, c("CropName", "CropDescription", "CropHeight")]
```

**Populations** -- the `PopulationName` column:

```{r populations}
dplyr::distinct(
  config_ambre$path$description[, c("PopulationID", "PopulationName")]
)
```

**Exposure pathways** -- the `PathName` column must match exactly a `PathDescription`
listed below:

```{r paths}
config_ambre$path$description[, c("PathID", "PathDescription", "PopulationName")] |>
  knitr::kable()
```

**Barriers and treatments** -- any of the `STEPtreatmentName`,
`CollectiveTreatmentName`, `InitialProcessName` or
`SupplementaryProcessName` cells must be one or more of these names, separated 
by a **comma* (Q. = treatment, E. = equipment, P. = practice):

```{r barriers}
sort(unique(config_ambre$treatment$processes$TreatmentName))
```

The pathogens you simulate are *not* set in the Excel file -- you pass them to
`run_qmra_intial_situation()` / `run_qmra_supplementary_process()`. 
The 6 that can be simulated are :

```{r pathogen}
library(dplyr)

pathogen <- config_ambre$health %>%
  filter(!if_else(is.na(infection_to_illness) & is.na(dalys_per_case), TRUE, infection_to_illness == dalys_per_case, missing = FALSE))
pathogen$PathogenName

```
More pathogen are available in `config_ambre$health` but parameters 
`nfection_to_illness` and `dalys_per_case` are not yet available.

## Known pitfalls

- **Spelling is exact and unforgiving.** `PathName` must reproduce a
  `PathDescription` exactly, including case and punctuation; `CropName` must be
  `Potato`, `Corn seed`, etc. A stray "s" is enough to fail. Even a bundled
  example, `input_cas_apprentissage_complet.xlsx`, is worth cross-checking:

  ```{r check-complet}
  vals <- readxl::read_excel(
    system.file("input_cas_apprentissage_complet.xlsx", package = "ambre")
  )$CropName
  setdiff(unique(vals), config_ambre$crop$CropName)   # crop names with no match
  ```

  Anything returned here is a label the database will not recognise. 
  To help you complete the Excel file without making any mistakes, use the 
  ‘instructions’ table, where you will find the names to use.

- **`PathName` implies its population.** Each pathway is tied to a population in
  the database, so keep `PopulationName` consistent with the `PathName` you
  choose.

## Validate your file

The check is simply to build the scenario. On a good file it returns a tibble
with one row per input row and the resolved integer IDs:

```{r validate}
scenario <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario[, c("CropName", "CropID", "PopulationName", "PopulationID",
             "PathName", "PathID")]
```

If a name is missing or misspelled, `create_scenario()` stops rather than
guessing. Fix the offending cell against the catalogs above and try again. Once
the scenario builds, you are ready to run the assessment -- see
`vignette("a-get-started", package = "ambre")` -- and to inspect or extend the
database behind these names in `vignette("h-config-ambre", package = "ambre")`.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_treatment.R
\name{simulate_treatment}
\alias{simulate_treatment}
\title{Simulate Treatment}
\usage{
simulate_treatment(config, wide = FALSE, debug = TRUE, lean = FALSE)
}
\arguments{
\item{config}{A configuration object containing treatment scheme and process information.}

\item{wide}{Logical indicating whether to return results in wide format.}

\item{debug}{Logical indicating whether to print debug information.}

\item{lean}{Logical indicating whether to return only essential results for web applications.}
}
\value{
If \code{lean} is TRUE, returns a list containing only the long-format treatment events.
If \code{wide} is TRUE, returns a list containing long-format events, wide-format events,
wide-format scheme events, treatment schemes, and treatment parameters.
If \code{wide} is FALSE, returns a list containing long-format events, treatment schemes,
and treatment parameters.
}
\description{
This function simulates treatment data for pathogen processes based on the provided configuration.
It generates treatment events and parameters, and can return results in either long or wide format.
}
\details{
This function does the same thing as kwb.qmra::simulate_treatment(),
but it is bundled with \emph{ambre} to avoid a hard dependency on the
unmaintained package.
}
\examples{
# Example usage of simulate_treatment
\donttest{
scenario_test <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario_updated <- update_pathogen(scenario = scenario_test,
 pathoName = c("Campylobacter jejuni","Rotavirus"))
config <- scenario_updated$config[[1]]

# Simulate treatment data
result <- simulate_treatment(config, wide = TRUE, debug = FALSE, lean = FALSE)

}
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regulation_matrix_concentration.R
\name{regulation_matrix_concentration}
\alias{regulation_matrix_concentration}
\title{Concentration reduction matrix calculation}
\usage{
regulation_matrix_concentration(scenario, regulation)
}
\arguments{
\item{scenario}{data.frame with log_reduction column obtained with simulate_treatment function}

\item{regulation}{data.frame regulation table to compare to simulation results}
}
\value{
formattable
}
\description{
Calculate the difference of achieve concentration in effluent according to regulation
}
\examples{
\donttest{
library(dplyr)
library(purrr)
scenario_example <- create_scenario(system.file("input_1culture_2pop.xlsx", 
                                                package = "ambre"))

scenario_exposure <- scenario_example |> mutate(
     volume = map(
         .x = config,                                         
         .f = ~ simulate_exposure(config = .x)))

scenario_patho <- update_pathogen(scenario = scenario_exposure, 
                                  pathoName = c("Rotavirus", "Campylobacter jejuni", "Norovirus"))
concentration = map(.x = scenario_patho$config,
                   .f= ~ simulate_inflow(.x))
scenario_concentration <- scenario_patho |> mutate(inflow_concentration = concentration)
scenario_ini_dose <- initial_dose_calculation(scenario = scenario_concentration)

scenario_barrier <- update_treatment_scheme(scenario = scenario_ini_dose, initial_situation = TRUE)

scenario_barrier$config <- purrr::pmap(
  list(
    config = scenario_barrier$config,
    barrierID = scenario_barrier$SupplementaryProcessID,
    cropHeight = scenario_barrier$CropHeight,
    matrixID = scenario_barrier$MatrixID,
    popID = scenario_barrier$PopulationID,
    decay = scenario_barrier$nb_day_decay
  ),
  function(config, barrierID, cropHeight, matrixID, popID, decay) {

    if (barrierID \%in\% c(24, 26, 27, 28) &&
        matrixID \%in\% c(3, 5)) {

      update_logreduction_specific_barrier(
        config = config,
        cropHeight = cropHeight,
        barrierID = barrierID
      )

    } else if (barrierID \%in\% c(12)) {

      update_logreduction_decay(
        config = config,
        cropHeight = cropHeight,
        barrierID = barrierID,
        popID = popID,
        nb_day_decay = decay
      )

    } else if (barrierID \%in\% c(9,13,14,15,16,17,18,24,26,27,28)) {

      update_logreduction_path(
        config = config,
        matrixID = matrixID,
        barrierID = barrierID,
        popID = popID
      )

    } else {
      config
    }
  }
)
  
scenario_logreduction <- scenario_barrier |> 
    mutate(log_reduction = map(config, simulate_treatment))

regulation_concentration <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Country, RegulationID, Reduction))

regulation_matrix_concentration(scenario = scenario_logreduction,
                                regulation = regulation_concentration)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inflow_concentration.R
\name{inflow_concentration}
\alias{inflow_concentration}
\title{Inflow concentration}
\usage{
inflow_concentration(scenario, pathogenName, monteCarlo = 1000)
}
\arguments{
\item{scenario}{list scenario created with create_scenario}

\item{pathogenName}{character names of the pathogens to be simulated}

\item{monteCarlo}{integer number of simulation iterations for Monte Carlo}
}
\value{
scenario_with_concentration
}
\description{
To calculate inflow concentration for specified pathogen
}
\examples{
scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
result <- inflow_concentration(scenario = scenario_test,
 pathogenName = c("Campylobacter jejuni", "Norovirus"),
 monteCarlo = 2000)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_inflow.R
\name{plot_inflow}
\alias{plot_inflow}
\title{Plot inflow concentration}
\usage{
plot_inflow(scenario)
}
\arguments{
\item{scenario}{data.frame with inflow_concentration column obtained from inflow_concentration function}
}
\value{
plot
}
\description{
Violin plot of inflow concentration according to pathogen simulated
}
\examples{
scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
scenario_with_inflow <- inflow_concentration(scenario = scenario_test,
 pathogenName = c("Campylobacter jejuni", "Norovirus"))
plot_inflow(scenario_with_inflow)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perimeter_calculation.R
\name{perimeter_calculation}
\alias{perimeter_calculation}
\title{Perimeter calculation}
\usage{
perimeter_calculation(scenario)
}
\arguments{
\item{scenario}{data.frame scenario obtained with create_scenario function}
}
\value{
scenario_updated
}
\description{
Calculate perimeter (linear meter) according to surface, and add the value to the scenario dataframe
}
\examples{
scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)

scenario_apprentissage_updated <- perimeter_calculation(scenario_apprentissage)
print(scenario_apprentissage_updated$perimeter)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/plot_comparison_qmra_initial_vs_supplementary_processes.R
\name{plot_comparison_qmra_initial_vs_supplementary_processes}
\alias{plot_comparison_qmra_initial_vs_supplementary_processes}
\title{Plot comparison QMRA calculation with initial vs supplementary processes}
\usage{
plot_comparison_qmra_initial_vs_supplementary_processes(
  scenario,
  pathogen,
  regulationLog,
  regulationConcentration
)
}
\arguments{
\item{scenario}{data.frame, scenario obtained with create_scenario function}

\item{pathogen}{character, Name of the pathogen to simulate}

\item{regulationLog}{data.frame regulation value of the log reduction to compare with simulation result}

\item{regulationConcentration}{data.frame regulation value of the log reduction to compare with simulation result}
}
\value{
combine_plots
}
\description{
Compare the result of the QMRA simulation with reduction_log from initial vs supplementary processes
}
\examples{
\donttest{
library(dplyr)
scenario_example <- create_scenario(
  filepath = system.file("input_1culture_2pop.xlsx", package = "ambre")
)


regulation_reduction <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Country, RegulationID, Reduction))

plot_comparison_qmra_initial_vs_supplementary_processes(
  scenario = scenario_example,
  pathogen = c("Campylobacter jejuni", "Norovirus"),
  regulationLog = regulation_reduction,
  regulationConcentration = regulation_concentration
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enumeration.R
\name{enumeration}
\alias{enumeration}
\title{Enumerate Elements}
\usage{
enumeration(x, type = "element", sorted = FALSE, suffix = "")
}
\arguments{
\item{x}{vector of elements to be enumerated}

\item{type}{type of element to appear in the message. Default: "element"}

\item{sorted}{logical. Whether or not to print the available elements in
lexical order. Default: \code{FALSE}}

\item{suffix}{suffix to be appended to \code{type}. Can be used to
distinguish between singular and plural form. Default: \code{""}}
}
\value{
character string containing the enumeration
}
\description{
This function does the same thing as kwb.utils::enumeration(),
but it is bundled with \emph{ambre} to avoid a hard dependency on the
unmaintained package.
}
\examples{

# Example usage of enumeration
cat(enumeration(c("a", "b", "c"), type = "element"))
cat(enumeration(c("x", "y"), type = "element", suffix = "s"))
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_logreduction_path.R
\name{update_logreduction_path}
\alias{update_logreduction_path}
\title{Update log reduction for specific path}
\usage{
update_logreduction_path(config, matrixID, popID, barrierID)
}
\arguments{
\item{config}{list configuration simulated}

\item{matrixID}{numeric, ID of the matrix simulated}

\item{popID}{numeric, ID of the population simulated}

\item{barrierID}{numeric, ID of the barrier simulated}
}
\value{
config_updated
}
\description{
Update the log reduction value for specific barrier according to path and population simulated
}
\examples{
scenario_example <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)

scenario_example$config <- purrr::pmap(
  list(
    config = scenario_example$config,
    barrierID = scenario_example$SupplementaryProcessID,
    matrixID = scenario_example$MatrixID,
    popID = scenario_example$PopulationID
  ),
  function(config, barrierID, matrixID, popID) {

    if (barrierID \%in\% c(9,13,14,15,16,17,18,24,26,27,28)) {

      update_logreduction_path(
        config = config,
        matrixID = matrixID,
        barrierID = barrierID,
        popID = popID
      )

    } else {
      config
    }
  }
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_crop_height.R
\name{query_crop_height}
\alias{query_crop_height}
\title{Query crop height}
\usage{
query_crop_height(cropName)
}
\arguments{
\item{cropName}{integer, identifier of the crop simulated}
}
\value{
height
}
\description{
Retrieve height of the crop simulated
}
\examples{
query_crop_height("Tomato")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_random_distribution.R
\name{create_random_distribution}
\alias{create_random_distribution}
\title{Create random distribution}
\usage{
create_random_distribution(
  type = "uniform",
  number_of_repeatings = 1,
  number_of_events = 365,
  value = 10,
  min = 10,
  max = 1000,
  percent_within_minmax = 0.9,
  min_zero = 0.01,
  log10_min = default_min(min, max, min_zero, f = log10),
  log10_max = default_max(max, min_zero * 10, f = log10),
  log10_mean = (log10_min + log10_max)/2,
  log10_sdev = abs((log10_max - log10_mean)/get_percentile(percent_within_minmax)),
  mean = (default_min(min, max, min_zero) + default_max(max, 10 * min_zero))/2,
  sdev = abs((default_max(max, 10 * min_zero) -
    mean)/get_percentile(percent_within_minmax)),
  meanlog = mean(log(default_min(min, max, min_zero) + default_max(max, 10 *
    min_zero))/2),
  sdlog = abs(sd(c(default_min(min, max, min_zero, f = log), default_max(max, 10 *
    min_zero, f = log)))),
  mode = (default_min(min, max, min_zero) + default_max(max, 10 * min_zero))/2,
  debug = TRUE
)
}
\arguments{
\item{type}{"uniform" calls runif(), "log10_uniform" calls
10^runif(number_of_events, log10_min, log10_max), "triangle" calls
EnvStats::rtri(), "lognorm" calls rlnorm(), "norm" calls rnorm() and
"log10_norm" calls 10^rnorm(number_of_events, mean = log10_mean,
sdev = log10_sdev), (default: "uniform")}

\item{number_of_repeatings}{how often should the random distribution with the
same parameters be generated (default: 1)}

\item{number_of_events}{number of events}

\item{value}{constant value (no random number), gets repeated number_of_events
times (if 'type' = 'value')}

\item{min}{minimum value (default: 10), only used if 'type' is "runif" or
"triangle"}

\item{max}{maximum value (default: 1000), only used if 'type' is "runif" or
"triangle"}

\item{percent_within_minmax}{percent of data point within min/max (default:
0.9 i.e. 90 percent}

\item{min_zero}{only used if 'type' is "log10_uniform" or
"log10_norm", "norm" or "lognorm" and "min" value equal zero.
In this case the zero is replaced by this value (default: 0.01), see also
\code{default_min()}}

\item{log10_min}{minimum value (default: default_min(min, max,
min_zero, f = log10)), only used if 'type' is "log10_uniform" or "log10_norm"}

\item{log10_max}{maximum value (default: ifelse(max > 0, log10(max),
log10_zero_threshold), only used if 'type' is "log10_uniform" or "log10_norm"}

\item{log10_mean}{mean value (default: (log10_min + log10_max)/2), only used
if 'type' is "log10_norm"}

\item{log10_sdev}{standard deviation (default: abs((log10_max- log10_mean) /
get_percentile(0.95)), only used if 'type' is "log10_norm"}

\item{mean}{mean value (default: (default_min(min, max, min_zero) /
default_max(max, 10*min_zero)) / 2), only used if 'type' is "norm"}

\item{sdev}{standard deviation (default: abs((default_max(max, 10*min_zero) -
mean) / get_percentile(0.95))), only used if 'type' is "norm"}

\item{meanlog}{log mean value (default: mean(log((min + max) / 2))), only
used if 'type' is "lognorm"}

\item{sdlog}{standard deviation (default: abs(sd(c(default_min(min, max,
min_zero, f = log)))) ), only used if 'type' is "lognorm"}

\item{mode}{(default: default_min(min, max, min_zero) +
default_max(max, 10 * min_zero) / 2), only used if 'type' is "triangle"}

\item{debug}{print debug information (default: TRUE)}
}
\value{
list with parameters of user defined random distribution and
corresponding values
}
\description{
This function does the same thing as kwb.utils::create_random_distribution(),
but it is bundled with \emph{ambre} to avoid a hard dependency on the
unmaintained package.
}
\examples{

# Example usage of create_random_distribution
# Uniform distribution
uniform_dist <- create_random_distribution(
  type = "uniform",
  number_of_repeatings = 2,
  number_of_events = 10,
  min = 0,
  max = 100
)

# Log10 uniform distribution
log10_uniform_dist <- create_random_distribution(
  type = "log10_uniform",
  number_of_events = 10,
  min = 1,
  max = 1000
)

# Triangle distribution
triangle_dist <- create_random_distribution(
  type = "triangle",
  number_of_events = 10,
  min = 0,
  max = 100,
  mode = 50
)

# Normal distribution
norm_dist <- create_random_distribution(
  type = "norm",
  number_of_events = 10,
  mean = 50,
  sdev = 10
)

# Log10 normal distribution
log10_norm_dist <- create_random_distribution(
  type = "log10_norm",
  number_of_events = 10,
  log10_mean = 2,
  log10_sdev = 0.5
)

# Lognormal distribution
lognorm_dist <- create_random_distribution(
  type = "lognorm",
  number_of_events = 10,
  meanlog = 2,
  sdlog = 0.5
)

# Constant value
value_dist <- create_random_distribution(
  type = "value",
  number_of_events = 10,
  value = 42
)
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retrieve_specific_barrier.R
\name{retrieve_specific_barrier}
\alias{retrieve_specific_barrier}
\title{Apply barriers' specific log-reduction to a config}
\usage{
retrieve_specific_barrier(
  config,
  barrierID,
  cropHeight,
  matrixID,
  popID,
  decay
)
}
\arguments{
\item{config}{A per-row config (list) from the scenario.}

\item{barrierID}{One or more supplementary barrier ids (a scalar or a vector);
each is applied in turn. \code{NA} matches no branch and is skipped.}

\item{cropHeight}{The crop height.}

\item{matrixID}{The exposure-matrix id.}

\item{popID}{The exposed-population id.}

\item{decay}{The number of decay days (for natural die-off barriers).}
}
\value{
The updated config.
}
\description{
For one or more supplementary barriers, updates the per-config log-reduction
according to each barrier's type: pathway-specific barriers use
\code{\link[=update_logreduction_path]{update_logreduction_path()}}, natural die-off uses
\code{\link[=update_logreduction_decay]{update_logreduction_decay()}}, and product/soil-specific barriers use
\code{\link[=update_logreduction_specific_barrier]{update_logreduction_specific_barrier()}}. Barriers are applied in turn so
their reductions accumulate; any other barrier leaves the config unchanged.
Extracted from \code{run_qmra_initial_situation()} /
\code{run_qmra_supplementary_process()} (was an inline anonymous function).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_scenario_from_df.R
\name{create_scenario_from_df}
\alias{create_scenario_from_df}
\title{Create Exposure Scenario from a data.frame}
\usage{
create_scenario_from_df(user_input, config = config_ambre)
}
\arguments{
\item{user_input}{A data.frame / tibble with columns \code{CropName}, \code{Area},
\code{PopulationName}, \code{nb_population}, \code{PathName}, \code{nb_day_decay}, and the four
treatment columns \code{STEPtreatmentName}, \code{CollectiveTreatmentName},
\code{InitialProcessName}, \code{SupplementaryProcessName}. Each treatment cell may be
a single name, a character vector of names, \code{NA}, or empty.}

\item{config}{list config file, default value \code{config_ambre}.}
}
\value{
A scenario tibble, same structure as \code{\link[=create_scenario]{create_scenario()}}.
}
\description{
Like \code{\link[=create_scenario]{create_scenario()}} but takes an in-memory \code{data.frame} instead of an
Excel file, and each of the four treatment roles
(STEP / Collective / Initial / Supplementary) may hold a \emph{vector} of process
names (a list-column), so a train can contain more than one process per role.
}
\details{
Single-value role columns are also accepted (they behave like
\code{\link[=create_scenario]{create_scenario()}}); \code{NA} / empty roles are dropped from the train.
}
\examples{
df <- data.frame(
  CropName = "Tomato", Area = 35,
  PopulationName = "Maintenance staff", nb_population = 1,
  PathName = "Consumption of the final product tomato",
  nb_day_decay = NA_real_
)
df$STEPtreatmentName <- list(c("Q.1 - Activated Sludge", "Q.2 - Maturation Pond"))
df$CollectiveTreatmentName <- list(NA_character_)
df$InitialProcessName <- list("Q.6 - Chlorination")
df$SupplementaryProcessName <- list(NA_character_)
create_scenario_from_df(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_exposure.R
\name{simulate_exposure}
\alias{simulate_exposure}
\title{Simulate: exposure}
\usage{
simulate_exposure(config, debug = TRUE)
}
\arguments{
\item{config}{as retrieved by config_read()}

\item{debug}{print debug information (default: TRUE)}
}
\value{
list with parameters of user defined exposure scenario (number of events
and volumes per Event)
}
\description{
This function does the same thing as kwb.qmra::get_treatment_data(),
but it is bundled with \emph{ambre} to avoid a hard dependency on the
unmaintained package.
}
\examples{
\donttest{
scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
scenario_updated <- update_pathogen(scenario = scenario_test,
 pathoName = c("Campylobacter jejuni", "Rotavirus"))

simulate_exposure(config = scenario_updated$config[[1]])
}
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_logreduction_specific_barrier.R
\name{update_logreduction_specific_barrier}
\alias{update_logreduction_specific_barrier}
\title{Update log reduction with specific value}
\usage{
update_logreduction_specific_barrier(config, cropHeight, barrierID)
}
\arguments{
\item{config}{list configuration simulated}

\item{cropHeight}{numeric, height of th crop simulated}

\item{barrierID}{numeric, ID of the barrier simulated}
}
\value{
config_updated
}
\description{
Update log reduction value according to treatment, crop, and population simulated
}
\examples{
scenario_example <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)

scenario_example$config <- purrr::pmap(
  list(
    config = scenario_example$config,
    barrierID = scenario_example$SupplementaryProcessID,
    matrixID = scenario_example$MatrixID,
    cropHeight = scenario_example$CropHeight
  ),
  function(config, barrierID, matrixID, cropHeight) {

    if (barrierID \%in\% c(24, 26, 27, 28) &&
        matrixID \%in\% c(3, 5)) {

      update_logreduction_specific_barrier(
        config = config,
        cropHeight = cropHeight,
        barrierID = barrierID
      )

    } else {
      config
    }
  }
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/total_irrigation_need.R
\name{total_irrigation_need}
\alias{total_irrigation_need}
\title{Total irrigation need}
\usage{
total_irrigation_need(scenario)
}
\arguments{
\item{scenario}{data.frame scenario with irrigation_need_m3an column obtained with irrigation_need_calculation function}
}
\value{
total_irrigation_need
}
\description{
Calculate total irrigation need according to irrigation need of each culture of the simulated scenario
}
\examples{
scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
total_irrigation_need(irrigation_need)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/total_area_calculation.R
\name{total_area_calculation}
\alias{total_area_calculation}
\title{Total area}
\usage{
total_area_calculation(scenario)
}
\arguments{
\item{scenario}{data.frame scenario created with create_scenario function}
}
\value{
surface_total data.frame
}
\description{
Calculate the total area of each culture of the simulated scenario
}
\examples{
scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
total_area_calculation(scenario = scenario_apprentissage)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_economic_analysis.R
\name{run_economic_analysis}
\alias{run_economic_analysis}
\title{Run economic analysis}
\usage{
run_economic_analysis(
  scenario,
  price_per_m3,
  membership_fee,
  grant,
  initialSituation,
  allocation_key = NULL
)
}
\arguments{
\item{scenario}{data.frame scenario obtained with create_scenario function}

\item{price_per_m3}{numeric, water price in euro per cubic meter}

\item{membership_fee}{numeric annual membership fee in euro per hectare}

\item{grant}{numeric amount of the grant in percentage}

\item{initialSituation}{boolean, TRUE to simulate initial situation cost, FALSE to simulate supplementary process cost}

\item{allocation_key}{numeric allocation key, NULL by default}
}
\value{
all_plots
}
\description{
Calculate annual cost and total investment fees for simulated management risk scenario
}
\examples{
scenario <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
run_economic_analysis(scenario = scenario,
                      price_per_m3 = 0.1, 
                      membership_fee = 200, 
                      grant = 0.5,
                      initialSituation = TRUE)
run_economic_analysis(scenario = scenario,
                      price_per_m3 = 0.1, 
                      membership_fee = 200, 
                      grant = 0.5,
                      initialSituation = FALSE)

crop <- scenario$CropName
allocation_custom <- data.frame(CropName = crop,
                                allocation = c(0.5, 0.5))
run_economic_analysis(scenario = scenario,
                      price_per_m3 = 0.1, 
                      membership_fee = 200, 
                      grant = 0,
                      initialSituation = TRUE,
                      allocation_key = allocation_custom)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_monte_carlo.R
\name{update_monte_carlo}
\alias{update_monte_carlo}
\title{Update monte carlo value}
\usage{
update_monte_carlo(scenario, monte_carlo = 1000)
}
\arguments{
\item{scenario}{data.frame scenario obtained with create_scenario function}

\item{monte_carlo}{integer number of simulation iterations for Monte Carlo}
}
\value{
scenario_updated
}
\description{
Update number_of_repeating in config$exposure database
}
\examples{
scenario_example <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario_updated <- update_monte_carlo(scenario = scenario_example, 
                   monte_carlo = 2000)
scenario_updated$config[[1]]$exposure
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_cost_calculation.R
\name{process_cost_calculation}
\alias{process_cost_calculation}
\title{Barrier Cost Calculation}
\usage{
process_cost_calculation(scenario, membership_fee, charge, initialSituation)
}
\arguments{
\item{scenario}{data.frame scenario with irrigation_need_m3an column obtained from irrigation_need_calculation function}

\item{membership_fee}{numeric annual membership fee}

\item{charge}{numeric water charge}

\item{initialSituation}{boolean TRUE to simulate initial simulation cost, FALSE to simulate supplementary process cost}
}
\value{
scenario_updated
}
\description{
Calculate cost of supplementary barriers
}
\examples{
scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre"))
irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
process_cost_calculation(scenario = irrigation_need,
                         membership_fee = 200,
                         charge = 0.1,
                         initialSituation = TRUE)


process_cost_calculation(scenario = irrigation_need,
                         membership_fee = 200,
                         charge = 0.1,
                         initialSituation = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collective_treatment_cost_calculation.R
\name{collective_treatment_cost_calculation}
\alias{collective_treatment_cost_calculation}
\title{Collective treatment cost calculation}
\usage{
collective_treatment_cost_calculation(scenario, grant, allocation_key = NULL)
}
\arguments{
\item{scenario}{data.frame scenario with irrigation_need_m3an column obtained with irrigation_need_calculation}

\item{grant}{numeric amount of the grant in percentage}

\item{allocation_key}{data.frame allocation key, NULL by default}
}
\value{
cost
}
\description{
Calculate cost of the simulated collective treatment train
}
\examples{
scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
collective_treatment_cost_calculation(scenario = irrigation_need, 
                                      grant = 0.5)

crop <- scenario_apprentissage$CropName |> unique()
allocation_custom <- data.frame(CropName = crop,
                                allocation = c(0.5, 0.1, 0.1, 0.1, 0.1, 0.1))
collective_treatment_cost_calculation(scenario = irrigation_need, 
                                      grant = 0,
                                      allocation_key = allocation_custom)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/water_price.R
\name{water_price}
\alias{water_price}
\title{Water charge}
\usage{
water_price(scenario, price_per_m3, membership_fee)
}
\arguments{
\item{scenario}{data.frame, with irrigation_need_m3an obtained with irrigation_need_calculation function}

\item{price_per_m3}{numeric, water price in euro per cubic meter}

\item{membership_fee}{numeric annual membership fee in euro per hectare}
}
\value{
updated_scenario
}
\description{
Calculate  cost of water charge
}
\examples{


scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
water_price(scenario = irrigation_need,
            price_per_m3 = 0.01,
            membership_fee = 200)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_infection_prob.R
\name{get_infection_prob}
\alias{get_infection_prob}
\title{Get infection probability values}
\usage{
get_infection_prob(tbl_risk, dose_response, health)
}
\arguments{
\item{tbl_risk}{data.frame containing PathogenID and dose_perEvent columns}

\item{dose_response}{data.frame database of dose_response factors presents in config_ambre}

\item{health}{data.frame database of health factors presents in config_ambre}
}
\value{
result_vector
}
\description{
Simulate infection probability according to pathogen consider
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_qmra_learning_case.R
\name{run_qmra_learning_case}
\alias{run_qmra_learning_case}
\title{QMRA on learning case}
\usage{
run_qmra_learning_case()
}
\value{
all_plots list
}
\description{
Run a learning case to evaluate risk on scenario with 6 crops (Corn Seed, Tomato, Potato, Salad, Onion) and 6 populations (maintainer, harverster, irrigation, consumer, local resident, passerby). Run QMRA calculation with barrier measures
}
\examples{
\donttest{
run_qmra_learning_case()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doc_config_ambre.R
\docType{data}
\name{config_ambre}
\alias{config_ambre}
\title{config_ambre}
\format{
A list of 3 lists and 5 tibbles :
\describe{
\item{ exposure }{  Exposure condition : number of repeatings for Monte Carlo simulation, number of exposures per year, volume (L) per exposure event }
\item{ inflow }{  Pathogen concentration (number of pathogen per L) }
\item{ treatment }{  Database of log reductions for risk management measures }
\item{ processes }{  General values for the log reductions }
\item{ schemes }{  Risk management process (sequence of measures) assessed }
\item{ barrier_path }{  Specific values for log reductions based on the assessed route of exposure }
\item{ barrier_specific }{  Specific values for log reductions based on the assessed measure }
\item{ barrier_decay }{  Specific values for log reductions for the natural die off measure }
\item{ doseresponse }{  Parameters for dose response model (https://qmrawiki.org/framework/dose-response/highest-quality-models-and-parameters) }
\item{ path }{  Details on pathways of exposure }
\item{ description }{  Description of pathways of exposure }
\item{ frequency }{  Frequency values of pathways of exposure }
\item{ volume }{  Volume values of pathways of exposure }
\item{ crop }{  Description of crop }
\item{ health }{ Parameters for risk characterization  }
\item{ economic }{  Parameters for economic analysis }
\item{ water_need }{  Value of irrigation need according to crop type }
\item{ population }{  Description of population exposed }
\item{ cost }{  Database of opex and capex cost of risk management measures (https://theses.hal.science/tel-05009670) }
\item{ regulation_value }{  Database of required microorganism removal performance (in terms of concentration or removal rate) by regulatory class and pathogen group }
\item{ regulation_crop }{  Database of required water class acording to crop }
}
}
\usage{
config_ambre
}
\description{
A structured database containing the parameters and values needed to perform Quantitative Microbial Risk Assessment (QMRA) and cost calculations.
}
\examples{
names(config_ambre)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_barrier.R
\name{query_barrier}
\alias{query_barrier}
\title{Barrier Query}
\usage{
query_barrier(config = config_ambre, barrierName, na.rm = FALSE)
}
\arguments{
\item{config}{list configuration file default value is config_ambre dataset}

\item{barrierName}{character Name of the barrier you want to find the ID}

\item{na.rm}{boolean FALSE by default}
}
\value{
barrierID
}
\description{
retrieve Barrier ID according to its name in the barrier database
}
\examples{
query_barrier(barrierName = "Q.1 - Activated Sludge")
query_barrier(barrierName = c("Q.1 - Activated Sludge", "Q.2 - Maturation Pond"))
query_barrier(barrierName = "P.9 - Peeling, P.7 - Cooking")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_qmra_custom.R
\name{run_qmra_custom}
\alias{run_qmra_custom}
\title{Run QMRA custom}
\usage{
run_qmra_custom(
  scenario,
  volume,
  frequency,
  concentration,
  objective = 1e-06,
  regulationLog,
  regulationConcentration,
  initialSituation
)
}
\arguments{
\item{scenario}{data.frame obtain with create_scenario function}

\item{volume}{data.frame of min and max value, same size as scenario}

\item{frequency}{array of integer same size as scenario}

\item{concentration}{data.frame of pathogenName, min, max and distribution law to simulate}

\item{objective}{numeric reference dalys threshold to compare simulation result on dalys plot. default value is OMS threshold 1e-6 dalys}

\item{regulationLog}{data.frame regulation value of the log reduction to compare with simulation result}

\item{regulationConcentration}{data.frame regulation value of the log reduction to compare with simulation result}

\item{initialSituation}{boolean, TRUE to consider InitialTrainID for simulation, FALSE to consider SupplementaryTrainID for simulation}
}
\value{
all_plots list
}
\description{
Run the full QMRA workflow using custom data for volume and frequency of exposure (not the default databases)
\enumerate{
\item simulate inflow concentration, volume of exposure
\item calculate initial dose
\item simulate log reduction for treatment processes
\item calculate final dose
\item calculate infection probability, illness probability and dalys reduction
\item calculate annual total risk
}
}
\examples{
\donttest{
sc <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

regulation_reduction <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Country, RegulationID, Reduction))

concentration_custom <- data.frame(PathogenName = c("Rotavirus"),
                                   min = c(1),
                                   max = c(2),
                                   type = c("uniform"))
run_qmra_custom(scenario = sc,
                  concentration = concentration_custom,
                  volume = data.frame(min=c(0.5,1), max = c(1,2), type= c("triangle", "triangle")),
                  frequency = c(200L, 300L),
                  regulationLog = regulation_reduction,
                  regulationConcentration = regulation_concentration,
                  initialSituation = TRUE)

run_qmra_custom(scenario = sc,
                  concentration = concentration_custom,
                  volume = data.frame(min=c(0.5,1), max = c(1,2), type= c("triangle", "triangle")),
                  frequency = c(200L, 300L),
                  regulationLog = regulation_reduction,
                  regulationConcentration = regulation_concentration,
                  initialSituation = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_random_values.R
\name{generate_random_values}
\alias{generate_random_values}
\title{Create random distribution based on configuration file}
\usage{
generate_random_values(
  config,
  number_of_repeatings = 1,
  number_of_events,
  debug = TRUE
)
}
\arguments{
\item{config}{as retrieved by config_read()}

\item{number_of_repeatings}{how often should the random distribution with the
same parameters be generated (default: 1)}

\item{number_of_events}{number of events}

\item{debug}{print debug information (default: TRUE)}
}
\value{
list random distributions based on configuration file
}
\description{
This function does the same thing as kwb.utils::generate_random_values(),
but it is bundled with \emph{ambre} to avoid a hard dependency on the
unmaintained package.
}
\examples{
# Example usage of generate_random_values
# Create a sample config object
config <- list(
  type = "norm",
  value = 10,
  min = 5,
  max = 15,
  mean = 10,
  sd = 2,
  meanlog = log(10),
  sdlog = 0.5,
  mode = 10
)

# Call generate_random_values with the sample config
result <- generate_random_values(
  config = config,
  number_of_repeatings = 2,
  number_of_events = 10,
  debug = FALSE
)

# Print the results
cat("Simulated events:")
print(result$events)

cat("Parameters used for simulation:")
print(result$paras)
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_frequency.R
\name{query_frequency}
\alias{query_frequency}
\title{Query frequency}
\usage{
query_frequency(pathID)
}
\arguments{
\item{pathID}{integer ID of the path for which we want to retrieve the frequency values}
}
\value{
frequency
}
\description{
Retrieve min and max frequency values according to path defined in the scenario (database ambre_frequence)
}
\examples{
scenario_example <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)

query_frequency(pathID = scenario_example$PathID)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/irrigation_need_calculation.R
\name{irrigation_need_calculation}
\alias{irrigation_need_calculation}
\title{Irrigation need calculation}
\usage{
irrigation_need_calculation(scenario)
}
\arguments{
\item{scenario}{data.frame scenario obtained with create_scenario function}
}
\value{
scenario_updated
}
\description{
Calculate irrigation need according to irrigation need of the culture simulated and its surface
}
\examples{
scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
irrigation_need <- irrigation_need_calculation(scenario_apprentissage)
print(irrigation_need$irrigation_need_m3an)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_qmra_initial_situation.R
\name{run_qmra_initial_situation}
\alias{run_qmra_initial_situation}
\title{Run QMRA on initial situation}
\usage{
run_qmra_initial_situation(
  scenario,
  pathogen,
  regulationLog,
  regulationConcentration
)
}
\arguments{
\item{scenario}{data.frame scenario obtained with create_scenario function path of the scenario file}

\item{pathogen}{character Name of the pathogen to simulate}

\item{regulationLog}{data.frame regulation value of the log reduction to compare with simulation result}

\item{regulationConcentration}{data.frame regulation value of the log reduction to compare with simulation result}
}
\value{
all_plots list of ggplot
}
\description{
Run the full QMRA workflow
\enumerate{
\item simulate inflow concentration, volume of exposure
\item calculate initial dose
\item simulate log reduction for treatment processes
\item calculate final dose
\item calculate infection probability, illness probability and dalys reduction
\item calculate annual total risk
}
}
\examples{
\donttest{
library(dplyr)
scenario <- create_scenario(system.file("input_cas_apprentissage_complet.xlsx", package = "ambre"))
regulation_reduction <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Concentration, Country, RegulationID))
regulation_concentration <- config_ambre$regulation$regulation_value |> 
  dplyr::filter(Country == "France") |>
  dplyr::select(-c(Country, RegulationID, Reduction))
run_qmra_initial_situation(scenario = scenario, 
                   pathogen = c("Campylobacter jejuni", "Escherichia coli"),
                   regulationLog = regulation_reduction,
                   regulationConcentration = regulation_concentration)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dr-expo.R
\name{dr.expo}
\alias{dr.expo}
\title{Dose-response model: exponential}
\usage{
dr.expo(dose = sfsmisc::lseq(from = 1, to = 10^10, length = 1000), k = 0.572)
}
\arguments{
\item{dose}{vector of dose data (default:
\code{sfsmisc::lseq(from = 0.1, to = 10^10, length = 1000)})}

\item{k}{k-value (default: 5.72E-01)}
}
\value{
tibble
}
\description{
Dose-response model: exponential
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_concentration.R
\name{update_concentration}
\alias{update_concentration}
\title{Update concentration with desired value}
\usage{
update_concentration(scenario, concentration)
}
\arguments{
\item{scenario}{data.frame}

\item{concentration}{data.frame of pathogenName, min, max and distribution law to simulate}
}
\value{
scenario_updated
}
\description{
Update the min, max and distribution law of the concentration in the simulated configuration according to a value defined by the user
}
\examples{
scenario_test <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)

pathogenName <- c("Norovirus", "Rotavirus")
concentration_custom <- data.frame(PathogenName = pathogenName,
                                   min = c(1, 1),
                                   max = c(2,2),
                                   type = c("uniform", "uniform"))

scenario_updated_patho <- update_pathogen(scenario = scenario_test, pathoName = pathogenName)
result <- update_concentration(scenario = scenario_updated_patho ,
                     concentration = concentration_custom)

dplyr::filter(result$config[[1]]$inflow, PathogenName == "Norovirus")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_frequency_with_desired_value.R
\name{update_frequency_with_desired_value}
\alias{update_frequency_with_desired_value}
\title{Updated frequency with desired value}
\usage{
update_frequency_with_desired_value(scenario, frequency)
}
\arguments{
\item{scenario}{data.frame}

\item{frequency}{numeric, list of max frequency to simulate}
}
\value{
scenario_updated
}
\description{
Update the value of the frequency in the simulated configuration according to a value defined by the user
}
\examples{
scenario_example <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario_example_updated <- update_frequency_with_desired_value(scenario = scenario_example, 
                                                                frequency = c(2L,3L))
print(scenario_example_updated$config[[1]]$exposure)
print(scenario_example_updated$config[[2]]$exposure)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_crop.R
\name{query_crop}
\alias{query_crop}
\title{Crop Query}
\usage{
query_crop(config = config_ambre, cropName)
}
\arguments{
\item{config}{list configuration file default value is config_ambre dataset}

\item{cropName}{character Name of crop for which you want to find the identifier}
}
\value{
cropID integer
}
\description{
Query to return crop ID from crop Name in crop database
}
\examples{
query_crop(cropName = "Corn seed")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_exp_path.R
\name{query_exp_path}
\alias{query_exp_path}
\title{Path Query}
\usage{
query_exp_path(config = config_ambre, pathName)
}
\arguments{
\item{config}{list configuration file default value is config_ambre dataset}

\item{pathName}{character Name of the path for which you want to find the ID}
}
\value{
pathID integer ID of the population define
}
\description{
Query to return path ID from path Name using the database
}
\examples{
query_exp_path(pathName = "Consumption of the final product potatoes")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_volume_with_desired_value.R
\name{update_volume_with_desired_value}
\alias{update_volume_with_desired_value}
\title{Update volume with desired value}
\usage{
update_volume_with_desired_value(scenario, volume)
}
\arguments{
\item{scenario}{data.frame}

\item{volume}{numeric, named list of min and max value to simulate}
}
\value{
scenario_updated
}
\description{
Update the min and max value of the volume in the simulated configuration according to a value defined by the user
}
\examples{
scenario_example <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
volumes <- data.frame("min" = c(1.1, 1.2), "max" = c(1.3,1.4), "type" = c("triangle", "triangle"))
scenario_example_updated <- update_volume_with_desired_value(scenario = scenario_example,
                                 volume = volumes)

print(scenario_example_updated$config[[1]]$exposure)
print(scenario_example_updated$config[[2]]$exposure)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_exposure_value_or_stop.R
\name{get_exposure_value_or_stop}
\alias{get_exposure_value_or_stop}
\title{Get exposure value or stop}
\usage{
get_exposure_value_or_stop(config, name)
}
\arguments{
\item{config}{list config}

\item{name}{character name of the column}
}
\value{
value
}
\description{
This function does the same thing as kwb.utils::exposure_value_or_stop(),
but it is bundled with \emph{ambre} to avoid a hard dependency on the
unmaintained package.
}
\details{
Description
}
\examples{
get_exposure_value_or_stop(config_ambre, "number_of_exposures")
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribution_repeater.R
\name{distribution_repeater}
\alias{distribution_repeater}
\title{Helper function: distribution repeater}
\usage{
distribution_repeater(
  number_of_repeatings = 10,
  number_of_events = 365,
  func,
  ...
)
}
\arguments{
\item{number_of_repeatings}{how often should the random distribution with the
same parameters be generated (default: 1)}

\item{number_of_events}{number of events (a single positive integer, >= 1)}

\item{func}{distribution function to be repeated (e.g. runif, rlnorm, rnorm)}

\item{...}{further parameters passed to func}
}
\value{
data.frame with columns repeatID, eventID and values
}
\description{
This function does the same thing as kwb.utils::distribution_repeater(),
but it is bundled with \emph{ambre} to avoid a hard dependency on the
unmaintained package.
}
\examples{
 distribution_repeater(
   number_of_repeatings = 2,
   number_of_events = 10,
   func = runif,
   min = 1,
   max = 10
)
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_pathogen.R
\name{query_pathogen}
\alias{query_pathogen}
\title{Pathogen Query}
\usage{
query_pathogen(config = config_ambre, pathogenName)
}
\arguments{
\item{config}{list configuration file default value is config_ambre dataset}

\item{pathogenName}{character Name of the pathogen for which you want to find the ID}
}
\value{
pathogenID integer ID of the pathogen define
}
\description{
Query to return pathogen ID from pathogen Name using the database
}
\examples{
query_pathogen(pathogenName = "Campylobacter jejuni")

query_pathogen(pathogenName = c("Campylobacter jejuni", "Rotavirus"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_volume.R
\name{update_volume}
\alias{update_volume}
\title{Volume update}
\usage{
update_volume(scenario)
}
\arguments{
\item{scenario}{dataframe scenario obtained with function create_scenario}
}
\value{
scenario_updated list configuration updated with the min and max volumes to be simulated
}
\description{
Update the configuration with min and max volumes to be simulated according to path ID
}
\examples{
scenario_example <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario_updated <- update_volume(scenario_example)
scenario_updated$config[[1]]$exposure
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_scheme_events_wide.R
\name{get_scheme_events_wide}
\alias{get_scheme_events_wide}
\title{Calculate Wide-Format Treatment Scheme Events}
\usage{
get_scheme_events_wide(config, treatment_events_wide)
}
\arguments{
\item{config}{A configuration object containing treatment scheme information.}

\item{treatment_events_wide}{A data frame in wide format containing treatment events
with columns for each treatment ID and log-reduction values.}
}
\value{
A data frame with columns for event ID, pathogen group, and one column
for each treatment scheme containing the sum of log-reduction values for that scheme.
}
\description{
This function and returns the results in a wide-format data frame.
}
\details{
This function does the same thing as kwb.qmra::get_scheme_events_wide(),
but it is bundled with \emph{ambre} to avoid a hard dependency on the
unmaintained package.
}
\examples{
# Example usage of get_scheme_events_wide
scenario_test <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario_updated <- update_pathogen(scenario = scenario_test,
 pathoName = c("Campylobacter jejuni", "Rotavirus"))
config <- scenario_updated$config[[1]]

# Simulate treatment data
treatment_data <- get_treatment_data(config$treatment$processes, n_repeatings = 5, n_events = 3)

treatment_events_wide <- treatment_data$events |> tidyr::spread(
      key = TreatmentID,
      value = logreduction
    )

# Apply get_scheme_events_wide
scheme_events_wide <- get_scheme_events_wide(config, treatment_events_wide)

# Print results
print(scheme_events_wide)
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ambre-package.R
\docType{package}
\name{ambre-package}
\alias{ambre}
\alias{ambre-package}
\title{ambre: Multi-Barrier Approach for Water Reuse Risk Assessment}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Provides a Quantitative Microbial Risk Assessment (QMRA) framework for the reuse of treated wastewater in agricultural irrigation. Following a multi-barrier approach, the package simulates pathogen inflow and removal along a treatment train, estimates human exposure, and computes health risk indicators such as infection probability, illness probability, and Disability-Adjusted Life Years (DALYs). It also supports an economic analysis of the treatment scenarios considered. For more details see \doi{10.36904/20240595}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://forge.inrae.fr/reversaal/reut/ambre-package}
  \item Report bugs at \url{https://forge.inrae.fr/reversaal/reut/ambre-package/-/issues}
}

}
\author{
\strong{Maintainer}: Flora Girard \email{flora.girard@inrae.fr} (\href{https://orcid.org/0000-0003-0537-4530}{ORCID})

Other contributors:
\itemize{
  \item Nicolas Forquet \email{nicolas.forquet@inrae.fr} (\href{https://orcid.org/0000-0003-1154-5498}{ORCID}) [contributor]
  \item Arthur Bréant \email{arthur@thinkr.fr} [contributor]
  \item Colin Fay \email{colin@thinkr.fr} (\href{https://orcid.org/0000-0001-7343-1846}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inflow_concentration_custom.R
\name{inflow_concentration_custom}
\alias{inflow_concentration_custom}
\title{Inflow concentration custom}
\usage{
inflow_concentration_custom(scenario, concentration, monteCarlo = 1000)
}
\arguments{
\item{scenario}{list scenario created with create_scenario}

\item{concentration}{data.frame min, max and distribution law of pathogen concentration}

\item{monteCarlo}{integer number of simulation iterations for Monte Carlo}
}
\value{
scenario_with_concentration
}
\description{
To calculate inflow concentration for specified pathogen
}
\examples{
sc <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
sc  <- update_volume_with_desired_value(sc, 
                                        volume = data.frame(min = c(0.5,1), 
                                                            max = c(1,2), 
                                                            type = c("triangle", "triangle")))
  
sc <- update_frequency_with_desired_value(scenario = sc, frequency = c(200L, 300L))

concentration_custom <- data.frame(PathogenName = c("Campylobacter jejuni", "Norovirus"),
                                   min = c(1, 10),
                                   max = c(2,20),
                                   type = c("uniform", "uniform"))
  
inflow_concentration_custom(scenario = sc,
                            concentration = concentration_custom)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_treatment_data.R
\name{get_treatment_data}
\alias{get_treatment_data}
\title{Simulate treatment data for pathogen processes}
\usage{
get_treatment_data(
  processes,
  n_repeatings,
  n_events,
  debug = TRUE,
  include_paras = TRUE
)
}
\arguments{
\item{processes}{A data frame containing information about pathogen processes,
including treatment names and pathogen groups.}

\item{n_repeatings}{The number of times each process should be repeated.}

\item{n_events}{The number of events to simulate for each process.}

\item{debug}{Logical indicating whether to print debug information.}

\item{include_paras}{Logical indicating whether to include parameter data.}
}
\value{
If \code{include_paras} is FALSE, returns a data frame of simulated events.
If \code{include_paras} is TRUE, returns a list containing both the events data frame
and a data frame of parameters.
}
\description{
This function simulates treatment data for a set of pathogen processes, including
random values for each process and optional parameters. It can be used to generate
synthetic data for analysis or testing.
}
\details{
This function does the same thing as kwb.qmra::get_treatment_data(),
but it is bundled with \emph{ambre} to avoid a hard dependency on the
unmaintained package.
}
\examples{
# Example usage (assuming 'processes' is defined)
scenario_test <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
scenario_updated <- update_pathogen(scenario = scenario_test,
 pathoName = c("Campylobacter jejuni", "Rotavirus"))
config = scenario_updated$config[[1]]
processes_simulated <- config$treatment$processes

result <- get_treatment_data(processes_simulated, n_repeatings = 10, n_events = 5)
print(result)
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_irrigation_need.R
\name{query_irrigation_need}
\alias{query_irrigation_need}
\title{Query irrigation need}
\usage{
query_irrigation_need(scenario)
}
\arguments{
\item{scenario}{data.frame scenario obtained with create_scenario function}
}
\value{
scenario_updated
}
\description{
Retrieve irrigation need according to crop ID
}
\examples{
scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)

irrigation_need <- query_irrigation_need(scenario_apprentissage)
print(irrigation_need$IrrigationNeed)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_pathogen.R
\name{update_pathogen}
\alias{update_pathogen}
\title{Pathogen update}
\usage{
update_pathogen(scenario, pathoName)
}
\arguments{
\item{scenario}{list obtain from create_scenario()}

\item{pathoName}{character name of the pathogens simulated}
}
\value{
scenario_updated list scenario updated with the pathogens to be simulated
}
\description{
Update the configuration with list of pathogen to be simulated according to pathogen ID
}
\examples{
scenario_test <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)

scenario_updated <- update_pathogen(scenario = scenario_test,
 pathoName = c("Campylobacter jejuni", "Rotavirus"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_logreduction_decay.R
\name{update_logreduction_decay}
\alias{update_logreduction_decay}
\title{Update log reduction for decay barrier}
\usage{
update_logreduction_decay(config, cropHeight, popID, barrierID, nb_day_decay)
}
\arguments{
\item{config}{list configuration simulated}

\item{cropHeight}{numeric, height of th crop simulated}

\item{popID}{numeric, ID of the population simulated}

\item{barrierID}{numeric, ID of the barrier simulated}

\item{nb_day_decay}{numeric, number of day of natural decay for the simulated crop}
}
\value{
config_updated
}
\description{
Update the log reduction value for the decay barrier according to crop and population simulated
}
\examples{
scenario_example <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)

scenario_example$config <- purrr::pmap(
  list(
    config = scenario_example$config,
    barrierID = scenario_example$SupplementaryProcessID,
    cropHeight = scenario_example$CropHeight,
    popID = scenario_example$PopulationID,
    decay = scenario_example$nb_day_decay
    
  ),
  function(config, barrierID, cropHeight, popID, decay) {

    if (barrierID \%in\% c(12)) {

      update_logreduction_decay(
        config = config,
        cropHeight = cropHeight,
        barrierID = barrierID,
        popID = popID,
        nb_day_decay = decay
      )

    } else {
      config
    }
  }
)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_matrix.R
\name{query_matrix}
\alias{query_matrix}
\title{Query Matrix}
\usage{
query_matrix(config = config_ambre, pathName)
}
\arguments{
\item{config}{list configuration file default value is config_ambre dataset}

\item{pathName}{character Name of the path for which you want to find the ID}
}
\value{
matrixID integer
}
\description{
Retrieve Matrix ID according to path
}
\examples{
query_matrix(pathName = "Consumption of the final product potatoes")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/illness_probability_calculation.R
\name{illness_probability_calculation}
\alias{illness_probability_calculation}
\title{Illness probability calculation}
\usage{
illness_probability_calculation(scenario)
}
\arguments{
\item{scenario}{data.frame with risk column}
}
\value{
scenario_updated
}
\description{
Calculate illness probability according to infection probability and pathogen simulated
}
\examples{
library(dplyr)
library(purrr)
scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

scenario_conc <- inflow_concentration(scenario = scenario_test,
 pathogenName = c("Campylobacter jejuni"))

scenario_volume <-
  scenario_conc |>                     
  mutate(
    volume = map(
      .x = config,                                         
      .f = ~ simulate_exposure(config = .x)              
    )
  )

scenario_dose_ini <- initial_dose_calculation(scenario_volume)

scenario_scheme <- update_treatment_scheme(scenario_dose_ini)

scenario_with_logreduc_and_co <- scenario_scheme |> 
  mutate( log_reduction = map(config, simulate_treatment))

scenario_final_dose_and_co_test <- final_dose_calculation(scenario_with_logreduc_and_co)

scenario_inf_proba_and_co <- infection_probability_calculation(scenario_final_dose_and_co_test)

illness_probability_calculation(scenario_inf_proba_and_co)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_scenario.R
\name{create_scenario}
\alias{create_scenario}
\title{Create Exposure Scenario}
\usage{
create_scenario(filepath, config = config_ambre)
}
\arguments{
\item{filepath}{Path to the Excel file containing input data}

\item{config}{list config file default value config_ambre}
}
\value{
A list containing the scenario data and a placeholder for future data
}
\description{
This function creates exposure scenarios during the two game turns.
}
\examples{
scenario <- create_scenario(system.file("input_cas_apprentissage_complet.xlsx", package = "ambre"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_pop.R
\name{query_pop}
\alias{query_pop}
\title{Population Query}
\usage{
query_pop(config = config_ambre, popName)
}
\arguments{
\item{config}{list configuration file default value is config_ambre dataset}

\item{popName}{character Name of the population for which you want to find the ID}
}
\value{
popID integer ID of the population define
}
\description{
Query to return population ID from population Name using the database
}
\examples{
query_pop(popName = "Irrigation staff")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_treatment_scheme.R
\name{update_treatment_scheme}
\alias{update_treatment_scheme}
\title{Update treatment scheme}
\usage{
update_treatment_scheme(scenario, initial_situation = TRUE)
}
\arguments{
\item{scenario}{created with function create_scenario}

\item{initial_situation}{boolean to retrieve simulate treatment train if TRUE or from Barrier train if FALSE}
}
\value{
updated_scenario
}
\description{
Update treatment scheme list in config data frame according to scenario define
}
\examples{
scenario_test <- create_scenario(
  system.file("input_cas_apprentissage_complet.xlsx", package = "ambre")
)
scenario_ini <- update_treatment_scheme(scenario_test, initial_situation = TRUE)
scenario_supp <- update_treatment_scheme(scenario_test, initial_situation = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_volume.R
\name{query_volume}
\alias{query_volume}
\title{Query volume}
\usage{
query_volume(pathID)
}
\arguments{
\item{pathID}{integer ID of the path for which we want to retrieve the volume min and max values}
}
\value{
volume
}
\description{
Retrieve min and max volume values according to path defined in the scenario (database ambre_volume)
}
\examples{
query_volume(pathID = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_tbl_risk.R
\name{get_tbl_risk}
\alias{get_tbl_risk}
\title{Get table risk}
\usage{
get_tbl_risk(scenario)
}
\arguments{
\item{scenario}{data.frame with columns inflow concentration, final dose and volume}
}
\value{
tbl_risk
}
\description{
Get table risk to calculate infection probability
}
\examples{

library(dplyr)
library(purrr)
scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

scenario_conc <- inflow_concentration(scenario = scenario_test,
 pathogenName = c("Campylobacter jejuni", "Escherichia coli"))

scenario_volume <-
  scenario_conc |>                     
  mutate(
    volume = map(
      .x = config,                                         
      .f = ~ simulate_exposure(config = .x)              
    )
  )

scenario_dose_ini <- initial_dose_calculation(scenario_volume)

scenario_scheme <- update_treatment_scheme(scenario_dose_ini)

scenario_with_logreduc_and_co <- scenario_scheme |> 
  mutate( log_reduction = map(config, simulate_treatment))

scenario_final_dose_and_co_test <- final_dose_calculation(scenario_with_logreduc_and_co)
tbl_risk_test <- get_tbl_risk(scenario_final_dose_and_co_test)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_volumes.R
\name{plot_volumes}
\alias{plot_volumes}
\title{Plot volume}
\usage{
plot_volumes(scenario)
}
\arguments{
\item{scenario}{data.frame with volume column obtained with simulate_exposure function}
}
\value{
plot
}
\description{
Boxplot of the volume of exposition for each population exposed
}
\examples{
library(dplyr)
library(purrr)
scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))

scenario_conc <- inflow_concentration(scenario = scenario_test,
 pathogenName = c("Campylobacter jejuni"))

scenario_volume <-
  scenario_conc |>                     
  mutate(
    volume = map(
      .x = config,                                         
      .f = ~ simulate_exposure(config = .x)              
    )
  )

plot_volumes(scenario_volume)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_inflow.R
\name{simulate_inflow}
\alias{simulate_inflow}
\title{Simulate: inflow}
\usage{
simulate_inflow(config, debug = TRUE, lean = FALSE)
}
\arguments{
\item{config}{list contained in the scenario object}

\item{debug}{print debug information (default: TRUE)}

\item{lean}{if \code{TRUE}, only the "events" are returned (in a reduced
version, i.e. without column \code{PathogenName} and with all ID columns
being of class \code{integer}), otherwise a list with the events in element
\code{events}" and the corresponding parameters in element \code{paras}.
The default is \code{FALSE}, i.e. events and parameters are returned in a
list.}
}
\value{
list with parameters of user defined random distribution and
corresponding values
}
\description{
This function does the same thing as kwb.utils::simulate_inflow(),
but it is bundled with \emph{ambre} to avoid a hard dependency on the
unmaintained package.
}
\details{
Simulate pathogen concentrations
}
\examples{
# Example usage of simulate_inflow
scenario_test <- create_scenario(system.file("input_1culture_2pop.xlsx", package = "ambre"))
scenario_updated <- update_pathogen(scenario = scenario_test,
 pathoName = c("Campylobacter jejuni", "Rotavirus"))

# Call simulate_inflow with the sample config
result_1 <- simulate_inflow(config = scenario_updated$config[[1]], debug = FALSE)

# Print the results
cat("Parameters used for simulation:")
print(result_1$paras)

# Example with lean = TRUE
lean_result_1 <- simulate_inflow(config = scenario_updated$config[[1]], debug = FALSE, lean = TRUE)
cat("Lean result (only events):")
print(lean_result_1)
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dr-betapoisson.R
\name{dr.betapoisson}
\alias{dr.betapoisson}
\title{Dose-response model: beta-poisson}
\usage{
dr.betapoisson(
  dose = sfsmisc::lseq(from = 1, to = 10^10, length = 1000),
  alpha = 0.328,
  N50 = 5430
)
}
\arguments{
\item{dose}{vector of dose data (default:
\code{sfsmisc::lseq(from = 0.1, to = 10^10, length = 1000)})}

\item{alpha}{alpha (default: 3.28E-01)}

\item{N50}{N50 (default: 5.43E+03)}
}
\value{
tibble
}
\description{
Dose-response model: beta-poisson
}
\references{
Sonnenberg H (2024). kwb.utils: General Utility Functions Developed at KWB. R package version 0.15.0
URL: \url{https://github.com/kwb-r/kwb.utils}
(MIT Licence)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annual_cost.R
\name{annual_cost}
\alias{annual_cost}
\title{Annual cost}
\usage{
annual_cost(
  scenario,
  price_per_m3,
  membership_fee,
  grant,
  initialSituation,
  allocation_key = NULL
)
}
\arguments{
\item{scenario}{data.frame scenario obtained with create_scenario function}

\item{price_per_m3}{numeric, water price in euro per cubic meter}

\item{membership_fee}{numeric annual membership fee in euro per hectare}

\item{grant}{numeric amount of the grant in percentage}

\item{initialSituation}{boolean TRUE to simulate initial situation cost, FALSE to simulation supplementary process cost}

\item{allocation_key}{numeric allocation key, NULL by default}
}
\value{
result list
}
\description{
Calculate annual cost of the scenario simulated
}
\examples{
scenario_apprentissage <- create_scenario(
  system.file("input_1culture_2pop.xlsx", package = "ambre")
)
annual_cost(scenario = scenario_apprentissage,
            price_per_m3 = 0.1,
            membership_fee = 200,
            grant = 0.5,
            initialSituation = TRUE)

annual_cost(scenario = scenario_apprentissage,
            price_per_m3 = 0.1,
            membership_fee = 200,
            grant = 0.5,
            initialSituation = FALSE)

crop <- scenario_apprentissage$CropName
allocation_custom <- data.frame(CropName = crop,
                                allocation = c(0.5, 0.5))
annual_cost(scenario = scenario_apprentissage,
            price_per_m3 = 0.1,
            membership_fee = 200,
            grant = 0,
            initialSituation = TRUE,
            allocation_key = allocation_custom)

annual_cost(scenario = scenario_apprentissage,
            price_per_m3 = 0.1,
            membership_fee = 200,
            grant = 0,
            initialSituation = FALSE,
            allocation_key = allocation_custom)

}
